--  ________________________________________________________________________  --
-- |                                                                        | --
-- |          _______ _______                               __              | --
-- |         |   _   |   |   |.-----.-----.---.-.----.----.|  |--.          | --
-- |         |       |       ||__ --|  -__|  _  |   _|  __||     |          | --
-- |         |___|___|___|___||_____|_____|___._|__| |____||__|__|          | --
-- |                                                                        | --
-- |________________________________________________________________________| --
--  ________________________________________________________________________  --
-- |                                                                        | --
-- |                           AHsearch by kunda                            | --
-- |________________________________________________________________________| --
-- |                                                                        | --
-- |                                                                        | --
-- | AHsearch is a World of Warcraft AddOn that saves Auction House search  | --
-- | queries.                                                               | --
-- |                                                                        | --
-- | AHsearch adds a 20x20 pixel 'Arrow Down' Button ('AHsearch Button') to | --
-- | the right side of the Auction House searchfield.                       | --
-- |                                                                        | --
-- | Features:                                                              | --
-- | # Right-click 'AHsearch Button' for configuration.                     | --
-- |   (or: Right-click on a category or a search query for configuration.) | --
-- |   - 'Search Queries'-Tab: Alt-click a bag item or an auction item to   | --
-- |     auto fill out some fields.                                         | --
-- | # Left-click 'AHsearch Button' for your saved Search Queries.          | --
-- | # Cycle Search: (Next/Previous/Random) DISABLED by DEFAULT             | --
-- |   - Key Bindings [ESC -> Key bindings -> AHsearch]                     | --
-- |________________________________________________________________________| --
-- |                                                                        | --
-- |                                                                        | --
-- | Credits:                                                               | --
-- | Idea based on AHFavorites from Narwick.                                | --
-- | Cycle search idea based on AHsearchNext from Evan.                     | --
-- |                                                                        | --
-- |________________________________________________________________________| --
--                                                                            --

-- ---------------------------------------------------------------------------------------------------------------------
AHsearch = CreateFrame("Frame")
AHsearch_Data = {}             -- SavedVariable Data table
AHsearch_Options = {}          -- SavedVariable Options table
local L = AHsearch_Locales     -- localization table

local GVAR = {}     -- UI Widgets
local TEMPLATE = {} -- Templates

local _G = _G
local table_insert = table.insert
local table_sort = table.sort
local math_floor = math.floor
local GetTime = GetTime
local unpack = unpack
local GetAuctionItemClasses = GetAuctionItemClasses
local GetAuctionItemSubClasses = GetAuctionItemSubClasses
local GetAuctionInvTypes = GetAuctionInvTypes

local maxBoxInputLength = 50        -- maximum characters for box entries
local cycleChk                      -- nil/false = NO key binding allowed | true = key binding allowed 
local cycleCat                      -- cycle search category
local cycleSQ                       -- cycle search search query

local FrameTime = GetTime()         -- for VisibilityCheck()

local minCat = 10                   -- minimum category size
local maxCat = 50                   -- maximum category size (>50 = scrollframe)
local GUI_CatFrame_data = {}        -- category data
local GUI_CatFrame_Count            -- category data count

local minQuery = 10                 -- minimum search query size
local maxQuery = 50                 -- maximum search query size (>50 = scrollframe)
local GUI_SQFrame_data = {}         -- search query data
local GUI_SQFrame_Count = {}        -- search query data count

local GUI_Tab = 1                   -- Configuration Tabs
local GUI_curCat                    -- current category
local GUI_curSQ                     -- current search query

local FilterTable = {}              -- AH Filter table
local Filter1Table = {}             -- Lvl1 AH Filter
local Filter2Table = {}             -- Lvl2 AH Filter
local Filter3Table = {}             -- Lvl3 AH Filter
local curFilter1                    -- selected Lvl1 Filter
local curFilter2                    -- selected Lvl2 Filter
local curFilter3                    -- selected Lvl3 Filter

local configMode = false            -- config mode check value
local confirmIsShown = false        -- confirm frame check value

local default_curCat        = 25    -- default option: height rows
local default_curCatWidth   = 175   -- default option: width px
local default_curQuery      = 25    -- default option: height rows
local default_curQueryWidth = 175   -- default option: width px
local default_cycleNP       = false -- default option: cycle search Next/Previous
local default_cycleRND      = false -- default option: cycle search Random
local default_scaleMain     = 100   -- default option: Main Frame (cat/sq) Scale
local default_hideMain      = false -- default option: hide/show cat/sq frames in config mode

-- color
local normalBorderColor = {0.4, 0.4, 0.4, 1}
local configBorderColor = {0.8, 0.2, 0.2, 1}

-- size START
local sizeGeneral = {}
sizeGeneral.Border     = 1
sizeGeneral.Space      = 2
sizeGeneral.Offset     = 5
sizeGeneral.BarHeight  = 14
sizeGeneral.SliderSize = 14--17

-- - category
local function CatSizeUpdate(newsize)
	sizeGeneral.CatButtonSFWidth     = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
	sizeGeneral.CatButtonNoSFWidth   = newsize-sizeGeneral.Offset-sizeGeneral.Offset
	sizeGeneral.CatTextSFWidth       = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border - sizeGeneral.BarHeight-sizeGeneral.Space
	sizeGeneral.CatTextNoSFWidth     = newsize-sizeGeneral.Offset-sizeGeneral.Offset - sizeGeneral.BarHeight-sizeGeneral.Space
end
CatSizeUpdate(default_curCatWidth)

-- - search queries
local function QuerySizeUpdate(newsize)
	sizeGeneral.QueryButtonSFWidth   = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
	sizeGeneral.QueryButtonNoSFWidth = newsize-sizeGeneral.Offset-sizeGeneral.Offset
	sizeGeneral.QueryTextSFWidth     = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
	sizeGeneral.QueryTextNoSFWidth   = newsize-sizeGeneral.Offset-sizeGeneral.Offset
end
QuerySizeUpdate(default_curQueryWidth)

-- - config
sizeGeneral.ConfigTextWidth   = 200
sizeGeneral.ConfigSliderWidth = 150
sizeGeneral.ConfigCatWidth    = 200

sizeGeneral.ConfigCatButtonSFWidth        = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigCatButtonNoSFWidth      = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset
sizeGeneral.ConfigCatTextSFWidth          = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border - sizeGeneral.BarHeight-sizeGeneral.Space
sizeGeneral.ConfigCatTextNoSFWidth        = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset - sizeGeneral.BarHeight-sizeGeneral.Space

sizeGeneral.ConfigQueryButtonSFWidth      = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigQueryButtonNoSFWidth    = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset
sizeGeneral.ConfigQueryTextSFWidth        = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigQueryTextNoSFWidth      = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownOrderWidth      = 50
sizeGeneral.ConfigPullDownOrderSFWidth    = sizeGeneral.ConfigPullDownOrderWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownOrderNoSFWidth  = sizeGeneral.ConfigPullDownOrderWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownColorWidth      = 110
sizeGeneral.ConfigPullDownColorSFWidth    = sizeGeneral.ConfigPullDownColorWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownColorNoSFWidth  = sizeGeneral.ConfigPullDownColorWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownRarityWidth     = 150
sizeGeneral.ConfigPullDownRaritySFWidth   = sizeGeneral.ConfigPullDownRarityWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownRarityNoSFWidth = sizeGeneral.ConfigPullDownRarityWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownIconWidth       = 50
sizeGeneral.ConfigPullDownIconSFWidth     = sizeGeneral.ConfigPullDownIconWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownIconNoSFWidth   = sizeGeneral.ConfigPullDownIconWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownFilterWidth     = 120
sizeGeneral.ConfigPullDownFilterSFWidth   = sizeGeneral.ConfigPullDownFilterWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownFilterNoSFWidth = sizeGeneral.ConfigPullDownFilterWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownSortWidth       = 120
sizeGeneral.ConfigPullDownSortSFWidth     = sizeGeneral.ConfigPullDownSortWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownSortNoSFWidth   = sizeGeneral.ConfigPullDownSortWidth-sizeGeneral.Offset-sizeGeneral.Offset
-- size END

-- magical 180 degree texture cut center rotation
local function rt(H,E,M,P)
	return E,P,E,M,H,P,H,M
end

local Textures = {
	AHsBmain        = {coords     =    { 37/128,  56/128, 19/64, 38/64},
	                   coordsPS   =    { 57/128,  76/128, 19/64, 38/64},
	                   coordsHL   =    { 77/128,  96/128, 19/64, 38/64}},
	AHsBprev        = {coords     =    { 37/128,  59/128, 39/64, 52/64},
	                   coordsPS   =    { 60/128,  82/128, 39/64, 52/64},
	                   coordsHL   =    { 83/128, 105/128, 39/64, 52/64}},
	AHsBnext        = {coords     = {rt( 37/128,  59/128, 39/64, 52/64)},
	                   coordsPS   = {rt( 60/128,  82/128, 39/64, 52/64)},
	                   coordsHL   = {rt( 83/128, 105/128, 39/64, 52/64)}},
	AHsBrandom      = {coords     =    { 97/128, 110/128, 19/64, 32/64},
	                   coordsHL   =    {111/128, 124/128, 19/64, 32/64}},

	HighLightDef    = {coords     =    { 19/128,  36/128, 19/64, 36/64}},
	ScrollFrameUp   = {coords     =    { 73/128,  90/128,  1/64, 18/64},
	                   coordsHL   =    { 91/128, 108/128,  1/64, 18/64},
	                   coordsDIS  =    {109/128, 126/128,  1/64, 18/64}},
	ScrollFrameDown = {coords     = {rt( 73/128,  90/128,  1/64, 18/64)},  -- 180 degree rota
	                   coordsHL   = {rt( 91/128, 108/128,  1/64, 18/64)},  -- 180 degree rota
	                   coordsDIS  = {rt(109/128, 126/128,  1/64, 18/64)}}, -- 180 degree rota
	ScrollFrameKnob = {coords     =    {  1/128,  18/128, 19/64, 36/64}},
	SearchBoxIcons  = {coordsLoff =    {  1/128,   6/128,  1/64, 18/64},
	                   coordsMoff =    {  9/128,  10/128,  1/64, 18/64},
	                   coordsRoff =    { 13/128,  18/128,  1/64, 18/64},
	                   coordsLon  =    { 19/128,  24/128,  1/64, 18/64},
	                   coordsMon  =    { 27/128,  28/128,  1/64, 18/64},
	                   coordsRon  =    { 31/128,  36/128,  1/64, 18/64},
	                   width      =    17,
	                   height     =    17},
	SliderKnob      = {coords     =    { 19/128,  36/128, 37/64, 54/64},
	                   coordsHL   =    { 91/128, 108/128,  1/64, 18/64}},
	SliderBG        = {coordsL    =    {  1/128,   6/128, 42/64, 48/64},
	                   coordsM    =    {  9/128,  10/128, 42/64, 48/64},
	                   coordsR    =    { 13/128,  18/128, 42/64, 48/64}},
	Close           = {coords     =    { 55/128,  72/128,  1/64, 18/64}},
	Expand          = {coords     =    { 37/128,  54/128,  1/64, 18/64}},
	Collapse        = {coords     = {rt( 37/128,  54/128,  1/64, 18/64)}}, -- 180 degree rota
	SelectArrow     = {coordsR    =    {119/128, 125/128, 39/64, 51/64},
	                   coordsL    = {rt(118/128, 127/128, 37/64, 52/64)},  -- 180 degree rota
	                   width      =    6,
	                   height     =    12},
	SoryOrderIcon   = {coordsDesc =    {109/128, 116/128, 41/64, 49/64},
	                   coordsAsc  = {rt(109/128, 116/128, 41/64, 49/64)}}, -- 180 degree rota
}

local itemRarity = {
  [1] = {color = "|cffffff9a", name = L["All"]},                                    -- all       (AH: -1)
  [2] = {color = "|cff9d9d9d", name = ITEM_QUALITY0_DESC.." ... "..L["or better"]}, -- poor      (AH: 0)
  [3] = {color = "|cffffffff", name = ITEM_QUALITY1_DESC.." ... "..L["or better"]}, -- common    (AH: 1)
  [4] = {color = "|cff1eff00", name = ITEM_QUALITY2_DESC.." ... "..L["or better"]}, -- uncommon  (AH: 2)
  [5] = {color = "|cff0070dd", name = ITEM_QUALITY3_DESC.." ... "..L["or better"]}, -- rare      (AH: 3)
  [6] = {color = "|cffa335ee", name = ITEM_QUALITY4_DESC.." ... "..L["or better"]}, -- epic      (AH: 4)
  [7] = {color = "|cffff8000", name = ITEM_QUALITY5_DESC.." ... "..L["or better"]}, -- legendary (AH: 5)
--[8] = {color = "|cffe6cc80", name = ITEM_QUALITY6_DESC.." ... "..L["or better"]}, -- artifact  (not in AH index)
}

local SortByTable = {
	[1] = "quality",  -- -> L["quality"]
	[2] = "level",    -- -> L["level"]
	[3] = "duration", -- -> L["duration"]
	[4] = "seller",   -- -> L["seller"]
	[5] = "bid"       -- -> L["bid"]
}

local categoryColors = {
	[1] = "|cffffffff",
	[2] = "|cffbbbbbb",
	[3] = "|cff777777",
	[4] = "|cfff32f00",
	[5] = "|cffff8c00",
	[6] = "|cffffd700",
	[7] = "|cffdeb887",
	[8] = "|cffa0522d",
	[9] = "|cffe9967a",
	[10] = "|cff4169e1",
	[11] = "|cff008cb4",
	[12] = "|cff7edef4",
	[13] = "|cffa020f0",
	[14] = "|cffee82ee",
	[15] = "|cff95004a",
	[16] = "|cff00ff00"
}

local categoryIntColors = {}
for i = 1, #categoryColors do
	categoryIntColors[ categoryColors[i] ] = i
end

local categoryIcons = { -- fudge factor ftw
	[1]  = {texture = "",                                                           coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[2]  = {texture = "Interface\\GroupFrame\\UI-Group-LeaderIcon",                 coords = {0.00000001, 0.93750001, 0.00000001, 0.99999999}}, --{0, 15/16, 0, 1}},
	[3]  = {texture = "Interface\\Minimap\\Tracking\\Auctioneer",                   coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[4]  = {texture = "Interface\\Minimap\\PartyRaidBlips",                         coords = {0.76953125, 0.85937501, 0.53125001, 0.71093751}}, --{197/256, 220/256, 68/128, 91/128}},
	[5]  = {texture = "Interface\\FriendsFrame\\InformationIcon",                   coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[6]  = {texture = "Interface\\FriendsFrame\\ClearBroadcastIcon",                coords = {0.00000001, 0.93750001, 0.06250001, 0.99999999}}, --{0, 15/16, 1/16, 1}},
	[7]  = {texture = "Interface\\FriendsFrame\\StatusIcon-DnD",                    coords = {0.00000001, 0.93750001, 0.00000001, 0.93750001}}, --{0, 15/16, 0, 15/16}},
	[8]  = {texture = "Interface\\GossipFrame\\AvailableQuestIcon",                 coords = {0.06250001, 0.99999999, 0.00000001, 0.99999999}}, --{1/16, 1, 0, 1}},
	[9]  = {texture = "Interface\\HelpFrame\\HotIssueIcon",                         coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[10] = {texture = "Interface\\LFGFrame\\LFGRole",                               coords = {0.75000001, 0.99999999, 0.00000001, 0.99999999}}, --{48/64, 64/64, 0, 1}},
	[11] = {texture = "Interface\\PaperDollInfoFrame\\UI-EquipmentManager-Toggle",  coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[12] = {texture = "Interface\\GossipFrame\\TrainerGossipIcon",                  coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[13] = {texture = "Interface\\Minimap\\Tracking\\Reagents",                     coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[14] = {texture = "Interface\\Minimap\\Tracking\\Food",                         coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[15] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.00000001, 0.12500001, 0.00000001, 0.50000001}}, --{  0/256,  32/256,  0/64, 32/64}},
	[16] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.12500001, 0.25000001, 0.00000001, 0.50000001}}, --{ 32/256,  64/256,  0/64, 32/64}},
	[17] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.25000001, 0.37500001, 0.00000001, 0.50000001}}, --{ 64/256,  96/256,  0/64, 32/64}},
	[18] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.37500001, 0.50000001, 0.00000001, 0.50000001}}, --{ 96/256, 128/256,  0/64, 32/64}},
	[19] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.50000001, 0.62500001, 0.00000001, 0.50000001}}, --{128/256, 160/256,  0/64, 32/64}},
	[20] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.62500001, 0.75000001, 0.00000001, 0.50000001}}, --{160/256, 192/256,  0/64, 32/64}},
	[21] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.75000001, 0.87500001, 0.00000001, 0.50000001}}, --{192/256, 224/256,  0/64, 32/64}},
	[22] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.87500001, 0.99999999, 0.00000001, 0.50000001}}, --{224/256, 256/256,  0/64, 32/64}},
	[23] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.00000001, 0.12500001, 0.50000001, 0.99999999}}, --{  0/256,  32/256, 32/64, 64/64}},
	[24] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.12500001, 0.25000001, 0.50000001, 0.99999999}}, --{ 32/256,  64/256, 32/64, 64/64}},
}
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function DBupdate()
	if AHsearch_Data then
		for k in pairs(AHsearch_Data) do

			if AHsearch_Data[k].icon then
				local newIcon = 1
				for i = 1, #categoryIcons do
					if AHsearch_Data[k].icon == categoryIcons[i].texture then
						newIcon = i
					end
				end
				AHsearch_Data[k].icon = newIcon
				--print("DBupdate - icon update:", k, newIcon) -- TEST
			end

			if AHsearch_Data[k].queries then
				local x = 0
				for k2 in pairs(AHsearch_Data[k].queries) do
					x = x + 1
				end
				if x == 0 then
					AHsearch_Data[k].queries = nil
					--print("DBupdate - delete empty queries:", k) -- TEST
				end
			end

			if AHsearch_Data[k].queries then
				for k2, v2 in pairs(AHsearch_Data[k].queries) do
					v2.SortBy = "quality"
					--v2.SortOrder = 1 -- nil is blizz AH default
					--print("DBupdate - added default SortBy (SortOrder=nil):", v2.SortBy) -- TEST
					if v2.minLevel then
						if v2.minLevel == "" then
							v2.minLevel = nil
						else
							local minNum = tonumber(v2.minLevel)
							if type(minNum) == "number" then
								if minNum < 1 or minNum > 99 then
									v2.minLevel = nil
								else
									v2.minLevel = minNum
								end
							else
								v2.minLevel = nil
							end
						end
						--print("DBupdate - minLevel to number:", type(v2.minLevel), v2.minLevel) -- TEST
					end
					if v2.maxLevel then
						if v2.maxLevel == "" then
							v2.maxLevel = nil
						else
							local maxNum = tonumber(v2.maxLevel)
							if type(maxNum) == "number" then
								if maxNum < 1 or maxNum > 99 then
									v2.maxLevel = nil
								else
									v2.maxLevel = maxNum
								end
							else
								v2.maxLevel = nil
							end
						end
						--print("DBupdate - maxLevel to number:", type(v2.maxLevel), v2.maxLevel) -- TEST
					end
					if v2.isUsable then
						if v2.isUsable == 0 then
							v2.isUsable = nil
						end
						--print("DBupdate - isUsable:", type(v2.isUsable), v2.isUsable) -- TEST
					end
					if v2.showOnPlayer then
						if v2.showOnPlayer == 0 then
							v2.showOnPlayer = nil
						end
						--print("DBupdate - showOnPlayer:", type(v2.showOnPlayer), v2.showOnPlayer) -- TEST
					end
				end
			end

		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function Desaturation(texture, desaturation)
	local shaderSupported = texture:SetDesaturated(desaturation)
	if not shaderSupported then
		if desaturation then
			texture:SetVertexColor(0.5, 0.5, 0.5)
		else
			texture:SetVertexColor(1.0, 1.0, 1.0)
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Template BorderTRBL START ----------------------------------------
TEMPLATE.BorderTRBL = function(frame, color) -- TRBL = Top-Right-Bottom-Left
	frame.FrameBackground = frame:CreateTexture(nil, "BORDER")
	frame.FrameBackground:SetPoint("TOPLEFT", 1, -1)
	frame.FrameBackground:SetPoint("BOTTOMRIGHT", -1, 1)
	frame.FrameBackground:SetTexture(0, 0, 0, 1)
	frame.FrameBorder = frame:CreateTexture(nil, "BACKGROUND")
	frame.FrameBorder:SetPoint("TOPLEFT", 0, 0)
	frame.FrameBorder:SetPoint("BOTTOMRIGHT", 0, 0)
	if color then
		if color == 1 then
			frame.FrameBorder:SetTexture(0.8, 0.2, 0.2, 1)
		end
	else
		frame.FrameBorder:SetTexture(0.4, 0.4, 0.4, 1)
	end
end
-- Template BorderTRBL END ----------------------------------------

-- Template TextButton START ----------------------------------------
TEMPLATE.DisableTextButton = function(button)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableTextButton = function(button, action)
	local buttoncolor
	if action == 1 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		bordercolor = {0.24, 0.46, 0.21, 1}
	elseif action == 4 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	else
		bordercolor = {1, 1, 1, 1}
	end
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])
	button:Enable()
end

TEMPLATE.TextButton = function(button, text, action)
	local buttoncolor
	local bordercolor
	if action == 1 then
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable") 
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		button:SetNormalFontObject("GameFontNormalSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0, 0.5, 1}
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		button:SetNormalFontObject("GameFontNormalSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0.2, 0, 1}
		bordercolor = {0.24, 0.46, 0.21, 1}
	elseif action == 4 then
		button:SetNormalFontObject("GameFontNormalSmall")
		button:SetDisabledFontObject("GameFontDisableSmall") 
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	else
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable")
		buttoncolor = {0, 0, 0, 1}
		bordercolor = {1, 1, 1, 1}
	end

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetPoint("TOPLEFT", 0, 0)
	button.Border:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", 2, -2)
	button.Normal:SetPoint("BOTTOMRIGHT", -2, 2)
	button.Normal:SetTexture(buttoncolor[1], buttoncolor[2], buttoncolor[3], buttoncolor[4])
	button:SetNormalTexture(button.Normal)

	button.Disabled = button:CreateTexture(nil, "OVERLAY")
	button.Disabled:SetPoint("TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.Disabled)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)

	button:SetPushedTextOffset(1, -1)	
	button:SetText(text)
end
-- Template TextButton END ----------------------------------------

-- Template IconButton START ----------------------------------------
--TEMPLATE.DisableIconButton = function(button)
--	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
--	button:Disable()
--end

--TEMPLATE.EnableIconButton = function(button)
--	button.Border:SetTexture(0.8, 0.2, 0.2, 1)
--	button:Enable()
--end

TEMPLATE.IconButton = function(button, cut)
	button.Back = button:CreateTexture(nil, "BORDER")
	button.Back:SetPoint("TOPLEFT", 1, -1)
	button.Back:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Back:SetTexture(0, 0, 0, 1)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetPoint("TOPLEFT", 0, 0)
	button.Border:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Border:SetTexture(0.8, 0.2, 0.2, 1)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", 3, -3)
	button.Normal:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Normal:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	button.Normal:SetTexCoord(unpack(Textures.Close.coords))
	button:SetNormalTexture(button.Normal)

	button.Push = button:CreateTexture(nil, "ARTWORK")
	button.Push:SetPoint("TOPLEFT", 4, -4)
	button.Push:SetPoint("BOTTOMRIGHT", -4, 4)
	button.Push:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	button.Push:SetTexCoord(unpack(Textures.Close.coords))
	button:SetPushedTexture(button.Push)

	button.Disabled = button:CreateTexture(nil, "ARTWORK")
	button.Disabled:SetPoint("TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	button.Disabled:SetTexCoord(unpack(Textures.Close.coords))
	button:SetDisabledTexture(button.Disabled)
	Desaturation(button.Disabled, true)
end
-- Template IconButton END ----------------------------------------

-- Template EditBox START ----------------------------------------
TEMPLATE.DisableEditBox = function(editbox)
	editbox.DisableTexture:Show()
	editbox:EnableMouse(false)
	editbox:ClearFocus()
	editbox.Left:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsLoff))
	editbox.Right:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsRoff))
	editbox.Middle:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsMoff))
end
TEMPLATE.EnableEditBox = function(editbox)
	editbox.DisableTexture:Hide()
	editbox:EnableMouse(true)
	editbox.Left:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsLon))
	editbox.Right:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsRon))
	editbox.Middle:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsMon))
end

TEMPLATE.EditBox = function(editbox)
	editbox:SetHeight(Textures.SearchBoxIcons.height)
	editbox:SetAutoFocus(false)
	editbox:SetFontObject(GameFontNormalSmall)
	editbox:SetTextColor(1, 1, 1, 1) 
	editbox:SetTextInsets(2, 0, 0, 0) 

	editbox.Left = editbox:CreateTexture(nil, "BACKGROUND")
	editbox.Left:SetWidth(5)
	editbox.Left:SetHeight(Textures.SearchBoxIcons.height)
	editbox.Left:SetPoint("LEFT", 0, 0)
	editbox.Left:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	editbox.Left:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsLoff))

	editbox.Right = editbox:CreateTexture(nil, "BACKGROUND")
	editbox.Right:SetWidth(5)
	editbox.Right:SetHeight(Textures.SearchBoxIcons.height)
	editbox.Right:SetPoint("RIGHT", 0, 0)
	editbox.Right:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	editbox.Right:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsRoff))

	editbox.Middle = editbox:CreateTexture(nil, "BACKGROUND")
	editbox.Middle:SetWidth(1)
	editbox.Middle:SetHeight(Textures.SearchBoxIcons.height)
	editbox.Middle:SetPoint("LEFT", editbox.Left, "RIGHT", 0, 0)
	editbox.Middle:SetPoint("RIGHT", editbox.Right, "LEFT", 0, 0)
	editbox.Middle:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	editbox.Middle:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsMoff))

	editbox.DisableTexture = editbox:CreateTexture(nil, "ARTWORK")
	editbox.DisableTexture:SetPoint("TOPLEFT", 4, -4)
	editbox.DisableTexture:SetPoint("BOTTOMRIGHT", -4, 4)
	editbox.DisableTexture:SetTexture(0.6, 0.6, 0.6, 0.2)
	editbox.DisableTexture:Hide()

	editbox:SetScript("OnShow",          function(self) self:SetAutoFocus(false) end)
	editbox:SetScript("OnEnterPressed",  function(self) self:ClearFocus() end)
	editbox:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
end
-- Template EditBox END ----------------------------------------

-- Template TabButton START ----------------------------------------
TEMPLATE.SetTabButton = function(button, show)
	if show then
		button.TextureBottom:SetTexture(0, 0, 0, 1)
		button.TextureBorder:SetTexture(unpack(configBorderColor))
		button.show = true
	else
		button.TextureBottom:SetTexture(unpack(configBorderColor))
		button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 0.4)
		button.show = false
	end
end
TEMPLATE.DisableTabButton = function(button)
	if button.Txt then
		button.Txt:SetTextColor(0.5, 0.5, 0.5, 1)
	elseif button.TabTexture then
		Desaturation(button.TabTexture, true)
	end
	button:Disable()
end

TEMPLATE.EnableTabButton = function(button)
	if button.Txt then
		button.Txt:SetTextColor(1, 1, 1, 1)
	elseif button.TabTexture then
		Desaturation(button.TabTexture, false)
	end
	button:Enable()
end

TEMPLATE.TabButton = function(button, text, detail, buttonwidth)
	button.Texture = button:CreateTexture(nil, "BORDER")
	button.Texture:SetPoint("TOPLEFT", 1, -1)
	button.Texture:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Texture:SetTexture(0, 0, 0, 1)

	button.TextureBorder = button:CreateTexture(nil, "BACKGROUND")
	button.TextureBorder:SetPoint("TOPLEFT", 0, 0)
	button.TextureBorder:SetPoint("BOTTOMRIGHT", -1, 1)
	button.TextureBorder:SetPoint("TOPRIGHT", 0, 0)
	button.TextureBorder:SetPoint("BOTTOMLEFT" ,1, 1)
	button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 1)

	button.TextureBottom = button:CreateTexture(nil, "ARTWORK")
	button.TextureBottom:SetPoint("TOPLEFT", button, "BOTTOMLEFT" ,1, 2)
	button.TextureBottom:SetPoint("BOTTOMLEFT", 1, 1)
	button.TextureBottom:SetPoint("TOPRIGHT", button, "BOTTOMRIGHT" ,-1, 2)
	button.TextureBottom:SetPoint("BOTTOMRIGHT" ,-1, 1)
	button.TextureBottom:SetTexture(0.4, 0.4, 0.4, 1)

	button.TextureHighlight = button:CreateTexture(nil, "ARTWORK")
	button.TextureHighlight:SetPoint("TOPLEFT", 3, -3)
	button.TextureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.TextureHighlight:SetTexture(1, 1, 1, 0.2)
	button:SetHighlightTexture(button.TextureHighlight)

	button.Txt = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	button.Txt:SetText(text)
	button.Txt:SetWidth(buttonwidth)
	button.Txt:SetHeight(12)
	button.Txt:SetPoint("CENTER", button, "CENTER", 0, 0)
	button.Txt:SetJustifyH("CENTER")

	if detail == 1 then
		button.Txt:SetTextColor(1, 1, 1, 1)
	elseif detail == 2 then
		button.Txt:SetTextColor(0, 0.75, 0, 1)
	elseif detail == 3 then
		button.Txt:SetTextColor(1, 0, 0, 1)
	end
	
	button:SetScript("OnEnter", function() if not button.show then button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 0.8) end end)
	button:SetScript("OnLeave", function() if not button.show then button.TextureBorder:SetTexture(0.4, 0.4, 0.4, 0.4) end end)
end
-- Template TabButton END ----------------------------------------

-- Template ScrollFrame START ----------------------------------------
TEMPLATE.ButtonTextureScrollFrame = function(button, pos)
	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetWidth(sizeGeneral.SliderSize)
	textureNormal:SetHeight(sizeGeneral.SliderSize)
	textureNormal:SetPoint("LEFT", 0, 0)
	textureNormal:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	textureNormal:SetTexCoord(unpack(Textures[pos].coords))
	button:SetNormalTexture(textureNormal)

	local texturePush = button:CreateTexture(nil, "ARTWORK")
	texturePush:SetWidth(sizeGeneral.SliderSize)
	texturePush:SetHeight(sizeGeneral.SliderSize)
	texturePush:SetPoint("TOPLEFT", 1, -1)
	texturePush:SetPoint("BOTTOMRIGHT", -1, 1)
	texturePush:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	texturePush:SetTexCoord(unpack(Textures[pos].coords))
	button:SetPushedTexture(texturePush)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetWidth(sizeGeneral.SliderSize)
	textureHighlight:SetHeight(sizeGeneral.SliderSize)
	textureHighlight:SetPoint("LEFT", 0, 0)
	textureHighlight:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	textureHighlight:SetTexCoord(unpack(Textures[pos].coordsHL))
	button:SetHighlightTexture(textureHighlight)

	local textureDisable = button:CreateTexture(nil, "ARTWORK")
	textureDisable:SetWidth(sizeGeneral.SliderSize)
	textureDisable:SetHeight(sizeGeneral.SliderSize)
	textureDisable:SetPoint("LEFT", 0, 0)
	textureDisable:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	textureDisable:SetTexCoord(unpack(Textures[pos].coordsDIS))
	button:SetDisabledTexture(textureDisable)
end

TEMPLATE.ScrollFrame = function(frame, child)
	frame.offset = 0
	frame:EnableMouseWheel(true)

	frame.ScrollBar = CreateFrame("Slider", nil, frame)
	frame.ScrollBar:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBar:SetHeight(0)
	frame.ScrollBar:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -(sizeGeneral.Space+sizeGeneral.Border), -(sizeGeneral.Offset+sizeGeneral.SliderSize))
	frame.ScrollBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset+sizeGeneral.SliderSize)
	frame.ScrollBar:SetMinMaxValues(0, 0)
	frame.ScrollBar:SetValue(0)
	frame.ScrollBar:SetOrientation("VERTICAL")
	frame.ScrollBar:EnableMouseWheel(true)
	frame.ScrollBar:SetScript("OnValueChanged", function(self, value)
		frame:SetVerticalScroll(value)
	end)
	frame.ScrollBar:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		else
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		end
	end)

	frame.ScrollBG = frame.ScrollBar:CreateTexture(nil, "BACKGROUND")
	frame.ScrollBG:SetAllPoints(frame.ScrollBar)
	frame.ScrollBG:SetTexture(1, 1, 1, 0.05)

	frame.ScrollBarScrollUpButton = CreateFrame("Button", nil, frame.ScrollBar)
	frame.ScrollBarScrollUpButton:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBarScrollUpButton:SetHeight(sizeGeneral.SliderSize)
	frame.ScrollBarScrollUpButton:SetPoint("BOTTOM", frame.ScrollBar, "TOP", 0, 0)
	frame.ScrollBarScrollUpButton:SetScript("OnClick", function()
		frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TEMPLATE.ButtonTextureScrollFrame(frame.ScrollBarScrollUpButton, "ScrollFrameUp")

	frame.ScrollBarScrollDownButton = CreateFrame("Button", nil, frame.ScrollBar)
	frame.ScrollBarScrollDownButton:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBarScrollDownButton:SetHeight(sizeGeneral.SliderSize)
	frame.ScrollBarScrollDownButton:SetPoint("TOP", frame.ScrollBar, "BOTTOM", 0, 0)
	frame.ScrollBarScrollDownButton:SetScript("OnClick", function()
		frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TEMPLATE.ButtonTextureScrollFrame(frame.ScrollBarScrollDownButton, "ScrollFrameDown")

	frame.ScrollBarThumbTexture = frame.ScrollBar:CreateTexture(nil, "BORDER")
	frame.ScrollBarThumbTexture:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBarThumbTexture:SetHeight(sizeGeneral.SliderSize)
	frame.ScrollBarThumbTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	frame.ScrollBarThumbTexture:SetTexCoord(unpack(Textures.ScrollFrameKnob.coords))
	frame.ScrollBar:SetThumbTexture(frame.ScrollBarThumbTexture)

	frame:SetScript("OnScrollRangeChanged", function(self, xrange, yrange)
		if not yrange then
			yrange = self:GetVerticalScrollRange()
		end
		local value = frame.ScrollBar:GetValue()
		if value > yrange then
			value = yrange
		end
		frame.ScrollBar:SetMinMaxValues(0, yrange)
		frame.ScrollBar:SetValue(value)
		if floor(yrange) == 0 then
			frame.ScrollBarScrollDownButton:Disable()
			frame.ScrollBarScrollUpButton:Disable()
			frame.ScrollBarScrollDownButton:Show()
			frame.ScrollBarScrollUpButton:Show()
		else
			frame.ScrollBarScrollDownButton:Show()
			frame.ScrollBarScrollUpButton:Show()
			frame.ScrollBar:Show()
			-- The 0.005 is to account for precision errors
			if yrange - value > 0.005 then
				frame.ScrollBarScrollDownButton:Enable()
			else
				frame.ScrollBarScrollDownButton:Disable()
			end
		end
	end)
	frame:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		else
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		end
	end)
	frame:SetScript("OnVerticalScroll", function(self, off)
		frame.ScrollBar:SetValue(off)
	end)

	if child then
		frame.ScrollChildFrame = CreateFrame("Frame",  nil, frame)
		frame.ScrollChildFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		frame.ScrollChildFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
		--childFrame:SetHeight(0)
		--childFrame:SetWidth(0)
		frame:SetScrollChild(frame.ScrollChildFrame)
	end
end
-- Template ScrollFrame END ----------------------------------------

-- Template PullDownMenu START ----------------------------------------
TEMPLATE.DisablePullDownMenu = function(button)
	button.PullDownMenu:Hide()
	button.PullDownButtonBorder:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnablePullDownMenu = function(button)
	button.PullDownButtonBorder:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.PullDownMenu = function(button, buttonText, pulldownWidth, contentTable, contentTexture, func)
	local maxNum = #contentTable
	if maxNum < 10 then
		maxNum = 10
	end

	button.PullDownButtonBG = button:CreateTexture(nil, "BORDER")
	button.PullDownButtonBG:SetPoint("TOPLEFT", 1, -1)
	button.PullDownButtonBG:SetPoint("BOTTOMRIGHT", -1, 1)
	button.PullDownButtonBG:SetTexture(0, 0, 0, 1)

	button.PullDownButtonBorder = button:CreateTexture(nil, "BACKGROUND")
	button.PullDownButtonBorder:SetPoint("TOPLEFT", 0, 0)
	button.PullDownButtonBorder:SetPoint("BOTTOMRIGHT", 0, 0)
	button.PullDownButtonBorder:SetTexture(0.4, 0.4, 0.4, 1)

	button.PullDownButtonNormal = button:CreateTexture(nil, "ARTWORK")
	button.PullDownButtonNormal:SetPoint("TOPLEFT", 2, -2)
	button.PullDownButtonNormal:SetPoint("BOTTOMRIGHT", -2, 2)
	button.PullDownButtonNormal:SetTexture(0, 0, 0, 1)

	button.PullDownButtonExpand = button:CreateTexture(nil, "ARTWORK")
	button.PullDownButtonExpand:SetHeight(sizeGeneral.BarHeight)
	button.PullDownButtonExpand:SetWidth(sizeGeneral.BarHeight)
	button.PullDownButtonExpand:SetPoint("RIGHT", button, "RIGHT", -(sizeGeneral.Space+sizeGeneral.Border), 0)
	button.PullDownButtonExpand:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
	button:SetNormalTexture(button.PullDownButtonExpand)

	button.PullDownButtonDisabled = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonDisabled:SetPoint("TOPLEFT", 3, -3)
	button.PullDownButtonDisabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.PullDownButtonDisabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.PullDownButtonDisabled)

	button.PullDownButtonHighlight = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonHighlight:SetPoint("TOPLEFT", 3, -3)
	button.PullDownButtonHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.PullDownButtonHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.PullDownButtonHighlight)

	if contentTexture then
		if contentTexture == 1 then
			button.PullDownMenuTexture = button:CreateTexture(nil, "ARTWORK")
			button.PullDownMenuTexture:SetWidth(sizeGeneral.BarHeight)
			button.PullDownMenuTexture:SetHeight(sizeGeneral.BarHeight)
			button.PullDownMenuTexture:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
		elseif contentTexture == 2 then
			button.PullDownMenuTexture = button:CreateTexture(nil, "ARTWORK")
			button.PullDownMenuTexture:SetWidth(sizeGeneral.BarHeight)
			button.PullDownMenuTexture:SetHeight(sizeGeneral.BarHeight)
			button.PullDownMenuTexture:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
			button.PullDownMenuText = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
			button.PullDownMenuText:SetTextColor(1, 1, 1, 1)
			button.PullDownMenuText:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.BarHeight - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
			button.PullDownMenuText:SetHeight(sizeGeneral.BarHeight)
			button.PullDownMenuText:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space+sizeGeneral.BarHeight+sizeGeneral.Space, 0)
			button.PullDownMenuText:SetJustifyH("LEFT")
			button.PullDownMenuText:SetText(buttonText)
		end
	else
		button.PullDownMenuText = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.PullDownMenuText:SetTextColor(1, 1, 1, 1)
		button.PullDownMenuText:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
		button.PullDownMenuText:SetHeight(sizeGeneral.BarHeight)
		button.PullDownMenuText:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
		button.PullDownMenuText:SetJustifyH("LEFT")
		button.PullDownMenuText:SetText(buttonText)
	end

	button.PullDownMenu = CreateFrame("Frame", nil, button)
	TEMPLATE.BorderTRBL(button.PullDownMenu, 1)
	button.PullDownMenu:EnableMouse(true)
	button.PullDownMenu:SetToplevel(true)
	button.PullDownMenu:SetWidth(pulldownWidth)
	button.PullDownMenu:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	button.PullDownMenu:SetPoint("TOPLEFT", button, "BOTTOMLEFT", 0, 1)
	button.PullDownMenu:Hide()

	button.ScrollFrame = CreateFrame("ScrollFrame", nil, button.PullDownMenu)
	TEMPLATE.ScrollFrame(button.ScrollFrame)
	button.ScrollFrame:SetPoint("TOPLEFT", 0, 0)
	button.ScrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	button.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(button, value) end)

	local function OnLeave()
		if not button:IsMouseOver() and not button.PullDownMenu:IsMouseOver() then
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		end
	end

	button.ScrollFrame.Button = {}
	for i = 1, maxNum do
		button.ScrollFrame.Button[i] = CreateFrame("Button", nil, button.PullDownMenu)
		local pdbutton = button.ScrollFrame.Button[i]
		pdbutton:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
		pdbutton:SetHeight(sizeGeneral.BarHeight)
		pdbutton:SetFrameLevel( button.ScrollFrame:GetFrameLevel() + 5 )
		pdbutton:Hide()
		if i == 1 then
			pdbutton:SetPoint("TOPLEFT", button.ScrollFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			pdbutton:SetPoint("TOPLEFT", button.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		if contentTexture then
			if contentTexture == 1 then
				pdbutton.Texture = pdbutton:CreateTexture(nil, "ARTWORK")
				pdbutton.Texture:SetWidth(sizeGeneral.BarHeight)
				pdbutton.Texture:SetHeight(sizeGeneral.BarHeight)
				pdbutton.Texture:SetPoint("LEFT", sizeGeneral.Space, 0)
			elseif contentTexture == 2 then
				pdbutton.Texture = pdbutton:CreateTexture(nil, "ARTWORK")
				pdbutton.Texture:SetWidth(sizeGeneral.BarHeight)
				pdbutton.Texture:SetHeight(sizeGeneral.BarHeight)
				pdbutton.Texture:SetPoint("LEFT", sizeGeneral.Space, 0)
				pdbutton.Text = pdbutton:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
				pdbutton.Text:SetTextColor(1, 1, 1, 1)
				pdbutton.Text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
				pdbutton.Text:SetHeight(sizeGeneral.BarHeight)
				pdbutton.Text:SetPoint("LEFT", sizeGeneral.Space+sizeGeneral.BarHeight+sizeGeneral.Space, 0)
				pdbutton.Text:SetJustifyH("LEFT")
			end
		else
			pdbutton.Text = pdbutton:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
			pdbutton.Text:SetTextColor(1, 1, 1, 1)
			pdbutton.Text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
			pdbutton.Text:SetHeight(sizeGeneral.BarHeight)
			pdbutton.Text:SetPoint("LEFT", sizeGeneral.Space, 0)
			pdbutton.Text:SetJustifyH("LEFT")
		end

		pdbutton.Selected = pdbutton:CreateTexture(nil, "ARTWORK")
		pdbutton.Selected:SetPoint("TOPLEFT", pdbutton, "TOPLEFT", 0, 0)
		pdbutton.Selected:SetPoint("BOTTOMRIGHT", pdbutton, "BOTTOMRIGHT", 0, 0)
		pdbutton.Selected:SetTexture(1, 1, 1, 0.1)
		pdbutton.Selected:Hide()

		pdbutton:SetScript("OnLeave", OnLeave)
		pdbutton:SetScript("OnClick", function()
			if contentTexture then
				if contentTexture == 1 then
					button.value = pdbutton.value
					button.PullDownMenuTexture:SetTexture( categoryIcons[button.value].texture )
					button.PullDownMenuTexture:SetTexCoord(unpack(categoryIcons[button.value].coords))
					button.PullDownMenu:Hide()
					button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
				elseif contentTexture == 2 then
					button.value = pdbutton.value
					button.PullDownMenuTexture:SetTexture( categoryIcons[ GUI_CatFrame_data[button.value].Texture ].texture )
					button.PullDownMenuTexture:SetTexCoord(unpack(categoryIcons[ GUI_CatFrame_data[button.value].Texture ].coords))
					button.PullDownMenuText:SetText( GUI_CatFrame_data[ button.value ].CatColor..GUI_CatFrame_data[ button.value ].Cat.."|r" )
					button.PullDownMenu:Hide()
					button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
				end
			else
				button.value = pdbutton.value
				button.PullDownMenuText:SetText( pdbutton.Text:GetText() )
				button.PullDownMenu:Hide()
				button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
			end
			if func then
				func(button.value)
			end
		end)

		local highlight = pdbutton:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", pdbutton, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", pdbutton, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		pdbutton:SetHighlightTexture(highlight)
	end
	AHsearch:FillList(button)

	button.PullDownMenu:SetScript("OnLeave", OnLeave)
	button.PullDownMenu:SetScript("OnHide", function(self) self:Hide() end) -- for esc close
	button.ScrollFrame:SetScript("OnLeave", OnLeave)
	button.ScrollFrame.ScrollBar:SetScript("OnLeave", OnLeave)
	button.ScrollFrame.ScrollBarScrollUpButton:SetScript("OnLeave", OnLeave)
	button.ScrollFrame.ScrollBarScrollDownButton:SetScript("OnLeave", OnLeave)

	button:SetScript("OnLeave", OnLeave)
	button:SetScript("OnClick", function()
		if button.PullDownMenu:IsShown() then
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		else
			button.PullDownMenu:Show()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Collapse.coords))
		end
	end)
end
-- Template PullDownMenu END ----------------------------------------

-- Template CheckButton START ----------------------------------------
TEMPLATE.DisableCheckButton = function(button)
	if button.Text then
		button.Text:SetTextColor(0.5, 0.5, 0.5)
	elseif button.Icon then
		Desaturation(button.Icon, true)
	end
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableCheckButton = function(button)
	if button.Text then
		button.Text:SetTextColor(1, 1, 1)
	elseif button.Icon then
		Desaturation(button.Icon, false)
	end
	button.Border:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.CheckButton = function(button, size, space, text, icon)
	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetWidth( size )
	button.Border:SetHeight( size )
	button.Border:SetPoint("LEFT", 0, 0)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Normal:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Normal:SetTexture(0, 0, 0, 1)
	button:SetNormalTexture(button.Normal)

	button.Push = button:CreateTexture(nil, "ARTWORK")
	button.Push:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 4, -4)
	button.Push:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -4, 4)
	button.Push:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetPushedTexture(button.Push)

	button.Disabled = button:CreateTexture(nil, "ARTWORK")
	button.Disabled:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetDisabledTexture(button.Disabled)

	button.Checked = button:CreateTexture(nil, "ARTWORK")
	button.Checked:SetWidth( size )
	button.Checked:SetHeight( size )
	button.Checked:SetPoint("LEFT", 0, 0)
	button.Checked:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button:SetCheckedTexture(button.Checked)

	if icon then
		if icon == 1 then
			button.Icon = button:CreateTexture(nil, "OVERLAY")
			button.Icon:SetWidth(7)
			button.Icon:SetHeight(8)
			button.Icon:SetPoint("LEFT", button.Normal, "RIGHT", space, 0)
			button.Icon:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
			button.Icon:SetTexCoord(unpack(Textures.SoryOrderIcon.coordsAsc))
			button:SetWidth(size + space + 7 + space)
			button:SetHeight(size)
		elseif icon == 2 then
			button.Icon = button:CreateTexture(nil, "OVERLAY")
			button.Icon:SetWidth(7)
			button.Icon:SetHeight(8)
			button.Icon:SetPoint("LEFT", button.Normal, "RIGHT", space, 0)
			button.Icon:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
			button.Icon:SetTexCoord(unpack(Textures.SoryOrderIcon.coordsDesc))
			button:SetWidth(size + space + 7 + space)
			button:SetHeight(size)
		end
	else
		button.Text = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		button.Text:SetHeight( size )
		button.Text:SetPoint("LEFT", button.Normal, "RIGHT", space, 0)
		button.Text:SetJustifyH("LEFT")
		button.Text:SetText(text)
		button.Text:SetTextColor(1, 1, 1, 1)
		button:SetWidth(size + space + button.Text:GetStringWidth() + space)
		button:SetHeight(size)
	end

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
	button.Highlight:SetTexture(1, 1, 1, 0.1)
	button.Highlight:Hide()

	button:SetScript("OnEnter", function() button.Highlight:Show() end)
	button:SetScript("OnLeave", function() button.Highlight:Hide() end)
end
-- Template CheckButton END ----------------------------------------

-- Template Slider START ----------------------------------------
TEMPLATE.Slider = function(slider, width, step, minVal, maxVal, curVal, func, pro)
	slider:SetWidth(width)
	slider:SetHeight(sizeGeneral.SliderSize)
	slider:SetValueStep(step) 
	slider:SetMinMaxValues(minVal, maxVal)
	slider:SetValue(curVal)
	slider:SetOrientation("HORIZONTAL")

	local textMin = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	textMin:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	textMin:SetPoint("LEFT", slider, "LEFT", 0, 0)
	textMin:SetJustifyH("CENTER")
	if pro then
		textMin:SetText(minVal.."%")
	else
		textMin:SetText(minVal)
	end
	local textMax = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	textMax:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	textMax:SetPoint("RIGHT", slider, "RIGHT", 0, 0)
	textMax:SetJustifyH("CENTER")
	if pro then
		textMax:SetText(maxVal.."%")
	else
		textMax:SetText(maxVal)
	end
	local textCur = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	textCur:SetTextColor(1, 1, 1, 1)
	textCur:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	textCur:SetPoint("CENTER", slider, "CENTER", 0, 0)
	textCur:SetJustifyH("CENTER")
	if pro then
		textCur:SetText(curVal.."%")
	else
		textCur:SetText(curVal)
	end

	local sliderBGL = slider:CreateTexture(nil, "BACKGROUND")
	sliderBGL:SetWidth(5)
	sliderBGL:SetHeight(6)
	sliderBGL:SetPoint("LEFT", slider, "LEFT", 0, 0)
	sliderBGL:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	sliderBGL:SetTexCoord(unpack(Textures.SliderBG.coordsL))
	local sliderBGM = slider:CreateTexture(nil, "BACKGROUND")
	sliderBGM:SetWidth(width-5-5)
	sliderBGM:SetHeight(6)
	sliderBGM:SetPoint("LEFT", sliderBGL, "RIGHT", 0, 0)
	sliderBGM:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	sliderBGM:SetTexCoord(unpack(Textures.SliderBG.coordsM))
	local sliderBGR = slider:CreateTexture(nil, "BACKGROUND")
	sliderBGR:SetWidth(5)
	sliderBGR:SetHeight(6)
	sliderBGR:SetPoint("LEFT", sliderBGM, "RIGHT", 0, 0)
	sliderBGR:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	sliderBGR:SetTexCoord(unpack(Textures.SliderBG.coordsR))

	local thumb = slider:CreateTexture(nil, "BORDER")
	thumb:SetWidth(sizeGeneral.SliderSize)
	thumb:SetHeight(sizeGeneral.SliderSize)
	thumb:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	thumb:SetTexCoord(unpack(Textures.SliderKnob.coords))
	slider:SetThumbTexture(thumb)

	slider:SetScript("OnValueChanged", function(self, value)
		if pro then
			textCur:SetText(value.."%")
		else
			textCur:SetText(value)
		end
		if func then
			func(self, value)
		end
	end)
end
-- Template Slider END ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function Print(...)
	print("|cffffff7fAHsearch:|r", ...)
end

local function NOOP() end

local function SortedPairs(t, lvl)
	local ntab = {}
	for k,v in pairs(t) do
		table_insert(ntab,k)
	end

	if lvl then
		table_sort(ntab,
			function(a,b)
				if t[a].order == t[b].order then
					if t[a].qualityIndex == t[b].qualityIndex then
						if a < b then
							return true
						end
					end
					if t[a].qualityIndex > t[b].qualityIndex then
						return true
					end	
				end
				if t[a].order < t[b].order then
					return true
				end
			end
		)
	else
		table_sort(ntab,
			function(a,b)
				if t[a].order == t[b].order then
					if a < b then
						return true
					end
				end
				if t[a].order < t[b].order then
					return true
				end
			end
		)
	end

	local i = 0
	local function _f(_s,_v)
		i = i + 1
		local k = ntab[i]
		if (k) then
			return k,t[k]
		end
	end
	return _f,nil,nil
end

local function SearchQueryGlobalCheck()
	for cat in pairs(AHsearch_Data) do
		if AHsearch_Data[cat].queries then
			for sq in pairs(AHsearch_Data[cat].queries) do
				return true
			end
		end
	end
	return false
end

local function SearchQueryCheck(cat)
	if AHsearch_Data[cat] and AHsearch_Data[cat].queries then
		for sq in pairs(AHsearch_Data[cat].queries) do
			return false
		end
	end
	return true
end

local function VisibilityCheck(newduration)
	if configMode then
		AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
		return
	end

	if (not GVAR.QueryFrame:IsMouseOver() or not GVAR.QueryFrame:IsShown()) and not GVAR.CatFrame:IsMouseOver() then
		FrameTime = GetTime()

		local duration = 0.75
		if newduration and type(newduration) == "number" then
			duration = newduration
		end

		AHsearch.FrameOnUpdate:SetScript("OnUpdate", function()
			if configMode then
				AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
				return
			end
			if GetTime() - FrameTime > duration then
				if (not GVAR.QueryFrame:IsMouseOver() or not GVAR.QueryFrame:IsShown()) and not GVAR.CatFrame:IsMouseOver() then
					GVAR.CatFrame:Hide()
					AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
				else
					AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
				end
			end
		end)

	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Categories settingcheck START
local function ConfigurationFrame_Categories_SetNewCat(text)
	if GVAR.ConfigFrame.TabCategories.editType == "new" then
		if text ~= "" then
			AHsearch:ConfigurationFrame_Categories_CreateNewCat()
		end
	end
end

local function ConfigurationFrame_Categories_SetCheck()
	if GVAR.ConfigFrame.TabCategories.editType ~= "selected" then return end

	if GVAR.ConfigFrame.Categories.CurrentCatTxt.value and GVAR.ConfigFrame.Categories.CurrentCatTxt.value ~= "" and
	   (
	   GVAR.ConfigFrame.TabCategories.originalCat ~= GVAR.ConfigFrame.Categories.CurrentCatTxt.value or
	   GVAR.ConfigFrame.TabCategories.originalOrder ~= GVAR.ConfigFrame.Categories.Order.value or
	   GVAR.ConfigFrame.TabCategories.originalColor ~= GVAR.ConfigFrame.Categories.ColorPullDown.value or
	   GVAR.ConfigFrame.TabCategories.originalIcon ~= GVAR.ConfigFrame.Categories.IconPullDown.value
	   )
	then
		GVAR.ConfigFrame.Categories.UpdateCat:SetText(L["Update"])
		TEMPLATE.EnableTextButton(GVAR.ConfigFrame.Categories.UpdateCat, 1)
	else
		GVAR.ConfigFrame.Categories.UpdateCat:SetText(L["Update"])
		TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.UpdateCat)
	end
end

local function ConfigurationFrame_Categories_SetName(text)
	local color = GVAR.ConfigFrame.Categories.ColorPullDown.value
	if not color then
		color = "|cffffffff"
		GVAR.ConfigFrame.Categories.ColorPullDown.value = color
	end
	GVAR.ConfigFrame.Categories.CurrentCatTxt.value = text
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetText(color..text.."|r")

	if GVAR.ConfigFrame.TabCategories.editType == "new" then return end
	ConfigurationFrame_Categories_SetCheck()
end

local function ConfigurationFrame_Categories_SetOrder()
	ConfigurationFrame_Categories_SetCheck()
end

local function ConfigurationFrame_Categories_SetColor(val)
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetText(val..GVAR.ConfigFrame.Categories.CurrentCatTxt.value)
	ConfigurationFrame_Categories_SetCheck()
end

local function ConfigurationFrame_Categories_SetIcon(val)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexture(categoryIcons[val].texture)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexCoord(unpack(categoryIcons[val].coords))
	ConfigurationFrame_Categories_SetCheck()
end
-- Categories settingcheck END

-- SearchQueries settingcheck START
local function ConfigurationFrame_SearchQueries_SetCheck()
	if GVAR.ConfigFrame.TabSearchQueries.editType ~= "selected" then return end
	if GVAR.ConfigFrame.TabSearchQueries.currentSQtitle ~= "" and
	  (
	  GVAR.ConfigFrame.TabSearchQueries.originalSQtitle ~= GVAR.ConfigFrame.TabSearchQueries.currentSQtitle or
	  GVAR.ConfigFrame.TabSearchQueries.originalNameQuerySQ ~= GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value or
	  GVAR.ConfigFrame.TabSearchQueries.originalOrder ~= GVAR.ConfigFrame.SearchQueries.Order.value or
	  GVAR.ConfigFrame.TabSearchQueries.originalRarity ~= GVAR.ConfigFrame.SearchQueries.RarityPullDown.value or

	  (
	   (
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter1 ~= GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value and
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter1 and GVAR.ConfigFrame.TabSearchQueries.originalFilter1 > 0
	   )
	  or
	   (
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter1 ~= GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value and
	    GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value and GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value > 0
	   )
	  ) or

	  (
	   (
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter2 ~= GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value and
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter2 and GVAR.ConfigFrame.TabSearchQueries.originalFilter2 > 0
	   )
	  or
	   (
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter2 ~= GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value and
	    GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value and GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value > 0
	   )
	  ) or

	  (
	   (
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter3 ~= GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value and
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter3 and GVAR.ConfigFrame.TabSearchQueries.originalFilter3 > 0
	   )
	  or
	   (
	    GVAR.ConfigFrame.TabSearchQueries.originalFilter3 ~= GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value and
	    GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value and GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value > 0
	   )
	  ) or

	  GVAR.ConfigFrame.TabSearchQueries.originalLevelMin ~= GVAR.ConfigFrame.SearchQueries.LevelMin.value or
	  GVAR.ConfigFrame.TabSearchQueries.originalLevelMax ~= GVAR.ConfigFrame.SearchQueries.LevelMax.value or

	  GVAR.ConfigFrame.TabSearchQueries.originalSortBy ~= GVAR.ConfigFrame.SearchQueries.SortByPullDown.value or
	  GVAR.ConfigFrame.TabSearchQueries.originalSortOrderAsc ~= GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value or

	  GVAR.ConfigFrame.TabSearchQueries.originalUsable ~= GVAR.ConfigFrame.SearchQueries.Usable.value or
	  GVAR.ConfigFrame.TabSearchQueries.originalDisplayOnChar ~= GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
	  )
	then
		if GVAR.ConfigFrame.SearchQueries.MovePullDown.value then
			if GVAR.ConfigFrame.SearchQueries.MovePullDown.value == GVAR.ConfigFrame.SearchQueries.SelectCat.cat then
				GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"])
				TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
			else
				GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"].." & "..L["Move"])
				TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
			end
		else
			GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"])
			TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
		end
	else
		if GVAR.ConfigFrame.SearchQueries.MovePullDown.value then
			if GVAR.ConfigFrame.SearchQueries.MovePullDown.value == GVAR.ConfigFrame.SearchQueries.SelectCat.cat then
				GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"])
				TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
			else
				if GVAR.ConfigFrame.TabSearchQueries.currentSQtitle ~= "" then
					GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Move"])
					TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
				else
					GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Move"])
					TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
				end
			end
		else
			GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"])
			TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
		end
	end
end

local function ConfigurationFrame_SearchQueries_SetTitleSQ(text)
	GVAR.ConfigFrame.TabSearchQueries.currentSQtitle = text
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetRarity(val)
	if val > 0 then
		GVAR.ConfigFrame.SearchQueries.RarityInfo:Enable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, false) -- RARUPD
	else
		GVAR.ConfigFrame.SearchQueries.RarityInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, true) -- RARUPD
	end
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetFilter1(val)
	curFilter1 = val
	curFilter2 = nil
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = nil
	curFilter3 = nil
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = nil
	AHsearch:UpdateFilter()
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetFilter2(val)
	curFilter2 = val
	curFilter3 = nil
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = nil
	AHsearch:UpdateFilter()
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetFilter3()
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetMove(val)
	GVAR.ConfigFrame.SearchQueries.MovePullDown.value = GUI_CatFrame_data[ val ].Cat
	if GVAR.ConfigFrame.SearchQueries.MovePullDown.value == GVAR.ConfigFrame.SearchQueries.SelectCat.cat then
		GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:Hide()
	else
		GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:Show()
	end
	ConfigurationFrame_SearchQueries_SetCheck()
end
-- SearchQueries settingcheck END
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:InitOptions()
	-- VERSION CHECK START --------------------
	-- new user START
	if AHsearch_Options.version == nil then
		AHsearch_Options.version = 4
	end
	-- new user END

	-- very very old user START
	if AHsearch_Options.version <= 3 then
		if AHsearch_Options then table.wipe(AHsearch_Options) end
		if AHsearch_Data then table.wipe(AHsearch_Data) end
		if AHsearchDB then table.wipe(AHsearchDB) end
		Print("Sorry, your AHsearch version is <= 3 (26 Sep 2008) and is no longer supported.")
		Print("All data and options has been deleted!")
		AHsearch_Options.version = 4
	end
	-- very very old user END

	-- toc30000+ user START
	if AHsearch_Options.version == 3.1 then
		AHsearch_Options.chatOutput = nil
		AHsearch_Options.WotLKfix = nil
		AHsearch_Options.noWotLKmessage = nil
		AHsearch_Options.ttDisplay = nil
		AHsearch_Options.ttFiltersDisplay = nil
		if AHsearch_Options.cycleButtonsNextPrev then AHsearch_Options.cycleNP  = AHsearch_Options.cycleButtonsNextPrev end
		if AHsearch_Options.cycleButtonsRandom   then AHsearch_Options.cycleRND = AHsearch_Options.cycleButtonsRandom   end
		AHsearch_Options.cycleButtonsNextPrev = nil
		AHsearch_Options.cycleButtonsRandom = nil
		DBupdate()
		AHsearch_Options.version = 4
	end
	-- toc30000+ user END
	-- VERSION CHECK END --------------------
	
	if AHsearch_Options.curCat        == nil then AHsearch_Options.curCat        = default_curCat        end -- = 25 rows
	if AHsearch_Options.curCatWidth   == nil then AHsearch_Options.curCatWidth   = default_curCatWidth   end -- = 175 px
	if AHsearch_Options.curQuery      == nil then AHsearch_Options.curQuery      = default_curQuery      end -- = 25 rows
	if AHsearch_Options.curQueryWidth == nil then AHsearch_Options.curQueryWidth = default_curQueryWidth end -- = 175 px
	if AHsearch_Options.cycleNP       == nil then AHsearch_Options.cycleNP       = default_cycleNP       end -- = false
	if AHsearch_Options.cycleRND      == nil then AHsearch_Options.cycleRND      = default_cycleRND      end -- = false
	if AHsearch_Options.scaleMain     == nil then AHsearch_Options.scaleMain     = default_scaleMain     end -- = 100%
	if AHsearch_Options.hideMain      == nil then AHsearch_Options.hideMain      = default_hideMain      end -- = false
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:MainFrame_Create()
	AHsearch.FrameOnUpdate = CreateFrame("Button", nil, UIParent)

	GVAR.MainFrame = CreateFrame("Frame", nil, AuctionFrameBrowse)
	GVAR.MainFrame:SetFrameLevel( AuctionFrameBrowse:GetFrameLevel() + 20 )
	GVAR.MainFrame:SetWidth(20)
	GVAR.MainFrame:SetHeight(20)
	GVAR.MainFrame:SetToplevel(true)
	GVAR.MainFrame:ClearAllPoints()
	GVAR.MainFrame:SetPoint("RIGHT", "BrowseName", "RIGHT", 0, 0)

	-- AHsearch Button START
	GVAR.MainFrame.AHsearchButton = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.AHsearchButton:SetFrameLevel( GVAR.MainFrame:GetFrameLevel() + 10 )
	GVAR.MainFrame.AHsearchButton:SetWidth(20)
	GVAR.MainFrame.AHsearchButton:SetHeight(20)
	GVAR.MainFrame.AHsearchButton:ClearAllPoints()
	GVAR.MainFrame.AHsearchButton:SetPoint("TOPLEFT", GVAR.MainFrame, 0, 0)
	GVAR.MainFrame.AHsearchButton:RegisterForClicks("AnyUp")
	GVAR.MainFrame.AHsearchButton:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			if configMode then return end
			AHsearch:CatFrame_Show()
			if GVAR.CatFrame:IsShown() then
				if GUI_CatFrame_Count == 0 then
					VisibilityCheck(4)
				else
					VisibilityCheck(2)
				end
			else
				AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
			end
		elseif button == "RightButton" then
			if configMode then
				AHsearch:ConfigDisable()
			else
				AHsearch:ConfigEnable()
			end
		end
	end)
	GVAR.MainFrame.AHsearchButton:EnableMouseWheel(true)
	GVAR.MainFrame.AHsearchButton:SetScript("OnMouseWheel", NOOP)
	
	GVAR.MainFrame.AHsearchButton.NormalTexture = GVAR.MainFrame.AHsearchButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchButton.NormalTexture:SetAllPoints(GVAR.MainFrame.AHsearchButton)
	GVAR.MainFrame.AHsearchButton.NormalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchButton.NormalTexture:SetTexCoord(unpack(Textures.AHsBmain.coords))
	GVAR.MainFrame.AHsearchButton:SetNormalTexture(GVAR.MainFrame.AHsearchButton.NormalTexture)

	GVAR.MainFrame.AHsearchButton.HighlightTexture = GVAR.MainFrame.AHsearchButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchButton.HighlightTexture:SetAllPoints(GVAR.MainFrame.AHsearchButton)
	GVAR.MainFrame.AHsearchButton.HighlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchButton.HighlightTexture:SetTexCoord(unpack(Textures.AHsBmain.coordsHL))
	GVAR.MainFrame.AHsearchButton:SetHighlightTexture(GVAR.MainFrame.AHsearchButton.HighlightTexture)

	GVAR.MainFrame.AHsearchButton.PushedTexture = GVAR.MainFrame.AHsearchButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchButton.PushedTexture:SetAllPoints(GVAR.MainFrame.AHsearchButton)
	GVAR.MainFrame.AHsearchButton.PushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchButton.PushedTexture:SetTexCoord(unpack(Textures.AHsBmain.coordsPS))
	GVAR.MainFrame.AHsearchButton:SetPushedTexture(GVAR.MainFrame.AHsearchButton.PushedTexture)
	-- AHsearch Button END

	-- AHsearch Previous Button START
	GVAR.MainFrame.AHsearchPrevButton = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.AHsearchPrevButton:SetFrameLevel( GVAR.MainFrame:GetFrameLevel() + 10 )
	GVAR.MainFrame.AHsearchPrevButton:SetWidth(20.30769) -- 22/13*12 original: 22
	GVAR.MainFrame.AHsearchPrevButton:SetHeight(12) -- original: 13
	GVAR.MainFrame.AHsearchPrevButton:ClearAllPoints()
	GVAR.MainFrame.AHsearchPrevButton:SetPoint("TOPRIGHT", GVAR.MainFrame, "BOTTOMLEFT", 1, 0)
	GVAR.MainFrame.AHsearchPrevButton:SetScript("OnClick", function()
		if configMode then return end
		AHsearch:CycleSearch(-1)
	end)
	GVAR.MainFrame.AHsearchPrevButton:EnableMouseWheel(true)
	GVAR.MainFrame.AHsearchPrevButton:SetScript("OnMouseWheel", NOOP)

	GVAR.MainFrame.AHsearchPrevButton.NormalTexture = GVAR.MainFrame.AHsearchPrevButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchPrevButton.NormalTexture:SetAllPoints(GVAR.MainFrame.AHsearchPrevButton)
	GVAR.MainFrame.AHsearchPrevButton.NormalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchPrevButton.NormalTexture:SetTexCoord(unpack(Textures.AHsBprev.coords))
	GVAR.MainFrame.AHsearchPrevButton:SetNormalTexture(GVAR.MainFrame.AHsearchPrevButton.NormalTexture)

	GVAR.MainFrame.AHsearchPrevButton.HighlightTexture = GVAR.MainFrame.AHsearchPrevButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchPrevButton.HighlightTexture:SetAllPoints(GVAR.MainFrame.AHsearchPrevButton)
	GVAR.MainFrame.AHsearchPrevButton.HighlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchPrevButton.HighlightTexture:SetTexCoord(unpack(Textures.AHsBprev.coordsHL))
	GVAR.MainFrame.AHsearchPrevButton:SetHighlightTexture(GVAR.MainFrame.AHsearchPrevButton.HighlightTexture)

	GVAR.MainFrame.AHsearchPrevButton.DisableTexture = GVAR.MainFrame.AHsearchPrevButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchPrevButton.DisableTexture:SetAllPoints(GVAR.MainFrame.AHsearchPrevButton)
	GVAR.MainFrame.AHsearchPrevButton.DisableTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchPrevButton.DisableTexture:SetTexCoord(unpack(Textures.AHsBprev.coords))
	GVAR.MainFrame.AHsearchPrevButton.DisableTexture:SetAlpha(0.5)
	local shaderSupported = GVAR.MainFrame.AHsearchPrevButton.DisableTexture:SetDesaturated(true)
	if not shaderSupported then
		GVAR.MainFrame.AHsearchPrevButton.DisableTexture:SetVertexColor(0.5, 0.5, 0.5)
	end
	GVAR.MainFrame.AHsearchPrevButton:SetDisabledTexture(GVAR.MainFrame.AHsearchPrevButton.DisableTexture)

	GVAR.MainFrame.AHsearchPrevButton.PushedTexture = GVAR.MainFrame.AHsearchPrevButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchPrevButton.PushedTexture:SetAllPoints(GVAR.MainFrame.AHsearchPrevButton)
	GVAR.MainFrame.AHsearchPrevButton.PushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchPrevButton.PushedTexture:SetTexCoord(unpack(Textures.AHsBprev.coordsPS))
	GVAR.MainFrame.AHsearchPrevButton:SetPushedTexture(GVAR.MainFrame.AHsearchPrevButton.PushedTexture)
	-- AHsearch Previous Button END

	-- AHsearch Next Button START
	GVAR.MainFrame.AHsearchNextButton = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.AHsearchNextButton:SetFrameLevel( GVAR.MainFrame:GetFrameLevel() + 10 )
	GVAR.MainFrame.AHsearchNextButton:SetWidth(20.30769) -- 22/13*12 original: 22
	GVAR.MainFrame.AHsearchNextButton:SetHeight(12) -- original: 13
	GVAR.MainFrame.AHsearchNextButton:ClearAllPoints()
	GVAR.MainFrame.AHsearchNextButton:SetPoint("TOPLEFT", GVAR.MainFrame, "BOTTOMRIGHT", -1, 0)
	GVAR.MainFrame.AHsearchNextButton:SetScript("OnClick", function()
		if configMode then return end
		AHsearch:CycleSearch(1)
	end)
	GVAR.MainFrame.AHsearchNextButton:EnableMouseWheel(true)
	GVAR.MainFrame.AHsearchNextButton:SetScript("OnMouseWheel", NOOP)

	GVAR.MainFrame.AHsearchNextButton.NormalTexture = GVAR.MainFrame.AHsearchNextButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchNextButton.NormalTexture:SetAllPoints(GVAR.MainFrame.AHsearchNextButton)
	GVAR.MainFrame.AHsearchNextButton.NormalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchNextButton.NormalTexture:SetTexCoord(unpack(Textures.AHsBnext.coords))
	GVAR.MainFrame.AHsearchNextButton:SetNormalTexture(GVAR.MainFrame.AHsearchNextButton.NormalTexture)

	GVAR.MainFrame.AHsearchNextButton.HighlightTexture = GVAR.MainFrame.AHsearchNextButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchNextButton.HighlightTexture:SetAllPoints(GVAR.MainFrame.AHsearchNextButton)
	GVAR.MainFrame.AHsearchNextButton.HighlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchNextButton.HighlightTexture:SetTexCoord(unpack(Textures.AHsBnext.coordsHL))
	GVAR.MainFrame.AHsearchNextButton:SetHighlightTexture(GVAR.MainFrame.AHsearchNextButton.HighlightTexture)

	GVAR.MainFrame.AHsearchNextButton.DisableTexture = GVAR.MainFrame.AHsearchNextButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchNextButton.DisableTexture:SetAllPoints(GVAR.MainFrame.AHsearchNextButton)
	GVAR.MainFrame.AHsearchNextButton.DisableTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchNextButton.DisableTexture:SetTexCoord(unpack(Textures.AHsBnext.coords))
	GVAR.MainFrame.AHsearchNextButton.DisableTexture:SetAlpha(0.5)
	local shaderSupported = GVAR.MainFrame.AHsearchNextButton.DisableTexture:SetDesaturated(true)
	if not shaderSupported then
		GVAR.MainFrame.AHsearchNextButton.DisableTexture:SetVertexColor(0.5, 0.5, 0.5)
	end
	GVAR.MainFrame.AHsearchNextButton:SetDisabledTexture(GVAR.MainFrame.AHsearchNextButton.DisableTexture)

	GVAR.MainFrame.AHsearchNextButton.PushedTexture = GVAR.MainFrame.AHsearchNextButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchNextButton.PushedTexture:SetAllPoints(GVAR.MainFrame.AHsearchNextButton)
	GVAR.MainFrame.AHsearchNextButton.PushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchNextButton.PushedTexture:SetTexCoord(unpack(Textures.AHsBnext.coordsPS))
	GVAR.MainFrame.AHsearchNextButton:SetPushedTexture(GVAR.MainFrame.AHsearchNextButton.PushedTexture)
	-- AHsearch Next Button END

	-- AHsearch Random Button START
	GVAR.MainFrame.AHsearchRandomButton = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.AHsearchRandomButton:SetFrameLevel( GVAR.MainFrame:GetFrameLevel() + 10 )
	GVAR.MainFrame.AHsearchRandomButton:SetWidth(12)
	GVAR.MainFrame.AHsearchRandomButton:SetHeight(12)
	GVAR.MainFrame.AHsearchRandomButton:ClearAllPoints()
	GVAR.MainFrame.AHsearchRandomButton:SetPoint("TOP", GVAR.MainFrame, "BOTTOM", 0, 0)
	GVAR.MainFrame.AHsearchRandomButton:SetScript("OnClick", function()
		if configMode then return end
		AHsearch:RandomSearch()
	end)
	GVAR.MainFrame.AHsearchRandomButton:EnableMouseWheel(true)
	GVAR.MainFrame.AHsearchRandomButton:SetScript("OnMouseWheel", NOOP)

	GVAR.MainFrame.AHsearchRandomButton.NormalTexture = GVAR.MainFrame.AHsearchRandomButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchRandomButton.NormalTexture:SetAllPoints(GVAR.MainFrame.AHsearchRandomButton)
	GVAR.MainFrame.AHsearchRandomButton.NormalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchRandomButton.NormalTexture:SetTexCoord(unpack(Textures.AHsBrandom.coords))
	GVAR.MainFrame.AHsearchRandomButton:SetNormalTexture(GVAR.MainFrame.AHsearchRandomButton.NormalTexture)

	GVAR.MainFrame.AHsearchRandomButton.HighlightTexture = GVAR.MainFrame.AHsearchRandomButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchRandomButton.HighlightTexture:SetAllPoints(GVAR.MainFrame.AHsearchRandomButton)
	GVAR.MainFrame.AHsearchRandomButton.HighlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchRandomButton.HighlightTexture:SetTexCoord(unpack(Textures.AHsBrandom.coordsHL))
	GVAR.MainFrame.AHsearchRandomButton:SetHighlightTexture(GVAR.MainFrame.AHsearchRandomButton.HighlightTexture)

	GVAR.MainFrame.AHsearchRandomButton.DisableTexture = GVAR.MainFrame.AHsearchRandomButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchRandomButton.DisableTexture:SetAllPoints(GVAR.MainFrame.AHsearchRandomButton)
	GVAR.MainFrame.AHsearchRandomButton.DisableTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchRandomButton.DisableTexture:SetTexCoord(unpack(Textures.AHsBrandom.coords))
	GVAR.MainFrame.AHsearchRandomButton.DisableTexture:SetAlpha(0.5)
	local shaderSupported = GVAR.MainFrame.AHsearchRandomButton.DisableTexture:SetDesaturated(true)
	if not shaderSupported then
		GVAR.MainFrame.AHsearchRandomButton.DisableTexture:SetVertexColor(0.5, 0.5, 0.5)
	end
	GVAR.MainFrame.AHsearchRandomButton:SetDisabledTexture(GVAR.MainFrame.AHsearchRandomButton.DisableTexture)

	GVAR.MainFrame.AHsearchRandomButton.PushedTexture = GVAR.MainFrame.AHsearchRandomButton:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.AHsearchRandomButton.PushedTexture:SetAllPoints(GVAR.MainFrame.AHsearchRandomButton)
	GVAR.MainFrame.AHsearchRandomButton.PushedTexture:SetPoint("TOPLEFT", 1, -1)
	GVAR.MainFrame.AHsearchRandomButton.PushedTexture:SetPoint("BOTTOMRIGHT", -1, 1)
	GVAR.MainFrame.AHsearchRandomButton.PushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	GVAR.MainFrame.AHsearchRandomButton.PushedTexture:SetTexCoord(unpack(Textures.AHsBrandom.coords))
	GVAR.MainFrame.AHsearchRandomButton:SetPushedTexture(GVAR.MainFrame.AHsearchRandomButton.PushedTexture)
	-- AHsearch Random Button END
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CycleButtons_Toggle()
	if AHsearch_Options.cycleNP then
		GVAR.MainFrame.AHsearchPrevButton:Show()
		GVAR.MainFrame.AHsearchNextButton:Show()
	else
		GVAR.MainFrame.AHsearchPrevButton:Hide()
		GVAR.MainFrame.AHsearchNextButton:Hide()
	end
	if AHsearch_Options.cycleRND then
		GVAR.MainFrame.AHsearchRandomButton:Show()
		GVAR.MainFrame.AHsearchPrevButton:SetPoint("TOPRIGHT", GVAR.MainFrame, "BOTTOMLEFT", 1, 0)
		GVAR.MainFrame.AHsearchNextButton:SetPoint("TOPLEFT", GVAR.MainFrame, "BOTTOMRIGHT", -1, 0)
	else
		GVAR.MainFrame.AHsearchRandomButton:Hide()
		GVAR.MainFrame.AHsearchPrevButton:SetPoint("TOPRIGHT", GVAR.MainFrame, "BOTTOMLEFT", 5, 0)
		GVAR.MainFrame.AHsearchNextButton:SetPoint("TOPLEFT", GVAR.MainFrame, "BOTTOMRIGHT", -5, 0)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:BuildStandardTable()
	table.wipe(GUI_CatFrame_data)
	table.wipe(GUI_SQFrame_data)
	local x = 1
	for cat in SortedPairs(AHsearch_Data) do
		GUI_CatFrame_data[x] = {}
		GUI_CatFrame_data[x].Texture = AHsearch_Data[cat].icon
		GUI_CatFrame_data[x].CatColor = AHsearch_Data[cat].color
		GUI_CatFrame_data[x].Order = AHsearch_Data[cat].order
		GUI_CatFrame_data[x].Cat = cat
		x = x + 1

		local y = 1
		if not GUI_SQFrame_data[cat] then GUI_SQFrame_data[cat] = {} end
		GUI_SQFrame_Count[cat] = 0
		if AHsearch_Data[cat].queries then
			for sq in SortedPairs(AHsearch_Data[cat].queries, 1) do
				GUI_SQFrame_data[cat][y] = {}
				GUI_SQFrame_data[cat][y].QueryColor = itemRarity[ AHsearch_Data[cat].queries[sq].qualityIndex+2 ].color
				GUI_SQFrame_data[cat][y].Order = AHsearch_Data[cat].queries[sq].order
				GUI_SQFrame_data[cat][y].Query = sq
				GUI_SQFrame_data[cat][y].Cat = cat
				y = y + 1
			end
			GUI_SQFrame_Count[cat] = #GUI_SQFrame_data[cat]
		end

	end
	GUI_CatFrame_Count = #GUI_CatFrame_data
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:BuildFilterTable()
	local x1 = 1
	local itemClasses = { GetAuctionItemClasses() }
	for itemClassIndex, itemClass in pairs(itemClasses) do
		--print(itemClassIndex, itemClass) -- TEST
		FilterTable[x1] = {}
		FilterTable[x1].itemClassIndex = itemClassIndex
		FilterTable[x1].itemClass = itemClass

		local x2 = 1
		local subClasses = { GetAuctionItemSubClasses(itemClassIndex) }
		for subClassIndex, subClass in pairs(subClasses) do
			--print("-", subClassIndex, subClass) -- TEST
			if not FilterTable[x1].subClasses then
				FilterTable[x1].subClasses = {}
			end
			FilterTable[x1].subClasses[x2] = {}
			FilterTable[x1].subClasses[x2].subClassIndex = subClassIndex
			FilterTable[x1].subClasses[x2].subClass = subClass

			local x3 = 1
			local typ
			local subClassInvTypes = { GetAuctionInvTypes(itemClassIndex,subClassIndex) }
			for subClassInvTypIndex, subClassInvTyp in pairs(subClassInvTypes) do
				if type(subClassInvTyp) == "string" then
					typ = subClassInvTyp
				elseif type(subClassInvTyp) == "number" then
					--print("--", subClassInvTypIndex/2, _G[typ]) -- TEST
					if not FilterTable[x1].subClasses[x2].subClassInvTypes then
						FilterTable[x1].subClasses[x2].subClassInvTypes = {}
					end
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3] = {}
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3].subClassInvTypIndex = subClassInvTypIndex/2
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3].subClassInvTyp = _G[typ] or "localization error"
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3].subClassInvRealTyp = typ
					x3 = x3 + 1
				end
			end
			x2 = x2 + 1			

		end
		x1 = x1 + 1
	end

--[[
	-- TEST
	print("----")
	for i = 1, #FilterTable do
		print(i, "#", FilterTable[i].itemClassIndex, FilterTable[i].itemClass)
		if FilterTable[i].subClasses then
			for j = 1, #FilterTable[i].subClasses do
				print(i, j, "#", "-", FilterTable[i].subClasses[j].subClassIndex, FilterTable[i].subClasses[j].subClass)
				if FilterTable[i].subClasses[j].subClassInvTypes then
					for k = 1, #FilterTable[i].subClasses[j].subClassInvTypes do
						print(i, j, k, "#", "--", FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvRealTyp)
					end
				end
			end
		end
	end
	-- TEST
--]]
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:OnVerticalScroll(frameName, value)
	frameName.ScrollFrame.ScrollBar:SetValue(value)
	frameName.ScrollFrame.offset = math_floor((value / sizeGeneral.BarHeight) + 0.5)

	if configMode then
		if frameName == GVAR.CatFrame then
			local catMaxConfig = (GUI_CatFrame_Count - 10) * sizeGeneral.BarHeight
			local catMaxNormal = (GUI_CatFrame_Count - AHsearch_Options.curCat) * sizeGeneral.BarHeight
			local nvalue = 0
			--print("#", catMaxNormal) -- TEST
			if catMaxNormal ~= 0 then
				nvalue = math_floor((value*(catMaxConfig/catMaxNormal)) + 0.5)
			end
			--print("#", GUI_CatFrame_Count, catMaxConfig, catMaxNormal, dif, nvalue, type(nvalue)) -- TEST
			local nvalueOffset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
			GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.ScrollBar:SetValue(nvalue)
			GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.offset = nvalueOffset
			GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.ScrollBar:SetValue(nvalue)
			GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.offset = nvalueOffset
		elseif frameName == GVAR.ConfigFrame.Categories.SelectCat then
			local catMaxConfig = (GUI_CatFrame_Count - 10) * sizeGeneral.BarHeight
			local catMaxNormal = (GUI_CatFrame_Count - AHsearch_Options.curCat) * sizeGeneral.BarHeight
			local nvalue = 0
			--print("-", catMaxConfig) -- TEST
			if catMaxConfig ~= 0 then
				nvalue = math_floor((value*(catMaxNormal/catMaxConfig)) + 0.5)
			end
			--print("-", GUI_CatFrame_Count, catMaxConfig, catMaxNormal, nvalue, type(nvalue)) -- TEST
			GVAR.CatFrame.ScrollFrame.ScrollBar:SetValue(nvalue)
			GVAR.CatFrame.ScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		elseif frameName == GVAR.ConfigFrame.SearchQueries.SelectCat then
			local catMaxConfig = (GUI_CatFrame_Count - 10) * sizeGeneral.BarHeight
			local catMaxNormal = (GUI_CatFrame_Count - AHsearch_Options.curCat) * sizeGeneral.BarHeight
			local nvalue = 0
			--print("*", catMaxConfig) -- TEST
			if catMaxConfig ~= 0 then
				nvalue = math_floor((value*(catMaxNormal/catMaxConfig)) + 0.5)
			end
			--print("*", GUI_CatFrame_Count, catMaxConfig, catMaxNormal, nvalue, type(nvalue)) -- TEST
			GVAR.CatFrame.ScrollFrame.ScrollBar:SetValue(nvalue)
			GVAR.CatFrame.ScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		elseif frameName == GVAR.QueryFrame then
			local sqMaxConfig = (GUI_SQFrame_Count[GUI_curCat] - 10) * sizeGeneral.BarHeight
			local sqMaxNormal = (GUI_SQFrame_Count[GUI_curCat] - AHsearch_Options.curQuery) * sizeGeneral.BarHeight
			local nvalue = 0
			--print("+", sqMaxNormal) -- TEST
			if sqMaxNormal ~= 0 then
				nvalue = math_floor((value*(sqMaxConfig/sqMaxNormal)) + 0.5)
			end
			--print("+", GUI_SQFrame_Count[GUI_curCat], sqMaxConfig, sqMaxNormal, nvalue, type(nvalue)) -- TEST
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.ScrollBar:SetValue(nvalue)
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		elseif frameName == GVAR.ConfigFrame.SearchQueries.SelectSQ then
			local sqMaxConfig = (GUI_SQFrame_Count[GUI_curCat] - 10) * sizeGeneral.BarHeight
			local sqMaxNormal = (GUI_SQFrame_Count[GUI_curCat] - AHsearch_Options.curQuery) * sizeGeneral.BarHeight
			local nvalue = 0
			--print("-", sqMaxConfig) -- TEST
			if sqMaxConfig ~= 0 then
				nvalue = math_floor((value*(sqMaxNormal/sqMaxConfig)) + 0.5)
			end
			--print("-", GUI_SQFrame_Count[GUI_curCat], sqMaxConfig, sqMaxNormal, nvalue, type(nvalue)) -- TEST
			GVAR.QueryFrame.ScrollFrame.ScrollBar:SetValue(nvalue)
			GVAR.QueryFrame.ScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		end
	end

	AHsearch:FillList(frameName)
end

function AHsearch:FillList(frameName)
	-- frameName dependent START ----------
	local maxNum = 0
	local countNum = 0
	local numToDisplay = 0
	if frameName == GVAR.CatFrame then--print("1")
		maxNum = maxCat
		countNum = GUI_CatFrame_Count
		numToDisplay = AHsearch_Options.curCat
	elseif frameName == GVAR.QueryFrame then--print("2")
		maxNum = maxQuery
		countNum = GUI_SQFrame_Count[GUI_curCat]
		numToDisplay = AHsearch_Options.curQuery
	elseif frameName == GVAR.ConfigFrame.Categories.SelectCat then--print("3")
		maxNum = 10
		countNum = GUI_CatFrame_Count
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.Categories.ColorPullDown then--print("4")
		maxNum = 10
		countNum = #categoryColors
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.Categories.IconPullDown then--print("5")
		maxNum = 10
		countNum = #categoryIcons
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.SelectCat then--print("6")
		maxNum = 10
		countNum = GUI_CatFrame_Count
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.SelectSQ then--print("7")
		maxNum = 10
		countNum = GUI_SQFrame_Count[GUI_curCat]
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.MovePullDown then--print("8")
		maxNum = 10
		countNum = GUI_CatFrame_Count
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.RarityPullDown then--print("9")
		maxNum = 10
		countNum = #itemRarity
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.Filter1PullDown then--print("10")
		maxNum = 10
		countNum = #Filter1Table
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.Filter2PullDown then--print("11")
		maxNum = 10
		countNum = #Filter2Table
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.Filter3PullDown then--print("12")
		maxNum = 10
		countNum = #Filter3Table
		numToDisplay = maxNum
	elseif frameName == GVAR.ConfigFrame.SearchQueries.SortByPullDown then--print("13")
		maxNum = 10
		countNum = #SortByTable
		numToDisplay = maxNum
	end
	-- frameName dependent END ----------

	local scrollFrame = frameName.ScrollFrame
	local buttonName = frameName.ScrollFrame.Button
	local scrollFrameHeight = 0
	--local numToDisplay = maxNum
	local valueStep = sizeGeneral.BarHeight
	local scrollFrameIsShown

	if countNum < numToDisplay then
		numToDisplay = countNum
	end

	if countNum > 0 then
		scrollFrameHeight = (countNum - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollFrame.ScrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollFrame.ScrollBar:SetValueStep(valueStep)

	if countNum <= numToDisplay then
		scrollFrameIsShown = false
		scrollFrame.ScrollBar:Hide()
	else
		scrollFrameIsShown = true
		scrollFrame.ScrollBar:Show()
	end

	if scrollFrame.ScrollBar:GetValue() == 0 then
		scrollFrame.ScrollBarScrollUpButton:Disable()
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
	end
	if ((scrollFrame.ScrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollFrame.ScrollBarScrollDownButton:Disable()
	else
		scrollFrame.ScrollBarScrollDownButton:Enable()
	end

	for i=1, maxNum do
		buttonName[i]:Hide()
	end
	if countNum == 0 then return end

	local j = 1
	local sliderPos = scrollFrame.offset
	if sliderPos < 0 then
		sliderPos = 0
		scrollFrame.offset = 0
	end
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < countNum then
			local button = buttonName[j]
			-- frameName dependent START ----------
			if frameName == GVAR.CatFrame then
				button.Texture:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				button.Texture:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				button.Text:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				button.cat = GUI_CatFrame_data[i].Cat
				if configMode and GUI_curCat == GUI_CatFrame_data[i].Cat then
					button.Selected:Show()
					button.SelectedArrow:Show()
				else
					button.Selected:Hide()
					button.SelectedArrow:Hide()
				end
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.CatButtonSFWidth)
					button.Text:SetWidth(sizeGeneral.CatTextSFWidth)
				else
					button:SetWidth(sizeGeneral.CatButtonNoSFWidth)
					button.Text:SetWidth(sizeGeneral.CatTextNoSFWidth)
				end
			elseif frameName == GVAR.QueryFrame then
				button.Text:SetText(GUI_SQFrame_data[GUI_curCat][i].QueryColor..GUI_SQFrame_data[GUI_curCat][i].Query.."|r")
				button.sq = GUI_SQFrame_data[GUI_curCat][i].Query
				button.cat = GUI_SQFrame_data[GUI_curCat][i].Cat
				if configMode and GUI_curSQ == GUI_SQFrame_data[GUI_curCat][i].Query then
					button.Selected:Show()
					button.SelectedArrow:Show()
				else
					button.Selected:Hide()
					button.SelectedArrow:Hide()
				end
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.QueryButtonSFWidth)
					button.Text:SetWidth(sizeGeneral.QueryTextSFWidth)
				else
					button:SetWidth(sizeGeneral.QueryButtonNoSFWidth)
					button.Text:SetWidth(sizeGeneral.QueryTextNoSFWidth)
				end

			elseif frameName == GVAR.ConfigFrame.Categories.SelectCat then
				button.Texture:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				button.Texture:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				button.Text:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				button.Order:SetText(GUI_CatFrame_data[i].Order)
				button.cat = GUI_CatFrame_data[i].Cat
				if GUI_curCat == GUI_CatFrame_data[i].Cat then
					button.Selected:Show()
					button.SelectedArrow:Show()
				else
					button.Selected:Hide()
					button.SelectedArrow:Hide()
				end
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigCatButtonNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigCatTextNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.Categories.ColorPullDown then
				button.Text:SetText(categoryColors[i].."### "..i)
				button.value = categoryColors[i]
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownColorSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownColorSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownColorNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownColorNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.Categories.IconPullDown then
				button.Texture:SetTexture(categoryIcons[i].texture)
				button.Texture:SetTexCoord(unpack(categoryIcons[i].coords))
				button.value = i
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownIconSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownIconNoSFWidth)
				end

			elseif frameName == GVAR.ConfigFrame.SearchQueries.SelectCat then
				button.Texture:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				button.Texture:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				button.Text:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				button.cat = GUI_CatFrame_data[i].Cat
				if GUI_curCat == GUI_CatFrame_data[i].Cat then
					button.Selected:Show()
					button.SelectedArrow:Show()
				else
					button.Selected:Hide()
					button.SelectedArrow:Hide()
				end
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigCatButtonNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigCatTextNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.SelectSQ then
				button.Text:SetText(GUI_SQFrame_data[GUI_curCat][i].QueryColor..GUI_SQFrame_data[GUI_curCat][i].Query.."|r")
				button.Order:SetText(GUI_SQFrame_data[GUI_curCat][i].Order)
				button.sq = GUI_SQFrame_data[GUI_curCat][i].Query
				button.cat = GUI_SQFrame_data[GUI_curCat][i].Cat
				if configMode and GUI_curSQ == GUI_SQFrame_data[GUI_curCat][i].Query then
					button.Selected:Show()
					button.SelectedArrow:Show()
				else
					button.Selected:Hide()
					button.SelectedArrow:Hide()
				end
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigQueryButtonSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigQueryTextSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigQueryButtonNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigQueryTextNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.MovePullDown then
				button.Texture:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				button.Texture:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				button.Text:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				button.value = i
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigCatButtonNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigCatTextNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.RarityPullDown then
				button.Text:SetText(itemRarity[i].color..itemRarity[i].name.."|r")
				button.value = i - 2
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownRaritySFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownRaritySFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownRarityNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownRarityNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.Filter1PullDown then
				button.Text:SetText(Filter1Table[i].name)
				button.value = Filter1Table[i].index
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.Filter2PullDown then
				button.Text:SetText(Filter2Table[i].name)
				button.value = Filter2Table[i].index
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.Filter3PullDown then
				button.Text:SetText(Filter3Table[i].name)
				button.value = Filter3Table[i].index
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
				end
			elseif frameName == GVAR.ConfigFrame.SearchQueries.SortByPullDown then
				button.Text:SetText(L[ SortByTable[i] ])
				button.value = SortByTable[i]
				if scrollFrameIsShown then
					button:SetWidth(sizeGeneral.ConfigPullDownSortSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownSortSFWidth)
				else
					button:SetWidth(sizeGeneral.ConfigPullDownSortNoSFWidth)
					button.Text:SetWidth(sizeGeneral.ConfigPullDownSortNoSFWidth)
				end
			end
			-- frameName dependent END ----------
			button:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:InitFrames()
	CatSizeUpdate(AHsearch_Options.curCatWidth)
	QuerySizeUpdate(AHsearch_Options.curQueryWidth)
	AHsearch:SetupMainScale(AHsearch_Options.scaleMain)
	AHsearch:ConfigFramePos()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CatFrame_Show(force)
	AHsearch:FillList(GVAR.CatFrame)

	local height = GUI_CatFrame_Count
	if GUI_CatFrame_Count > AHsearch_Options.curCat then
		height = AHsearch_Options.curCat
	end

	if GUI_CatFrame_Count == 0 then
		GVAR.CatFrame.NoCatText:Show()
		GVAR.CatFrame:SetHeight(150)
		GVAR.ConfigFrame.Categories.SelectCatTitle:SetTextColor(0.5, 0.5, 0.5)
		GVAR.ConfigFrame.Categories.SelectCat.FrameBorder:SetTexture(0.4, 0.4, 0.4, 1)
	else
		GVAR.CatFrame.NoCatText:Hide()
		GVAR.CatFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)
		GVAR.ConfigFrame.Categories.SelectCatTitle:SetTextColor(1, 0.82, 0)
		GVAR.ConfigFrame.Categories.SelectCat.FrameBorder:SetTexture(0.8, 0.2, 0.2, 1)
	end

	GVAR.CatFrame:SetPoint("TOPLEFT", GVAR.MainFrame, "BOTTOMLEFT", -(20/(AHsearch_Options.scaleMain/100)), -(12/(AHsearch_Options.scaleMain/100)))

	if configMode and AHsearch_Options.hideMain then return end

	if not force then
		if GVAR.CatFrame:IsShown() then
			GVAR.CatFrame:Hide()
		else
			GVAR.CatFrame:Show()
		end
	else
		GVAR.CatFrame:Show()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:QueryFrame_Show(cat)
	if cat then
		GUI_curCat = cat -- need check
	end

	if not cat or SearchQueryCheck(cat) then
		if not configMode then
			for i = 1, maxCat do
				GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
			end
		end
		GVAR.QueryFrame:Hide()

		for i = 1, maxQuery do
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i]:Hide()
		end
		GVAR.ConfigFrame.SearchQueries.SelectSQ:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetTextColor(0.5, 0.5, 0.5)

		return
	end

	GUI_curCat = cat
	AHsearch:FillList(GVAR.QueryFrame)

	GVAR.ConfigFrame.SearchQueries.SelectSQ:Show()
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetTextColor(1, 0.82, 0)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)

	local height = GUI_SQFrame_Count[GUI_curCat]
	if GUI_SQFrame_Count[GUI_curCat] > AHsearch_Options.curQuery then
		height = AHsearch_Options.curQuery
	end
	GVAR.QueryFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)

	-- sq posi calc START VODOO
	local dataNum = 1
	for i = 1, GUI_CatFrame_Count do
		if GUI_CatFrame_data[i].Cat == cat then
			dataNum = i
		end
	end
	local scrollval = GVAR.CatFrame.ScrollFrame.ScrollBar:GetValue()
	GVAR.QueryFrame:SetPoint("TOP", GVAR.CatFrame.ScrollFrame.Button[dataNum], "TOP", 0, sizeGeneral.Offset+(scrollval) )
	-- sq posi calc END

	GVAR.QueryFrame:SetPoint("LEFT", GVAR.CatFrame, "RIGHT", -sizeGeneral.Border, 0)

	if configMode and AHsearch_Options.hideMain then return end
	GVAR.QueryFrame:Show()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CatFrame_ButtonEventUpdate(cat, RightClickConfig)
	if configMode then
		if confirmIsShown then return end

		for i = 1, maxCat do
			GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
			GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
			if GVAR.CatFrame.ScrollFrame.Button[i].cat == cat then
				GVAR.CatFrame.ScrollFrame.Button[i].Selected:Show()
				GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Show()
			end
			GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Hide()
			GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
			if GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].cat == cat then
				GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Show()
				GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Show()
			end
			GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Hide()
			GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
			if GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].cat == cat then
				GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Show()
				GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Show()
			end
		end

		for i = 1, maxQuery do
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Hide()
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Hide()
		end

		AHsearch:QueryFrame_Show(cat)
		AHsearch:ConfigurationFrame_Categories_SelectCat(cat)
		AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

		if RightClickConfig then
			GUI_Tab = 1
			AHsearch:ConfigurationFrame_ShowContent()
		end
		if AHsearch_Options.hideMain then
			GVAR.CatFrame:Hide()
		end
	else
		for i = 1, maxCat do
			GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
			if GVAR.CatFrame.ScrollFrame.Button[i].cat == cat then
				GVAR.CatFrame.ScrollFrame.Button[i].Selected:Show()
			end
		end
		AHsearch:QueryFrame_Show(cat)
		VisibilityCheck()
	end
end

function AHsearch:QueryFrame_ButtonEventUpdate(cat, sq, RightClickConfig, TabChange)
	if configMode then
		if confirmIsShown then return end
		GUI_curSQ = sq
		for i = 1, maxQuery do
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Hide()
			GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Hide()
			if GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].sq == sq then
				GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Show()
				GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Show()
			end
			GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Hide()
			GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
			if GVAR.QueryFrame.ScrollFrame.Button[i].sq == sq then
				GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Show()
				GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Show()
			end
		end
		if RightClickConfig then
			AHsearch:QueryFrame_Show(cat)
			AHsearch:ConfigurationFrame_Categories_SelectCat(cat)
			AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

			for i = 1, maxCat do
				GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
				GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
				if GVAR.CatFrame.ScrollFrame.Button[i].cat == cat then
					GVAR.CatFrame.ScrollFrame.Button[i].Selected:Show()
					GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Show()
				end
				GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Hide()
				GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
				if GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].cat == cat then
					GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Show()
					GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Show()
				end
				GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Hide()
				GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
				if GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].cat == cat then
					GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Show()
					GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Show()
				end
			end

			for i = 1, maxQuery do
				if GVAR.QueryFrame.ScrollFrame.Button[i].sq == sq then
					GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Show()
					GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Show()
				end
				if GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].sq == sq then
					GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Show()
					GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Show()
				end
			end
		end
		if TabChange then
			GUI_Tab = 2
			AHsearch:ConfigurationFrame_ShowContent()
		end
		AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, sq)
		if AHsearch_Options.hideMain then
			GUI_curSQ = sq
			GVAR.CatFrame:Hide()
		end
	else
		VisibilityCheck()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CatFrame_CreateFrame()
	GVAR.CatFrame = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.CatFrame)
	GVAR.CatFrame:SetWidth(AHsearch_Options.curCatWidth)
	GVAR.CatFrame:Hide()
	GVAR.CatFrame:EnableMouse(true)
	GVAR.CatFrame:SetToplevel(true)
	GVAR.CatFrame:SetScript("OnLeave", VisibilityCheck)
	GVAR.CatFrame:SetScript("OnEnter", VisibilityCheck)

	GVAR.CatFrame.NoCatText = GVAR.CatFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.CatFrame.NoCatText:SetWidth(125)
	GVAR.CatFrame.NoCatText:SetHeight(150)
	GVAR.CatFrame.NoCatText:SetPoint("CENTER", 0, 0)
	GVAR.CatFrame.NoCatText:SetJustifyH("CENTER")
	GVAR.CatFrame.NoCatText:SetText(L["No category available!"].."\n\n"..L["Right-click AHsearch Button for configuration!"])

	GVAR.CatFrame.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.CatFrame)
	TEMPLATE.ScrollFrame(GVAR.CatFrame.ScrollFrame)
	GVAR.CatFrame.ScrollFrame:SetPoint("TOPLEFT", 0, 0)
	GVAR.CatFrame.ScrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.CatFrame.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(GVAR.CatFrame, value) end)
	GVAR.CatFrame.ScrollFrame.ScrollBar:SetScript("OnEnter", VisibilityCheck)
	GVAR.CatFrame.ScrollFrame.ScrollBar:SetScript("OnLeave", VisibilityCheck)
	GVAR.CatFrame.ScrollFrame.ScrollBarScrollUpButton:SetScript("OnEnter", VisibilityCheck)
	GVAR.CatFrame.ScrollFrame.ScrollBarScrollUpButton:SetScript("OnLeave", VisibilityCheck)
	GVAR.CatFrame.ScrollFrame.ScrollBarScrollDownButton:SetScript("OnEnter", VisibilityCheck)
	GVAR.CatFrame.ScrollFrame.ScrollBarScrollDownButton:SetScript("OnLeave", VisibilityCheck)

	GVAR.CatFrame.ScrollFrame.Button = {}
	for i = 1, maxCat do
		GVAR.CatFrame.ScrollFrame.Button[i] = CreateFrame("Button", nil, GVAR.CatFrame.ScrollFrame)
		GVAR.CatFrame.ScrollFrame.Button[i]:SetWidth(sizeGeneral.CatButtonSFWidth)
		GVAR.CatFrame.ScrollFrame.Button[i]:SetHeight(sizeGeneral.BarHeight)
		GVAR.CatFrame.ScrollFrame.Button[i]:SetFrameLevel( GVAR.CatFrame.ScrollFrame:GetFrameLevel() + 5 )
		GVAR.CatFrame.ScrollFrame.Button[i]:Hide()
		if i == 1 then
			GVAR.CatFrame.ScrollFrame.Button[i]:SetPoint("TOPLEFT", GVAR.CatFrame.ScrollFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			GVAR.CatFrame.ScrollFrame.Button[i]:SetPoint("TOPLEFT", GVAR.CatFrame.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		GVAR.CatFrame.ScrollFrame.Button[i].Texture = GVAR.CatFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.CatFrame.ScrollFrame.Button[i].Texture:SetWidth(sizeGeneral.BarHeight)
		GVAR.CatFrame.ScrollFrame.Button[i].Texture:SetHeight(sizeGeneral.BarHeight)
		GVAR.CatFrame.ScrollFrame.Button[i].Texture:SetPoint("LEFT", 0, 0)

		GVAR.CatFrame.ScrollFrame.Button[i].Text = GVAR.CatFrame.ScrollFrame.Button[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.CatFrame.ScrollFrame.Button[i].Text:SetWidth(sizeGeneral.CatTextSFWidth)
		GVAR.CatFrame.ScrollFrame.Button[i].Text:SetHeight(sizeGeneral.BarHeight)
		GVAR.CatFrame.ScrollFrame.Button[i].Text:SetPoint("LEFT", sizeGeneral.BarHeight+sizeGeneral.Space, 0)
		GVAR.CatFrame.ScrollFrame.Button[i].Text:SetJustifyH("LEFT")

		GVAR.CatFrame.ScrollFrame.Button[i].Selected = GVAR.CatFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:SetPoint("TOPLEFT", GVAR.CatFrame.ScrollFrame.Button[i], "TOPLEFT", 0, 0)
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:SetPoint("BOTTOMRIGHT", GVAR.CatFrame.ScrollFrame.Button[i], "BOTTOMRIGHT", 0, 0)
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0.1)
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow = GVAR.CatFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:SetWidth(Textures.SelectArrow.width)
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:SetHeight(Textures.SelectArrow.height)
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Hide()

		GVAR.CatFrame.ScrollFrame.Button[i]:RegisterForClicks("AnyUp")
		GVAR.CatFrame.ScrollFrame.Button[i]:SetScript("OnClick", function(self, button)
			if button == "RightButton" then
				if not configMode then
					AHsearch:ConfigEnable()
				end
				AHsearch:CatFrame_ButtonEventUpdate(self.cat, 1)
			end
		end)
		GVAR.CatFrame.ScrollFrame.Button[i]:SetScript("OnEnter", function(self) AHsearch:CatFrame_ButtonEventUpdate(self.cat) end)
		GVAR.CatFrame.ScrollFrame.Button[i]:SetScript("OnLeave", VisibilityCheck)

		GVAR.CatFrame.ScrollFrame.Button[i].Highlight = GVAR.CatFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.CatFrame.ScrollFrame.Button[i].Highlight:SetPoint("TOPLEFT", GVAR.CatFrame.ScrollFrame.Button[i], "TOPLEFT", 0, 0)
		GVAR.CatFrame.ScrollFrame.Button[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.CatFrame.ScrollFrame.Button[i], "BOTTOMRIGHT", 0, 0)
		GVAR.CatFrame.ScrollFrame.Button[i].Highlight:SetTexture(1, 1, 1, 0.2)
		GVAR.CatFrame.ScrollFrame.Button[i]:SetHighlightTexture(GVAR.CatFrame.ScrollFrame.Button[i].Highlight)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:QueryFrame_CreateFrame()
	GVAR.QueryFrame = CreateFrame("Frame", nil, GVAR.CatFrame)
	TEMPLATE.BorderTRBL(GVAR.QueryFrame)
	GVAR.QueryFrame:SetWidth(AHsearch_Options.curQueryWidth)
	GVAR.QueryFrame:SetClampedToScreen(true)
	GVAR.QueryFrame:Hide()
	GVAR.QueryFrame:EnableMouse(true)
	GVAR.QueryFrame:SetToplevel(true)
	GVAR.QueryFrame:SetScript("OnLeave", VisibilityCheck)
	GVAR.QueryFrame:SetScript("OnEnter", VisibilityCheck)

	GVAR.QueryFrame.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.QueryFrame)
	TEMPLATE.ScrollFrame(GVAR.QueryFrame.ScrollFrame)
	GVAR.QueryFrame.ScrollFrame:SetPoint("TOPLEFT", 0, 0)
	GVAR.QueryFrame.ScrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.QueryFrame.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(GVAR.QueryFrame, value) end)
	GVAR.QueryFrame.ScrollFrame.ScrollBar:SetScript("OnEnter", VisibilityCheck)
	GVAR.QueryFrame.ScrollFrame.ScrollBar:SetScript("OnLeave", VisibilityCheck)
	GVAR.QueryFrame.ScrollFrame.ScrollBarScrollUpButton:SetScript("OnEnter", VisibilityCheck)
	GVAR.QueryFrame.ScrollFrame.ScrollBarScrollUpButton:SetScript("OnLeave", VisibilityCheck)
	GVAR.QueryFrame.ScrollFrame.ScrollBarScrollDownButton:SetScript("OnEnter", VisibilityCheck)
	GVAR.QueryFrame.ScrollFrame.ScrollBarScrollDownButton:SetScript("OnLeave", VisibilityCheck)

	GVAR.QueryFrame.ScrollFrame.Button = {}
	for i = 1, maxQuery do
		GVAR.QueryFrame.ScrollFrame.Button[i] = CreateFrame("Button", nil, GVAR.QueryFrame)
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetWidth(sizeGeneral.QueryButtonSFWidth)
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetHeight(sizeGeneral.BarHeight)
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetFrameLevel( GVAR.QueryFrame.ScrollFrame:GetFrameLevel() + 5 )
		GVAR.QueryFrame.ScrollFrame.Button[i]:Hide()
		if i == 1 then
			GVAR.QueryFrame.ScrollFrame.Button[i]:SetPoint("TOPLEFT", GVAR.QueryFrame.ScrollFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			GVAR.QueryFrame.ScrollFrame.Button[i]:SetPoint("TOPLEFT", GVAR.QueryFrame.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		GVAR.QueryFrame.ScrollFrame.Button[i].Text = GVAR.QueryFrame.ScrollFrame.Button[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.QueryFrame.ScrollFrame.Button[i].Text:SetWidth(sizeGeneral.QueryTextSFWidth)
		GVAR.QueryFrame.ScrollFrame.Button[i].Text:SetHeight(sizeGeneral.BarHeight)
		GVAR.QueryFrame.ScrollFrame.Button[i].Text:SetPoint("LEFT", 0, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].Text:SetJustifyH("LEFT")

		GVAR.QueryFrame.ScrollFrame.Button[i].Selected = GVAR.QueryFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:SetPoint("TOPLEFT", GVAR.QueryFrame.ScrollFrame.Button[i], "TOPLEFT", 0, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:SetPoint("BOTTOMRIGHT", GVAR.QueryFrame.ScrollFrame.Button[i], "BOTTOMRIGHT", 0, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0.1)
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow = GVAR.QueryFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:SetWidth(Textures.SelectArrow.width)
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:SetHeight(Textures.SelectArrow.height)
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Hide()

		GVAR.QueryFrame.ScrollFrame.Button[i]:RegisterForClicks("AnyUp")
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetScript("OnClick", function(self, button)
			if button == "LeftButton" then
				if configMode then return end
				AHsearch:Search(self.cat, self.sq)
			elseif button == "RightButton" then
				if configMode then
					AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq, nil, 1)
				else
					AHsearch:ConfigEnable()
					AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq, 1, 1)
				end
			end
		end)
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetScript("OnEnter", function(self) AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq) end)
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetScript("OnLeave", VisibilityCheck)

		GVAR.QueryFrame.ScrollFrame.Button[i].Highlight = GVAR.QueryFrame.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		GVAR.QueryFrame.ScrollFrame.Button[i].Highlight:SetPoint("TOPLEFT", GVAR.QueryFrame.ScrollFrame.Button[i], "TOPLEFT", 0, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.QueryFrame.ScrollFrame.Button[i], "BOTTOMRIGHT", 0, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].Highlight:SetTexture(1, 1, 1, 0.2)
		GVAR.QueryFrame.ScrollFrame.Button[i]:SetHighlightTexture(GVAR.QueryFrame.ScrollFrame.Button[i].Highlight)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_CreateFrame()
	local mainWidth = 485 -- 425 + 60 = 485
	local mainHeight = 637
	local topheight = 87-24-10

	GVAR.ConfigFrame = CreateFrame("Frame", nil, UIParent)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame)
	GVAR.ConfigFrame:SetWidth(mainWidth)
	GVAR.ConfigFrame:SetHeight(mainHeight)
	GVAR.ConfigFrame:SetMovable(true)
	GVAR.ConfigFrame:SetToplevel(true)
	GVAR.ConfigFrame:SetClampedToScreen(true)
	GVAR.ConfigFrame:SetClampRectInsets((mainWidth-50)/2, -((mainWidth-50)/2), -(mainHeight-topheight), mainHeight-topheight)
	GVAR.ConfigFrame:EnableMouse(true)
	GVAR.ConfigFrame:EnableMouseWheel(true)
	GVAR.ConfigFrame:SetScript("OnMouseWheel", NOOP)
	GVAR.ConfigFrame:Hide()



	-- tab START ----------------------------------------
	GVAR.ConfigFrame.Selection = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.Selection)
	GVAR.ConfigFrame.Selection:SetWidth(mainWidth)
	GVAR.ConfigFrame.Selection:SetHeight(topheight)
	GVAR.ConfigFrame.Selection:SetPoint("TOPLEFT", GVAR.ConfigFrame, "TOPLEFT", 0, 0)
	GVAR.ConfigFrame.Selection:EnableMouse(true)
	GVAR.ConfigFrame.Selection:SetToplevel(true)
	GVAR.ConfigFrame.Selection:SetScript("OnMouseDown", function()
		GVAR.ConfigFrame:StartMoving() GVAR.ConfigFrame:StopMovingOrSizing()
	end)

	GVAR.ConfigFrame.Selection.SearchQueryInfo = CreateFrame("Button", nil, GVAR.ConfigFrame.Selection)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:EnableMouse(true)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:SetWidth(18)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:SetHeight(18)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:SetPoint("LEFT", GVAR.ConfigFrame.Selection, "LEFT", 10, 8)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:Hide()
	GVAR.ConfigFrame.Selection.SearchQueryInfoTexture = GVAR.ConfigFrame.Selection.SearchQueryInfo:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.Selection.SearchQueryInfoTexture:SetWidth(18)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTexture:SetHeight(18)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTexture:SetPoint("LEFT", 0, 0)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTexture:SetTexture("Interface\\FriendsFrame\\InformationIcon")
	GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame = CreateFrame("Frame", nil, GVAR.ConfigFrame.Selection.SearchQueryInfo)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame, 1)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:SetWidth(280)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:SetToplevel(true)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:SetPoint("BOTTOM", GVAR.ConfigFrame.Selection.SearchQueryInfo, "TOP", 0, 0)
	GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:Hide()
	GVAR.ConfigFrame.Selection.SearchQueryInfoText = GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Selection.SearchQueryInfoText:SetWidth(260)
	GVAR.ConfigFrame.Selection.SearchQueryInfoText:SetPoint("CENTER", 0, 0)
	GVAR.ConfigFrame.Selection.SearchQueryInfoText:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Selection.SearchQueryInfoText:SetText(L["An |cffffff7fAlt-Click|r on a |cffffff7fbag item|r or an |cffffff7fauction item|r starts a new 'Search Query' in the selected 'Category' and automatically fills out some fields."])
	GVAR.ConfigFrame.Selection.SearchQueryInfoText:SetTextColor(1, 1, 1, 1)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:SetScript("OnEnter", function() GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:Show() end)
	GVAR.ConfigFrame.Selection.SearchQueryInfo:SetScript("OnLeave", function() GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:Hide() end)
	local txtHeight = GVAR.ConfigFrame.Selection.SearchQueryInfoText:GetStringHeight()
	GVAR.ConfigFrame.Selection.SearchQueryInfoTextFrame:SetHeight(txtHeight+30)
	GVAR.ConfigFrame.Selection.SearchQueryInfoText:SetHeight(txtHeight+10)

	GVAR.ConfigFrame.CloseConfig = CreateFrame("Button", nil, GVAR.ConfigFrame.Selection)
	TEMPLATE.TextButton(GVAR.ConfigFrame.CloseConfig, L["Close Configuration"], 1)
	GVAR.ConfigFrame.CloseConfig:SetWidth(300)
	GVAR.ConfigFrame.CloseConfig:SetHeight(24)
	GVAR.ConfigFrame.CloseConfig:SetPoint("BOTTOM", GVAR.ConfigFrame, "BOTTOM", 0, 10)
	GVAR.ConfigFrame.CloseConfig:SetScript("OnClick", function() AHsearch:ConfigDisable() end)

	GVAR.ConfigFrame.HideCatSQ = CreateFrame("Button", nil, GVAR.ConfigFrame.Selection)
	TEMPLATE.TextButton(GVAR.ConfigFrame.HideCatSQ, L["HIDE Categories / Search Queries"], 3)
	GVAR.ConfigFrame.HideCatSQ:SetWidth(250)
	GVAR.ConfigFrame.HideCatSQ:SetHeight(18)
	GVAR.ConfigFrame.HideCatSQ:SetPoint("TOP", GVAR.ConfigFrame.Selection, "TOP", 0, -8)
	GVAR.ConfigFrame.HideCatSQ:SetScript("OnMouseDown", function()
		GVAR.ConfigFrame:StartMoving() GVAR.ConfigFrame:StopMovingOrSizing()
	end)
	GVAR.ConfigFrame.HideCatSQ:SetScript("OnClick", function()
		if GVAR.CatFrame:IsShown() then
			AHsearch_Options.hideMain = true
			GVAR.CatFrame:Hide()
			GVAR.QueryFrame:Hide()
			GVAR.ConfigFrame.HideCatSQ:SetText(L["SHOW Categories / Search Queries"])
			GVAR.ConfigFrame.AlertIconL_HideCatSQ:Show()
			GVAR.ConfigFrame.AlertIconR_HideCatSQ:Show()
		else
			AHsearch_Options.hideMain = false
			GVAR.CatFrame:Show()
			if GUI_curCat then
				if SearchQueryCheck(GUI_curCat) then
					GVAR.QueryFrame:Hide()
				else
					GVAR.QueryFrame:Show()
				end
			else
				GVAR.QueryFrame:Hide()
			end
			GVAR.ConfigFrame.HideCatSQ:SetText(L["HIDE Categories / Search Queries"])
			GVAR.ConfigFrame.AlertIconL_HideCatSQ:Hide()
			GVAR.ConfigFrame.AlertIconR_HideCatSQ:Hide()
		end
	end)

	GVAR.ConfigFrame.AlertIconL_HideCatSQ = GVAR.ConfigFrame.Selection:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.AlertIconL_HideCatSQ:SetHeight(20)
	GVAR.ConfigFrame.AlertIconL_HideCatSQ:SetWidth(20)
	GVAR.ConfigFrame.AlertIconL_HideCatSQ:SetPoint("RIGHT", GVAR.ConfigFrame.HideCatSQ, "LEFT", -5, -2)
	GVAR.ConfigFrame.AlertIconL_HideCatSQ:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")

	GVAR.ConfigFrame.AlertIconR_HideCatSQ = GVAR.ConfigFrame.Selection:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.AlertIconR_HideCatSQ:SetHeight(20)
	GVAR.ConfigFrame.AlertIconR_HideCatSQ:SetWidth(20)
	GVAR.ConfigFrame.AlertIconR_HideCatSQ:SetPoint("LEFT", GVAR.ConfigFrame.HideCatSQ, "RIGHT", 5, -2)
	GVAR.ConfigFrame.AlertIconR_HideCatSQ:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")

	if AHsearch_Options.hideMain then
		GVAR.ConfigFrame.HideCatSQ:SetText(L["SHOW Categories / Search Queries"])
		GVAR.ConfigFrame.AlertIconL_HideCatSQ:Show()
		GVAR.ConfigFrame.AlertIconR_HideCatSQ:Show()
	else
		GVAR.ConfigFrame.HideCatSQ:SetText(L["HIDE Categories / Search Queries"])
		GVAR.ConfigFrame.AlertIconL_HideCatSQ:Hide()
		GVAR.ConfigFrame.AlertIconR_HideCatSQ:Hide()
	end

	local tabwidth = (mainWidth-(4*10))/3

	GVAR.ConfigFrame.TabCategories = CreateFrame("Button", nil, GVAR.ConfigFrame.Selection)
	TEMPLATE.TabButton(GVAR.ConfigFrame.TabCategories, L["Categories"], 1, tabwidth)
	GVAR.ConfigFrame.TabCategories:SetWidth(tabwidth)
	GVAR.ConfigFrame.TabCategories:SetHeight(20)
	GVAR.ConfigFrame.TabCategories:SetPoint("BOTTOMLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 10, -1)
	GVAR.ConfigFrame.TabCategories:SetScript("OnMouseDown", function()
		GVAR.ConfigFrame:StartMoving() GVAR.ConfigFrame:StopMovingOrSizing()
	end)
	GVAR.ConfigFrame.TabCategories:SetScript("OnClick", function()
		if GUI_Tab == 1 then return end
		GUI_Tab = 1
		AHsearch:ConfigurationFrame_ShowContent()
	end)

	GVAR.ConfigFrame.TabSearchQueries = CreateFrame("Button", nil, GVAR.ConfigFrame.Selection)
	TEMPLATE.TabButton(GVAR.ConfigFrame.TabSearchQueries, L["Search Queries"], 1, tabwidth)
	GVAR.ConfigFrame.TabSearchQueries:SetWidth(tabwidth)
	GVAR.ConfigFrame.TabSearchQueries:SetHeight(20)
	GVAR.ConfigFrame.TabSearchQueries:SetPoint("BOTTOMLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 10+tabwidth+10, -1)
	GVAR.ConfigFrame.TabSearchQueries:SetScript("OnMouseDown", function()
		GVAR.ConfigFrame:StartMoving() GVAR.ConfigFrame:StopMovingOrSizing()
	end)
	GVAR.ConfigFrame.TabSearchQueries:SetScript("OnClick", function()
		if GUI_Tab == 2 then return end
		GUI_Tab = 2
		AHsearch:ConfigurationFrame_ShowContent()
	end)

	GVAR.ConfigFrame.TabOptions = CreateFrame("Button", nil, GVAR.ConfigFrame.Selection)
	TEMPLATE.TabButton(GVAR.ConfigFrame.TabOptions, L["Options"], 1, tabwidth)
	GVAR.ConfigFrame.TabOptions:SetWidth(tabwidth)
	GVAR.ConfigFrame.TabOptions:SetHeight(20)
	GVAR.ConfigFrame.TabOptions:SetPoint("BOTTOMLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 10+tabwidth+10+tabwidth+10, -1)
	GVAR.ConfigFrame.TabOptions:SetScript("OnMouseDown", function()
		GVAR.ConfigFrame:StartMoving() GVAR.ConfigFrame:StopMovingOrSizing()
	end)
	GVAR.ConfigFrame.TabOptions:SetScript("OnClick", function()
		if GUI_Tab == 3 then return end
		GUI_Tab = 3
		AHsearch:ConfigurationFrame_ShowContent()
	end)
	-- tab END ----------------------------------------



	-- Categories START ----------------------------------------
	GVAR.ConfigFrame.Categories = {}
	GVAR.ConfigFrame.Categories.Content = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.Categories.Content)
	GVAR.ConfigFrame.Categories.Content:SetWidth(mainWidth)
	GVAR.ConfigFrame.Categories.Content:SetHeight(mainHeight-topheight+sizeGeneral.Border)
	GVAR.ConfigFrame.Categories.Content:SetPoint("TOPLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 0, 1)
	GVAR.ConfigFrame.Categories.Content:EnableMouse(true)
	GVAR.ConfigFrame.Categories.Content:Hide()

	-- - Categories create new cat START
	GVAR.ConfigFrame.Categories.NewCatTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.NewCatTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.NewCatTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.Content, "TOPLEFT", 10, -10)
	GVAR.ConfigFrame.Categories.NewCatTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.NewCatTitle:SetText(L["Create New"]..":")

	GVAR.ConfigFrame.Categories.NewCat = CreateFrame("EditBox", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.Categories.NewCat, sizeGeneral.ConfigTextWidth)
	GVAR.ConfigFrame.Categories.NewCat:SetWidth(sizeGeneral.ConfigTextWidth)
	GVAR.ConfigFrame.Categories.NewCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NewCatTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.Categories.NewCat:SetMaxLetters(maxBoxInputLength)
	GVAR.ConfigFrame.Categories.NewCat:SetScript("OnTextChanged", function(self)
		local text = self:GetText()
		if text ~= "" then
			GVAR.ConfigFrame.TabCategories.editType = "new"
		end
		ConfigurationFrame_Categories_SetNewCat(text)
	end)
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.Categories.NewCat)

	GVAR.ConfigFrame.Categories.NewCatNote = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.NewCatNote:SetHeight(20)
	GVAR.ConfigFrame.Categories.NewCatNote:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NewCat, "RIGHT", 5, 0)
	GVAR.ConfigFrame.Categories.NewCatNote:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.NewCatNote:SetText(L["*NEW"].." ("..L["Title"]..")")
	GVAR.ConfigFrame.Categories.NewCatNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - Categories create new cat END

	-- - Categories select cat START
	GVAR.ConfigFrame.Categories.SelectCatTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.SelectCatTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.SelectCatTitle:SetPoint("TOP", GVAR.ConfigFrame.Categories.NewCat, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.Categories.SelectCatTitle:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NewCatTitle, "LEFT", 0, 0)
	GVAR.ConfigFrame.Categories.SelectCatTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.SelectCatTitle:SetText(L["Select"]..":")

	GVAR.ConfigFrame.Categories.SelectCat = CreateFrame("Frame", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.Categories.SelectCat, 1)
	GVAR.ConfigFrame.Categories.SelectCat:SetWidth(sizeGeneral.ConfigCatWidth)
	GVAR.ConfigFrame.Categories.SelectCat:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	GVAR.ConfigFrame.Categories.SelectCat:SetPoint("TOP", GVAR.ConfigFrame.Categories.NewCat, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.Categories.SelectCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.SelectCatTitle, "RIGHT", 10, 0)

	GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.ConfigFrame.Categories.SelectCat)
	TEMPLATE.ScrollFrame(GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame)
	GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame:SetPoint("TOPLEFT", 0, 0)
	GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(GVAR.ConfigFrame.Categories.SelectCat, value) end)

	GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button = {}
	for i = 1, maxCat do
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i] = CreateFrame("Button", nil, GVAR.ConfigFrame.Categories.SelectCat)
		local button = GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i]
		button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		button.Texture = button:CreateTexture(nil, "ARTWORK")
		button.Texture:SetWidth(sizeGeneral.BarHeight)
		button.Texture:SetHeight(sizeGeneral.BarHeight)
		button.Texture:SetPoint("LEFT", 0, 0)

		button.Text = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.Text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
		button.Text:SetHeight(sizeGeneral.BarHeight)
		button.Text:SetPoint("LEFT", sizeGeneral.BarHeight+sizeGeneral.Space, 0)
		button.Text:SetJustifyH("LEFT")

		button.Selected = button:CreateTexture(nil, "ARTWORK")
		button.Selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Selected:SetTexture(1, 1, 1, 0.1)
		button.Selected:Hide()
		button.SelectedArrow = button:CreateTexture(nil, "ARTWORK")
		button.SelectedArrow:SetWidth(Textures.SelectArrow.width)
		button.SelectedArrow:SetHeight(Textures.SelectArrow.height)
		button.SelectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		button.SelectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		button.SelectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		button.SelectedArrow:Hide()

		button.Order = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.Order:SetWidth(30)
		button.Order:SetHeight(sizeGeneral.BarHeight)
		button.Order:SetPoint("RIGHT", button, "LEFT", -(sizeGeneral.Offset+sizeGeneral.Space), 0)
		button.Order:SetJustifyH("RIGHT")
		button.Order:SetTextColor(0, 0.75, 0) -- ORDER_COLOR

		button:SetScript("OnClick", function(self) AHsearch:CatFrame_ButtonEventUpdate(self.cat) end)

		button.Highlight = button:CreateTexture(nil, "ARTWORK")
		button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(button.Highlight)
	end
	AHsearch:FillList(GVAR.ConfigFrame.Categories.SelectCat)
	-- - Categories select cat END

	-- - Categories current cat START
	GVAR.ConfigFrame.Categories.CurrentCatTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetPoint("TOP", GVAR.ConfigFrame.Categories.SelectCat, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetPoint("LEFT", GVAR.ConfigFrame.Categories.SelectCatTitle, "LEFT", 0, 0)
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetText(L["Current"]..":")
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.Categories.CurrentCatTexture = GVAR.ConfigFrame.Categories.Content:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetWidth(sizeGeneral.BarHeight)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetHeight(sizeGeneral.BarHeight)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetPoint("LEFT", GVAR.ConfigFrame.Categories.CurrentCatTitle, "LEFT", 20, 0)

	GVAR.ConfigFrame.Categories.CurrentCatTxt = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetHeight(20)
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetPoint("LEFT", GVAR.ConfigFrame.Categories.CurrentCatTexture, "RIGHT", 2, 0)
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetText("")
	-- - Categories current cat END

	-- - Categories name START
	GVAR.ConfigFrame.Categories.NameCatTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.NameCatTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.NameCatTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.CurrentCatTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.Categories.NameCatTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.NameCatTitle:SetText(L["Title"]..":") -- formerly (L["Name"]..":")
	GVAR.ConfigFrame.Categories.NameCatTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.Categories.NameCat = CreateFrame("EditBox", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.Categories.NameCat)
	GVAR.ConfigFrame.Categories.NameCat:SetWidth(sizeGeneral.ConfigTextWidth)
	GVAR.ConfigFrame.Categories.NameCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NameCatTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.Categories.NameCat:SetMaxLetters(maxBoxInputLength)
	GVAR.ConfigFrame.Categories.NameCat:SetScript("OnTextChanged", function(self)
		local text = self:GetText()
		ConfigurationFrame_Categories_SetName(text)
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.Categories.NameCat)

	GVAR.ConfigFrame.Categories.NameCatNote = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.NameCatNote:SetHeight(20)
	GVAR.ConfigFrame.Categories.NameCatNote:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NameCat, "RIGHT", 5, 0)
	GVAR.ConfigFrame.Categories.NameCatNote:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.NameCatNote:SetText(L["*Required"])
	GVAR.ConfigFrame.Categories.NameCatNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - Categories name END

	-- - Categories order START
	GVAR.ConfigFrame.Categories.OrderTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.OrderTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.OrderTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.NameCatTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.Categories.OrderTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.OrderTitle:SetText(L["Position"]..":")
	GVAR.ConfigFrame.Categories.OrderTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.Categories.Order = CreateFrame("EditBox", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.Categories.Order)
	GVAR.ConfigFrame.Categories.Order:SetWidth(40)
	GVAR.ConfigFrame.Categories.Order:SetPoint("LEFT", GVAR.ConfigFrame.Categories.OrderTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.Categories.Order:SetNumeric(true)
	GVAR.ConfigFrame.Categories.Order:SetMaxLetters(2)
	GVAR.ConfigFrame.Categories.Order:SetTextColor(0, 0.75, 0) -- ORDER_COLOR
	GVAR.ConfigFrame.Categories.Order:SetScript("OnTextChanged", function(self)
		local number = GVAR.ConfigFrame.Categories.Order:GetNumber()
		if number == 0 then
			self:SetNumber(1)
			number = 1
		end
		GVAR.ConfigFrame.Categories.Order.value = number
		ConfigurationFrame_Categories_SetOrder(number)
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.Categories.Order)
	-- - Categories order END

	-- - Categories color START
	GVAR.ConfigFrame.Categories.ColorTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.ColorTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.ColorTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.OrderTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.Categories.ColorTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.ColorTitle:SetText(L["Color"]..":")
	GVAR.ConfigFrame.Categories.ColorTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.Categories.ColorPullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.Categories.ColorPullDown, "", sizeGeneral.ConfigPullDownColorWidth, categoryColors, false, ConfigurationFrame_Categories_SetColor)
	GVAR.ConfigFrame.Categories.ColorPullDown:SetWidth(sizeGeneral.ConfigPullDownColorWidth)
	GVAR.ConfigFrame.Categories.ColorPullDown:SetHeight(18)
	GVAR.ConfigFrame.Categories.ColorPullDown:SetPoint("LEFT", GVAR.ConfigFrame.Categories.ColorTitle, "RIGHT", 10, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.Categories.ColorPullDown)
	-- - Categories color END

	-- - Categories icon START
	GVAR.ConfigFrame.Categories.IconTitle = GVAR.ConfigFrame.Categories.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Categories.IconTitle:SetHeight(20)
	GVAR.ConfigFrame.Categories.IconTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.Categories.ColorTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.Categories.IconTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Categories.IconTitle:SetText(L["Icon"]..":")
	GVAR.ConfigFrame.Categories.IconTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.Categories.IconPullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.Categories.IconPullDown, "", sizeGeneral.ConfigPullDownIconWidth, categoryIcons, 1, ConfigurationFrame_Categories_SetIcon)
	GVAR.ConfigFrame.Categories.IconPullDown:SetWidth(sizeGeneral.ConfigPullDownIconWidth)
	GVAR.ConfigFrame.Categories.IconPullDown:SetHeight(18)
	GVAR.ConfigFrame.Categories.IconPullDown:SetPoint("LEFT", GVAR.ConfigFrame.Categories.IconTitle, "RIGHT", 10, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.Categories.IconPullDown)
	-- - Categories icon END

	-- - Categories update START
	GVAR.ConfigFrame.Categories.UpdateCat = CreateFrame("Button", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.Categories.UpdateCat, L["Update"], 1)
	GVAR.ConfigFrame.Categories.UpdateCat:SetWidth(200)
	GVAR.ConfigFrame.Categories.UpdateCat:SetHeight(24)
	GVAR.ConfigFrame.Categories.UpdateCat:SetPoint("TOP", GVAR.ConfigFrame.Categories.IconPullDown, "BOTTOM", 0, -20)
	GVAR.ConfigFrame.Categories.UpdateCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.Content, "LEFT", (mainWidth-200-10-200)/2, 0)
	GVAR.ConfigFrame.Categories.UpdateCat:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_UpdateCat(false) end)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.UpdateCat)
	-- - Categories update END

	-- - Categories delete START
	GVAR.ConfigFrame.Categories.DeleteCat = CreateFrame("Button", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.Categories.DeleteCat, L["Delete"], 1)
	GVAR.ConfigFrame.Categories.DeleteCat:SetWidth(200)
	GVAR.ConfigFrame.Categories.DeleteCat:SetHeight(24)
	GVAR.ConfigFrame.Categories.DeleteCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.UpdateCat, "RIGHT", 10, 0)
	GVAR.ConfigFrame.Categories.DeleteCat:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteCat(false) end)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.DeleteCat)
	-- - Categories delete END

	-- - Categories clearall START
	GVAR.ConfigFrame.Categories.ClearAllCat = CreateFrame("Button", nil, GVAR.ConfigFrame.Categories.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.Categories.ClearAllCat, L["Clear All"], 2)
	GVAR.ConfigFrame.Categories.ClearAllCat:SetWidth(120)
	GVAR.ConfigFrame.Categories.ClearAllCat:SetHeight(18)
	GVAR.ConfigFrame.Categories.ClearAllCat:SetPoint("TOP", GVAR.ConfigFrame.Categories.DeleteCat, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.Categories.ClearAllCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.Content, "LEFT", (mainWidth-120)/2, 0)
	GVAR.ConfigFrame.Categories.ClearAllCat:SetScript("OnClick", function() AHsearch:ConfigurationFrame_ClearAll() end)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.ClearAllCat)
	-- - Categories clearall END

	-- - Categories reposition START
	local nw = 90
	GVAR.ConfigFrame.Categories.NewCatTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.Categories.SelectCatTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.Categories.NameCatTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.Categories.OrderTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.Categories.ColorTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.Categories.IconTitle:SetWidth(nw-4)

	GVAR.ConfigFrame.Categories.NewCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NewCatTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.Categories.SelectCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.SelectCatTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetPoint("LEFT", GVAR.ConfigFrame.Categories.CurrentCatTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.Categories.NameCat:SetPoint("LEFT", GVAR.ConfigFrame.Categories.NameCatTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.Categories.Order:SetPoint("LEFT", GVAR.ConfigFrame.Categories.OrderTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.Categories.ColorPullDown:SetPoint("LEFT", GVAR.ConfigFrame.Categories.ColorTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.Categories.IconPullDown:SetPoint("LEFT", GVAR.ConfigFrame.Categories.IconTitle, "LEFT", nw, 0)
	-- - Categories reposition END
	-- Categories END ----------------------------------------



	-- SearchQueries START ----------------------------------------
	GVAR.ConfigFrame.SearchQueries = {}
	GVAR.ConfigFrame.SearchQueries.Content = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.SearchQueries.Content)
	GVAR.ConfigFrame.SearchQueries.Content:SetWidth(mainWidth)
	GVAR.ConfigFrame.SearchQueries.Content:SetHeight(mainHeight-topheight+sizeGeneral.Border)
	GVAR.ConfigFrame.SearchQueries.Content:SetPoint("TOPLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 0, 1)
	GVAR.ConfigFrame.SearchQueries.Content:EnableMouse(true)
	GVAR.ConfigFrame.SearchQueries.Content:Hide()

	-- - SearchQueries create new query START
	GVAR.ConfigFrame.SearchQueries.NewSQTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.Content, "TOPLEFT", 10, -10)
	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetText(L["Create New"]..":")
	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.NewSQ = CreateFrame("EditBox", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.SearchQueries.NewSQ)
	GVAR.ConfigFrame.SearchQueries.NewSQ:SetWidth(sizeGeneral.ConfigTextWidth)
	GVAR.ConfigFrame.SearchQueries.NewSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.NewSQTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.NewSQ:SetMaxLetters(maxBoxInputLength)
	GVAR.ConfigFrame.SearchQueries.NewSQ:SetScript("OnTextChanged", function(self)
		local text = self:GetText()
		if text ~= "" then
			AHsearch:ConfigurationFrame_SearchQueries_CreateNewSQ()
			GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Create New"])
			TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
		else
			if GVAR.ConfigFrame.TabSearchQueries.editType == "new" then
				AHsearch:ConfigurationFrame_SearchQueries_CreateNewSQ()
				GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Create New"])
				TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
			end
		end
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.NewSQ)

	GVAR.ConfigFrame.SearchQueries.NewSQNote = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.NewSQNote:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.NewSQNote:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.NewSQ, "RIGHT", 5, 0)
	GVAR.ConfigFrame.SearchQueries.NewSQNote:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.NewSQNote:SetText(L["*NEW"].." ("..L["Title"]..")")
	GVAR.ConfigFrame.SearchQueries.NewSQNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - SearchQueries create new query END

	-- - SearchQueries selectCat START
	GVAR.ConfigFrame.SearchQueries.SelectCatTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.SelectCatTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.SelectCatTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.SelectCatTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.NewSQ, "BOTTOM", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectCatTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.SelectCatTitle:SetText(L["Select Category"]..":")

	GVAR.ConfigFrame.SearchQueries.SelectCat = CreateFrame("Frame", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.SearchQueries.SelectCat, 1)
	GVAR.ConfigFrame.SearchQueries.SelectCat:SetWidth(sizeGeneral.ConfigCatWidth)
	GVAR.ConfigFrame.SearchQueries.SelectCat:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	GVAR.ConfigFrame.SearchQueries.SelectCat:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.SelectCatTitle, "BOTTOM", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectCat:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SelectCatTitle, "LEFT", 0, 0)

	GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.ConfigFrame.SearchQueries.SelectCat)
	TEMPLATE.ScrollFrame(GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame)
	GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame:SetPoint("TOPLEFT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(GVAR.ConfigFrame.SearchQueries.SelectCat, value) end)

	GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button = {}
	for i = 1, maxCat do
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i] = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.SelectCat)
		local button = GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i]
		button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		button.Texture = button:CreateTexture(nil, "ARTWORK")
		button.Texture:SetWidth(sizeGeneral.BarHeight)
		button.Texture:SetHeight(sizeGeneral.BarHeight)
		button.Texture:SetPoint("LEFT", 0, 0)

		button.Text = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.Text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
		button.Text:SetHeight(sizeGeneral.BarHeight)
		button.Text:SetPoint("LEFT", sizeGeneral.BarHeight+sizeGeneral.Space, 0)
		button.Text:SetJustifyH("LEFT")

		button.Selected = button:CreateTexture(nil, "ARTWORK")
		button.Selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Selected:SetTexture(1, 1, 1, 0.1)
		button.Selected:Hide()
		button.SelectedArrow = button:CreateTexture(nil, "ARTWORK")
		button.SelectedArrow:SetWidth(Textures.SelectArrow.width)
		button.SelectedArrow:SetHeight(Textures.SelectArrow.height)
		button.SelectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		button.SelectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		button.SelectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		button.SelectedArrow:Hide()

		button:SetScript("OnClick", function(self) AHsearch:CatFrame_ButtonEventUpdate(self.cat) end)

		button.Highlight = button:CreateTexture(nil, "ARTWORK")
		button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(button.Highlight)
	end
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectCat)
	-- - SearchQueries selectCat END

	-- - SearchQueries selectSQ START
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SelectCat, "RIGHT", 30, 0)
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.SelectCatTitle, "TOP", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetText(L["Select Search Query"]..":")
	GVAR.ConfigFrame.SearchQueries.SelectSQTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.SelectSQ = CreateFrame("Frame", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.SearchQueries.SelectSQ, 1)
	GVAR.ConfigFrame.SearchQueries.SelectSQ:SetWidth(sizeGeneral.ConfigCatWidth)
	GVAR.ConfigFrame.SearchQueries.SelectSQ:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	GVAR.ConfigFrame.SearchQueries.SelectSQ:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.SelectSQTitle, "BOTTOM", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SelectSQTitle, "LEFT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectSQ:Hide()

	GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.ConfigFrame.SearchQueries.SelectSQ)
	TEMPLATE.ScrollFrame(GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame)
	GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame:SetPoint("TOPLEFT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(GVAR.ConfigFrame.SearchQueries.SelectSQ, value) end)

	GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button = {}
	for i = 1, maxQuery do
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i] = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.SelectSQ)
		local button = GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i]
		button:SetWidth(sizeGeneral.ConfigQueryButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		button.Text = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.Text:SetWidth(sizeGeneral.ConfigQueryTextSFWidth)
		button.Text:SetHeight(sizeGeneral.BarHeight)
		button.Text:SetPoint("LEFT", 0, 0)
		button.Text:SetJustifyH("LEFT")

		button.Selected = button:CreateTexture(nil, "ARTWORK")
		button.Selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Selected:SetTexture(1, 1, 1, 0.1)
		button.Selected:Hide()
		button.SelectedArrow = button:CreateTexture(nil, "ARTWORK")
		button.SelectedArrow:SetWidth(Textures.SelectArrow.width)
		button.SelectedArrow:SetHeight(Textures.SelectArrow.height)
		button.SelectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		button.SelectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		button.SelectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		button.SelectedArrow:Hide()

		button.Order = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.Order:SetWidth(30)
		button.Order:SetHeight(sizeGeneral.BarHeight)
		button.Order:SetPoint("RIGHT", button, "LEFT", -(sizeGeneral.Offset+sizeGeneral.Space), 0)
		button.Order:SetJustifyH("RIGHT")
		button.Order:SetTextColor(0, 0.75, 0) -- ORDER_COLOR

		button:SetScript("OnClick", function(self) AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq) end)

		button.Highlight = button:CreateTexture(nil, "ARTWORK")
		button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		button.Highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(button.Highlight)
	end
	-- - SearchQueries selectSQ END

	-- - SearchQueries move START
	GVAR.ConfigFrame.SearchQueries.MovePullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.SearchQueries.MovePullDown, "", sizeGeneral.ConfigCatWidth, GUI_CatFrame_data, 2, ConfigurationFrame_SearchQueries_SetMove)
	GVAR.ConfigFrame.SearchQueries.MovePullDown:SetWidth(sizeGeneral.ConfigCatWidth)
	GVAR.ConfigFrame.SearchQueries.MovePullDown:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.MovePullDown:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.SelectSQ, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.MovePullDown:SetPoint("RIGHT", GVAR.ConfigFrame.SearchQueries.SelectSQ, "RIGHT", 0, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.MovePullDown)

	GVAR.ConfigFrame.SearchQueries.MoveTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetPoint("RIGHT", GVAR.ConfigFrame.SearchQueries.MovePullDown, "LEFT", -10, 0)
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetJustifyH("RIGHT")
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetText(L["Move to"]..":")
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon = GVAR.ConfigFrame.SearchQueries.MovePullDown:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:SetWidth(20)
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.MovePullDown, "RIGHT", 5, -2)
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:Hide()
	-- - SearchQueries move END

	-- - SearchQueries title START
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.MovePullDown, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetText(L["Title"]..":")
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.TitleSQ = CreateFrame("EditBox", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.SearchQueries.TitleSQ)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetWidth(sizeGeneral.ConfigTextWidth)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.TitleSQTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetMaxLetters(maxBoxInputLength)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetScript("OnTextChanged", function(self)
		if GVAR.ConfigFrame.TabSearchQueries.editType == "new" then
			return
		end
		local text = self:GetText()
		ConfigurationFrame_SearchQueries_SetTitleSQ(text)
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.TitleSQ)

	GVAR.ConfigFrame.SearchQueries.TitleSQNote = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.TitleSQNote:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.TitleSQNote:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.TitleSQ, "RIGHT", 5, 0)
	GVAR.ConfigFrame.SearchQueries.TitleSQNote:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.TitleSQNote:SetText(L["*Required"])
	GVAR.ConfigFrame.SearchQueries.TitleSQNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - SearchQueries title END

	-- - SearchQueries query START
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.TitleSQ, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetText(L["Name"]..":")
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.NameQuerySQ = CreateFrame("EditBox", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.SearchQueries.NameQuerySQ)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetWidth(sizeGeneral.ConfigTextWidth)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetMaxLetters(maxBoxInputLength)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetScript("OnTextChanged", function(self)
		local text = self:GetText()
		GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value = text
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.NameQuerySQ)

	GVAR.ConfigFrame.SearchQueries.NameQuerySQNote = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.NameQuerySQNote:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQNote:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.NameQuerySQ, "RIGHT", 5, 0)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQNote:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.NameQuerySQNote:SetText(L["*Searchfield"])
	GVAR.ConfigFrame.SearchQueries.NameQuerySQNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - SearchQueries query END

	-- - SearchQueries order START
	GVAR.ConfigFrame.SearchQueries.OrderTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetText(L["Position"]..":")
	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.Order = CreateFrame("EditBox", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.SearchQueries.Order)
	GVAR.ConfigFrame.SearchQueries.Order:SetWidth(40)
	GVAR.ConfigFrame.SearchQueries.Order:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.OrderTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.Order:SetNumeric(true)
	GVAR.ConfigFrame.SearchQueries.Order:SetMaxLetters(2)
	GVAR.ConfigFrame.SearchQueries.Order:SetTextColor(0, 0.75, 0) -- ORDER_COLOR
	GVAR.ConfigFrame.SearchQueries.Order:SetScript("OnTextChanged", function(self)
		local number = GVAR.ConfigFrame.SearchQueries.Order:GetNumber()
		if number == 0 then
			self:SetNumber(1)
			number = 1
		end
		GVAR.ConfigFrame.SearchQueries.Order.value = number
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.Order)
	-- - SearchQueries order END

	-- - SearchQueries rarity START
	GVAR.ConfigFrame.SearchQueries.RarityTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.OrderTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetText(L["Rarity"]..":")
	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.RarityPullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.SearchQueries.RarityPullDown, "", sizeGeneral.ConfigPullDownRarityWidth, itemRarity, false, ConfigurationFrame_SearchQueries_SetRarity)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown:SetWidth(sizeGeneral.ConfigPullDownRarityWidth)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.RarityTitle, "RIGHT", 10, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.RarityPullDown)

	GVAR.ConfigFrame.SearchQueries.RarityInfo = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:EnableMouse(true)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:SetWidth(16)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:SetHeight(16)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:SetPoint("RIGHT", GVAR.ConfigFrame.SearchQueries.RarityPullDown, "LEFT", -5, 0)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTexture = GVAR.ConfigFrame.SearchQueries.RarityInfo:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.SearchQueries.RarityInfoTexture:SetWidth(16)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTexture:SetHeight(16)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTexture:SetPoint("LEFT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTexture:SetTexture("Interface\\FriendsFrame\\InformationIcon")
	GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame = CreateFrame("Frame", nil, GVAR.ConfigFrame.SearchQueries.RarityInfo)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame, 1)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:SetToplevel(true)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:SetPoint("BOTTOM", GVAR.ConfigFrame.SearchQueries.RarityInfo, "TOP", 0, 0)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:Hide()
	GVAR.ConfigFrame.SearchQueries.RarityInfoText = GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.RarityInfoText:SetPoint("CENTER", 0, 0)
	GVAR.ConfigFrame.SearchQueries.RarityInfoText:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.RarityInfoText:SetText("|TInterface\\DialogFrame\\UI-Dialog-Icon-AlertNew:24|t "..L["Can change between Patches."])
	GVAR.ConfigFrame.SearchQueries.RarityInfoText:SetTextColor(1, 1, 0.49, 1)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:SetScript("OnEnter", function() GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:Show() end)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:SetScript("OnLeave", function() GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:Hide() end)
	local txtWidth = GVAR.ConfigFrame.SearchQueries.RarityInfoText:GetStringWidth()
	local txtHeight = GVAR.ConfigFrame.SearchQueries.RarityInfoText:GetStringHeight()
	GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:SetWidth(txtWidth+30)
	GVAR.ConfigFrame.SearchQueries.RarityInfoTextFrame:SetHeight(txtHeight+30)
	GVAR.ConfigFrame.SearchQueries.RarityInfoText:SetWidth(txtWidth+10)
	GVAR.ConfigFrame.SearchQueries.RarityInfoText:SetHeight(txtHeight+10)
	GVAR.ConfigFrame.SearchQueries.RarityInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, true) -- RARUPD
	-- - SearchQueries rarity END

	-- - SearchQueries filter START
	GVAR.ConfigFrame.SearchQueries.FilterTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.SearchQueries.RarityTitle, "BOTTOMLEFT", 0, -10)
	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetText(L["Filter"]..":")
	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.Filter1PullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter1PullDown, "", sizeGeneral.ConfigPullDownFilterWidth, Filter1Table, false, ConfigurationFrame_SearchQueries_SetFilter1)
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown:SetWidth(sizeGeneral.ConfigPullDownFilterWidth)
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.FilterTitle, "RIGHT", 10, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter2PullDown, "", sizeGeneral.ConfigPullDownFilterWidth, Filter2Table, false, ConfigurationFrame_SearchQueries_SetFilter2)
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown:SetWidth(sizeGeneral.ConfigPullDownFilterWidth)
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Filter1PullDown, "RIGHT", 2, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter3PullDown, "", sizeGeneral.ConfigPullDownFilterWidth, Filter3Table, false, ConfigurationFrame_SearchQueries_SetFilter3)
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown:SetWidth(sizeGeneral.ConfigPullDownFilterWidth)
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Filter2PullDown, "RIGHT", 2, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)

	GVAR.ConfigFrame.SearchQueries.FilterInfo = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:EnableMouse(true)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:SetWidth(16)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:SetHeight(16)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:SetPoint("RIGHT", GVAR.ConfigFrame.SearchQueries.Filter1PullDown, "LEFT", -5, 0)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTexture = GVAR.ConfigFrame.SearchQueries.FilterInfo:CreateTexture(nil, "ARTWORK")
	GVAR.ConfigFrame.SearchQueries.FilterInfoTexture:SetWidth(16)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTexture:SetHeight(16)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTexture:SetPoint("LEFT", 0, 0)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTexture:SetTexture("Interface\\FriendsFrame\\InformationIcon")
	GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame = CreateFrame("Frame", nil, GVAR.ConfigFrame.SearchQueries.FilterInfo)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame, 1)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:SetToplevel(true)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:SetPoint("BOTTOM", GVAR.ConfigFrame.SearchQueries.FilterInfo, "TOP", 0, 0)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:Hide()
	GVAR.ConfigFrame.SearchQueries.FilterInfoText = GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.FilterInfoText:SetPoint("CENTER", 0, 0)
	GVAR.ConfigFrame.SearchQueries.FilterInfoText:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.FilterInfoText:SetText("|TInterface\\DialogFrame\\UI-Dialog-Icon-AlertNew:24|t "..L["Can change between Patches."])
	GVAR.ConfigFrame.SearchQueries.FilterInfoText:SetTextColor(1, 1, 0.49, 1)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:SetScript("OnEnter", function() GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:Show() end)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:SetScript("OnLeave", function() GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:Hide() end)
	local txtWidth = GVAR.ConfigFrame.SearchQueries.FilterInfoText:GetStringWidth()
	local txtHeight = GVAR.ConfigFrame.SearchQueries.FilterInfoText:GetStringHeight()
	GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:SetWidth(txtWidth+30)
	GVAR.ConfigFrame.SearchQueries.FilterInfoTextFrame:SetHeight(txtHeight+30)
	GVAR.ConfigFrame.SearchQueries.FilterInfoText:SetWidth(txtWidth+10)
	GVAR.ConfigFrame.SearchQueries.FilterInfoText:SetHeight(txtHeight+10)
	GVAR.ConfigFrame.SearchQueries.FilterInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.FilterInfoTexture, true) -- FINUPD
	-- - SearchQueries filter END

	-- - SearchQueries level START
	GVAR.ConfigFrame.SearchQueries.LevelTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.Filter1PullDown, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetText(L["Level"]..":")
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.LevelMinTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.LevelTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetText(L["min."]..":")
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMin = CreateFrame("EditBox", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.SearchQueries.LevelMin)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetWidth(40)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.LevelMinTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetNumeric(true)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetMaxLetters(2)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetScript("OnTextChanged", function(self)
		local number = GVAR.ConfigFrame.SearchQueries.LevelMin:GetNumber()
		if number == 0 then
			self:SetNumber("")
			number = nil
		end
		GVAR.ConfigFrame.SearchQueries.LevelMin.value = number
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMin)

	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.LevelMin, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetText(L["max."]..":")
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMax = CreateFrame("EditBox", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.EditBox(GVAR.ConfigFrame.SearchQueries.LevelMax)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetWidth(40)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.LevelMaxTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetNumeric(true)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetMaxLetters(2)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetScript("OnTextChanged", function(self)
		local number = GVAR.ConfigFrame.SearchQueries.LevelMax:GetNumber()
		if number == 0 then
			self:SetNumber("")
			number = nil
		end
		GVAR.ConfigFrame.SearchQueries.LevelMax.value = number
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMax)
	-- - SearchQueries level END

	-- - SearchQueries sort START
	GVAR.ConfigFrame.SearchQueries.SortTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.LevelMin, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetText(L["Sort"]..":")
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.SortByPullDown = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.PullDownMenu(GVAR.ConfigFrame.SearchQueries.SortByPullDown, "", sizeGeneral.ConfigPullDownSortWidth, SortByTable, false, ConfigurationFrame_SearchQueries_SetCheck)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown:SetWidth(sizeGeneral.ConfigPullDownSortWidth)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SortTitle, "RIGHT", 10, 0)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.SortByPullDown)

	GVAR.ConfigFrame.SearchQueries.SortOrderAsc = CreateFrame("CheckButton", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.CheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderAsc, 16, 4, nil, 1)
	GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SortByPullDown, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderAsc)

	GVAR.ConfigFrame.SearchQueries.SortOrderDesc = CreateFrame("CheckButton", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.CheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderDesc, 16, 4, nil, 2)
	GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SortOrderAsc, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderDesc)

	GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetScript("OnClick", function()
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = true
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(true)
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = false
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(false)
		ConfigurationFrame_SearchQueries_SetCheck()
	end)

	GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetScript("OnClick", function()
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = true
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(true)
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = false
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(false)
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	-- - SearchQueries sort END

	-- - SearchQueries misc START
	GVAR.ConfigFrame.SearchQueries.MiscTitle = GVAR.ConfigFrame.SearchQueries.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetHeight(20)
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.SortByPullDown, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetText(L["Misc."]..":")
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetTextColor(0.5, 0.5, 0.5)

	GVAR.ConfigFrame.SearchQueries.Usable = CreateFrame("CheckButton", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.CheckButton(GVAR.ConfigFrame.SearchQueries.Usable, 16, 4, L["Usable Items"])
	GVAR.ConfigFrame.SearchQueries.Usable:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.MiscTitle, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.Usable:SetChecked(0)
	GVAR.ConfigFrame.SearchQueries.Usable:SetScript("OnClick", function(self)
		if self:GetChecked() then
			self.value = 1
			self:SetChecked(1)
		else
			self.value = 0
			self:SetChecked(0)
		end
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.Usable)

	GVAR.ConfigFrame.SearchQueries.DisplayOnChar = CreateFrame("CheckButton", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.CheckButton(GVAR.ConfigFrame.SearchQueries.DisplayOnChar, 16, 4, L["Display on Character"])
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Usable, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetChecked(0)
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetScript("OnClick", function(self)
		if self:GetChecked() then
			self.value = 1
			self:SetChecked(1)
		else
			self.value = 0
			self:SetChecked(0)
		end
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.DisplayOnChar)
	-- - SearchQueries misc END

	-- - SearchQueries save START
	GVAR.ConfigFrame.SearchQueries.SaveSQ = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, L["Update"], 1)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetWidth(200)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetHeight(24)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.Usable, "BOTTOM", 0, -20)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", (mainWidth-200-10-200)/2, 0)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ() end)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
	-- - SearchQueries save END

	-- - SearchQueries delete START
	GVAR.ConfigFrame.SearchQueries.DeleteSQ = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ,  L["Delete"], 1)
	GVAR.ConfigFrame.SearchQueries.DeleteSQ:SetWidth(200)
	GVAR.ConfigFrame.SearchQueries.DeleteSQ:SetHeight(24)
	GVAR.ConfigFrame.SearchQueries.DeleteSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SaveSQ, "RIGHT", 10, 0)
	GVAR.ConfigFrame.SearchQueries.DeleteSQ:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteSQ() end)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ)
	-- - SearchQueries delete END

	-- - SearchQueries clearall START
	GVAR.ConfigFrame.SearchQueries.ClearAllSQ = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ, L["Clear All"], 2)
	GVAR.ConfigFrame.SearchQueries.ClearAllSQ:SetWidth(120)
	GVAR.ConfigFrame.SearchQueries.ClearAllSQ:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.ClearAllSQ:SetPoint("TOP", GVAR.ConfigFrame.SearchQueries.DeleteSQ, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.SearchQueries.ClearAllSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.Content, "LEFT", (mainWidth-120-10-120)/2, 0)
	GVAR.ConfigFrame.SearchQueries.ClearAllSQ:SetScript("OnClick", function() AHsearch:ConfigurationFrame_ClearAll() end)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ)
	-- - SearchQueries clearall END

	-- testsearch
	GVAR.ConfigFrame.SearchQueries.TestSearch = CreateFrame("Button", nil, GVAR.ConfigFrame.SearchQueries.Content)
	TEMPLATE.TextButton(GVAR.ConfigFrame.SearchQueries.TestSearch, L["Test Search"], 3)
	GVAR.ConfigFrame.SearchQueries.TestSearch:SetWidth(120)
	GVAR.ConfigFrame.SearchQueries.TestSearch:SetHeight(18)
	GVAR.ConfigFrame.SearchQueries.TestSearch:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.ClearAllSQ, "RIGHT", 10, 0)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.TestSearch)
	GVAR.ConfigFrame.SearchQueries.TestSearch:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_TestSearch() end)
	GVAR.ConfigFrame.SearchQueries.TestSearch:SetScript("OnEnter", function() if not AHsearch_Options.hideMain then GVAR.CatFrame:Hide() end end)
	GVAR.ConfigFrame.SearchQueries.TestSearch:SetScript("OnLeave", function() if not AHsearch_Options.hideMain then GVAR.CatFrame:Show() end end)
	-- testsearch

	-- - SearchQueries reposition START
	local nw = 90
	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetWidth(nw-16-5)
	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetWidth(nw-16-5)
	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.SearchQueries.SortTitle:SetWidth(nw-4)
	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetWidth(nw-4)

	GVAR.ConfigFrame.SearchQueries.NewSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.NewSQTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.TitleSQTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.Order:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.OrderTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.RarityTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.FilterTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.LevelTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.SortTitle, "LEFT", nw, 0)
	GVAR.ConfigFrame.SearchQueries.Usable:SetPoint("LEFT", GVAR.ConfigFrame.SearchQueries.MiscTitle, "LEFT", nw, 0)
	-- - SearchQueries reposition END
	-- SearchQueries END ----------------------------------------

	-- Options START ----------------------------------------
	GVAR.ConfigFrame.Options = {}
	GVAR.ConfigFrame.Options.Content = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.Options.Content)
	GVAR.ConfigFrame.Options.Content:SetWidth(mainWidth)
	GVAR.ConfigFrame.Options.Content:SetHeight(mainHeight-topheight+sizeGeneral.Border)
	GVAR.ConfigFrame.Options.Content:SetPoint("TOPLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 0, 1)
	GVAR.ConfigFrame.Options.Content:EnableMouse(true)
	GVAR.ConfigFrame.Options.Content:Hide()

	-- - button settings START
	GVAR.ConfigFrame.Options.CyclePrevNextTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.CyclePrevNextTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.CyclePrevNextTitle:SetPoint("TOPLEFT", GVAR.ConfigFrame.Options.Content, "TOPLEFT", 10, -10)
	GVAR.ConfigFrame.Options.CyclePrevNextTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.CyclePrevNextTitle:SetText(L["Button settings"]..":")

	GVAR.ConfigFrame.Options.CyclePrevNext = CreateFrame("CheckButton", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.CheckButton(GVAR.ConfigFrame.Options.CyclePrevNext, 16, 4, L["Show Button for Next and Previous Search"])
	GVAR.ConfigFrame.Options.CyclePrevNext:SetPoint("TOP", GVAR.ConfigFrame.Options.CyclePrevNextTitle, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.Options.CyclePrevNext:SetPoint("LEFT", GVAR.ConfigFrame.Options.CyclePrevNextTitle, "LEFT", 20, 0)
	GVAR.ConfigFrame.Options.CyclePrevNext:SetChecked(AHsearch_Options.cycleNP)
	GVAR.ConfigFrame.Options.CyclePrevNext:SetScript("OnClick", function(self)
		AHsearch_Options.cycleNP = not AHsearch_Options.cycleNP
		self:SetChecked(AHsearch_Options.cycleNP)
		AHsearch:CycleButtons_Toggle()
	end)

	GVAR.ConfigFrame.Options.CycleRandom = CreateFrame("CheckButton", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.CheckButton(GVAR.ConfigFrame.Options.CycleRandom, 16, 4, L["Show Button for Random Search"])
	GVAR.ConfigFrame.Options.CycleRandom:SetPoint("TOP", GVAR.ConfigFrame.Options.CyclePrevNext, "BOTTOM", 0, -10)
	GVAR.ConfigFrame.Options.CycleRandom:SetPoint("LEFT", GVAR.ConfigFrame.Options.CyclePrevNextTitle, "LEFT", 20, 0)
	GVAR.ConfigFrame.Options.CycleRandom:SetChecked(AHsearch_Options.cycleRND)
	GVAR.ConfigFrame.Options.CycleRandom:SetScript("OnClick", function(self)
		AHsearch_Options.cycleRND = not AHsearch_Options.cycleRND
		self:SetChecked(AHsearch_Options.cycleRND)
		AHsearch:CycleButtons_Toggle()
	end)
	-- - button settings END

	-- category frame settings START
	GVAR.ConfigFrame.Options.CatSizeTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.CatSizeTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.CatSizeTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.CycleRandom, "BOTTOM", 0, -20)
	GVAR.ConfigFrame.Options.CatSizeTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.CyclePrevNextTitle, "LEFT", 0, 0)
	GVAR.ConfigFrame.Options.CatSizeTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.CatSizeTitle:SetText(L["Category"]..":")

	GVAR.ConfigFrame.Options.CatWidthTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.CatWidthTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.CatWidthTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.CatSizeTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.CatWidthTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.CatSizeTitle, "LEFT", 10, 0)
	GVAR.ConfigFrame.Options.CatWidthTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.CatWidthTitle:SetText(L["Width"]..":")

	GVAR.ConfigFrame.Options.CatWidthSlider = CreateFrame("Slider", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.Slider(GVAR.ConfigFrame.Options.CatWidthSlider, sizeGeneral.ConfigSliderWidth, 5, 125, 250, AHsearch_Options.curCatWidth, AHsearch.SetupCatWidth)
	GVAR.ConfigFrame.Options.CatWidthSlider:SetPoint("TOP", GVAR.ConfigFrame.Options.CatWidthTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.CatWidthSlider:SetPoint("LEFT", GVAR.ConfigFrame.Options.CatWidthTitle, "LEFT", 0, 0)

	GVAR.ConfigFrame.Options.CatHeightTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.CatHeightTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.CatHeightTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.CatWidthTitle, "TOP", 0, 0)
	GVAR.ConfigFrame.Options.CatHeightTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.CatWidthSlider, "RIGHT", 10, 0)
	GVAR.ConfigFrame.Options.CatHeightTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.CatHeightTitle:SetText(L["Height (Rows)"]..":")

	GVAR.ConfigFrame.Options.CatHeightSlider = CreateFrame("Slider", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.Slider(GVAR.ConfigFrame.Options.CatHeightSlider, sizeGeneral.ConfigSliderWidth, 1, minCat, maxCat, AHsearch_Options.curCat, AHsearch.SetupCatHeight)
	GVAR.ConfigFrame.Options.CatHeightSlider:SetPoint("TOP", GVAR.ConfigFrame.Options.CatHeightTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.CatHeightSlider:SetPoint("LEFT", GVAR.ConfigFrame.Options.CatHeightTitle, "LEFT", 0, 0)
	-- category frame settings END

	-- search query frame settings START
	GVAR.ConfigFrame.Options.SQSizeTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.SQSizeTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.SQSizeTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.CatHeightSlider, "BOTTOM", 0, -30)
	GVAR.ConfigFrame.Options.SQSizeTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.CyclePrevNextTitle, "LEFT", 0, 0)
	GVAR.ConfigFrame.Options.SQSizeTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.SQSizeTitle:SetText(L["Search Query"]..":")

	GVAR.ConfigFrame.Options.SQWidthTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.SQWidthTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.SQWidthTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.SQSizeTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.SQWidthTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.SQSizeTitle, "LEFT", 10, 0)
	GVAR.ConfigFrame.Options.SQWidthTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.SQWidthTitle:SetText(L["Width"]..":")

	GVAR.ConfigFrame.Options.SQWidthSlider = CreateFrame("Slider", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.Slider(GVAR.ConfigFrame.Options.SQWidthSlider, sizeGeneral.ConfigSliderWidth, 5, 125, 250, AHsearch_Options.curQueryWidth, AHsearch.SetupSQWidth)
	GVAR.ConfigFrame.Options.SQWidthSlider:SetPoint("TOP", GVAR.ConfigFrame.Options.SQWidthTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.SQWidthSlider:SetPoint("LEFT", GVAR.ConfigFrame.Options.SQWidthTitle, "LEFT", 0, 0)

	GVAR.ConfigFrame.Options.SQHeightTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.SQHeightTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.SQHeightTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.SQWidthTitle, "TOP", 0, 0)
	GVAR.ConfigFrame.Options.SQHeightTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.SQWidthSlider, "RIGHT", 10, 0)
	GVAR.ConfigFrame.Options.SQHeightTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.SQHeightTitle:SetText(L["Height (Rows)"]..":")

	GVAR.ConfigFrame.Options.SQHeightSlider = CreateFrame("Slider", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.Slider(GVAR.ConfigFrame.Options.SQHeightSlider, sizeGeneral.ConfigSliderWidth, 1, minQuery, maxQuery, AHsearch_Options.curQuery, AHsearch.SetupSQHeight)
	GVAR.ConfigFrame.Options.SQHeightSlider:SetPoint("TOP", GVAR.ConfigFrame.Options.SQHeightTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.SQHeightSlider:SetPoint("LEFT", GVAR.ConfigFrame.Options.SQHeightTitle, "LEFT", 0, 0)
	-- search query frame settings END
	
	-- scale START
	GVAR.ConfigFrame.Options.ScaleTitle = GVAR.ConfigFrame.Options.Content:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.Options.ScaleTitle:SetHeight(20)
	GVAR.ConfigFrame.Options.ScaleTitle:SetPoint("TOP", GVAR.ConfigFrame.Options.SQHeightSlider, "BOTTOM", 0, -30)
	GVAR.ConfigFrame.Options.ScaleTitle:SetPoint("LEFT", GVAR.ConfigFrame.Options.CyclePrevNextTitle, "LEFT", 0, 0)
	GVAR.ConfigFrame.Options.ScaleTitle:SetJustifyH("LEFT")
	GVAR.ConfigFrame.Options.ScaleTitle:SetText(L["Scale"]..":")

	GVAR.ConfigFrame.Options.ScaleMain = CreateFrame("Slider", nil, GVAR.ConfigFrame.Options.Content)
	TEMPLATE.Slider(GVAR.ConfigFrame.Options.ScaleMain, sizeGeneral.ConfigSliderWidth, 1, 75, 150, AHsearch_Options.scaleMain, AHsearch.SetupMainScale, true)
	GVAR.ConfigFrame.Options.ScaleMain:SetPoint("TOP", GVAR.ConfigFrame.Options.ScaleTitle, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.Options.ScaleMain:SetPoint("LEFT", GVAR.ConfigFrame.Options.ScaleTitle, "LEFT", 10, 0)
	-- scale END
	-- Options END ----------------------------------------

	-- ConfirmPopup START ----------------------------------------
	GVAR.ConfigFrame.ConfirmFrame = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	GVAR.ConfigFrame.ConfirmFrame:SetWidth(mainWidth)
	GVAR.ConfigFrame.ConfirmFrame:SetHeight(mainHeight-topheight-(10+24+10))
	GVAR.ConfigFrame.ConfirmFrame:SetPoint("TOPLEFT", GVAR.ConfigFrame.Selection, "BOTTOMLEFT", 0, 1)
	GVAR.ConfigFrame.ConfirmFrame:EnableMouse(true)
	GVAR.ConfigFrame.ConfirmFrame:SetToplevel(true)
	GVAR.ConfigFrame.ConfirmFrame:Hide()

	GVAR.ConfigFrame.ConfirmTexture = GVAR.ConfigFrame.ConfirmFrame:CreateTexture(nil, "BACKGROUND")
	GVAR.ConfigFrame.ConfirmTexture:SetPoint("TOPLEFT", GVAR.ConfigFrame.ConfirmFrame, "TOPLEFT", 1, -1)
	GVAR.ConfigFrame.ConfirmTexture:SetPoint("BOTTOMRIGHT", GVAR.ConfigFrame.ConfirmFrame, "BOTTOMRIGHT", -1, 1)
	GVAR.ConfigFrame.ConfirmTexture:SetTexture(0, 0, 0, 0.8)

	GVAR.ConfigFrame.ConfirmText1 = GVAR.ConfigFrame.ConfirmFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	GVAR.ConfigFrame.ConfirmText1:SetWidth(mainWidth-50)
	GVAR.ConfigFrame.ConfirmText1:SetPoint("TOP", 0, -150)
	GVAR.ConfigFrame.ConfirmText1:SetJustifyH("CENTER")
	GVAR.ConfigFrame.ConfirmText2 = GVAR.ConfigFrame.ConfirmFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	GVAR.ConfigFrame.ConfirmText2:SetWidth(mainWidth-50)
	GVAR.ConfigFrame.ConfirmText2:SetPoint("TOP", GVAR.ConfigFrame.ConfirmText1, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.ConfirmText2:SetJustifyH("CENTER")
	GVAR.ConfigFrame.ConfirmText3 = GVAR.ConfigFrame.ConfirmFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	GVAR.ConfigFrame.ConfirmText3:SetWidth(mainWidth-50)
	GVAR.ConfigFrame.ConfirmText3:SetPoint("TOP", GVAR.ConfigFrame.ConfirmText2, "BOTTOM", 0, -5)
	GVAR.ConfigFrame.ConfirmText3:SetJustifyH("CENTER")

	GVAR.ConfigFrame.ConfirmOkay = CreateFrame("Button", nil, GVAR.ConfigFrame.ConfirmFrame)
	TEMPLATE.TextButton(GVAR.ConfigFrame.ConfirmOkay, L["YES"], 1)
	GVAR.ConfigFrame.ConfirmOkay:SetWidth(200)
	GVAR.ConfigFrame.ConfirmOkay:SetHeight(24)
	GVAR.ConfigFrame.ConfirmOkay:SetPoint("TOP", GVAR.ConfigFrame.ConfirmText3, "BOTTOM", 0, -30)
	GVAR.ConfigFrame.ConfirmOkay:SetPoint("LEFT", GVAR.ConfigFrame.ConfirmFrame, "LEFT", (mainWidth-200-10-200)/2, 0)

	GVAR.ConfigFrame.ConfirmCancel = CreateFrame("Button", nil, GVAR.ConfigFrame.ConfirmFrame)
	TEMPLATE.TextButton(GVAR.ConfigFrame.ConfirmCancel, L["NO"], 1)
	GVAR.ConfigFrame.ConfirmCancel:SetWidth(200)
	GVAR.ConfigFrame.ConfirmCancel:SetHeight(24)
	GVAR.ConfigFrame.ConfirmCancel:SetPoint("LEFT", GVAR.ConfigFrame.ConfirmOkay, "RIGHT", 10, 0)
	GVAR.ConfigFrame.ConfirmCancel:SetScript("OnClick", function() GVAR.ConfigFrame.ConfirmFrame:Hide() confirmIsShown = false end)
	-- ConfirmPopup END ----------------------------------------

	-- Mover Frame START ----------------------------------------
	GVAR.ConfigFrame.MoverTopFrame = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.MoverTopFrame, 1)
	GVAR.ConfigFrame.MoverTopFrame:SetWidth(mainWidth)
	GVAR.ConfigFrame.MoverTopFrame:SetHeight(20)
	GVAR.ConfigFrame.MoverTopFrame:SetPoint("BOTTOM", GVAR.ConfigFrame, "TOP", 0, -sizeGeneral.Border)
	GVAR.ConfigFrame.MoverTopFrame:EnableMouse(true)
	GVAR.ConfigFrame.MoverTopFrame:EnableMouseWheel(true)
	GVAR.ConfigFrame.MoverTopFrame:SetScript("OnMouseWheel", NOOP)
	GVAR.ConfigFrame.MoverTopTexture = GVAR.ConfigFrame.MoverTopFrame:CreateTexture(nil, "BACKGROUND")
	GVAR.ConfigFrame.MoverTopTexture:SetAllPoints(GVAR.ConfigFrame.MoverTopFrame)
	GVAR.ConfigFrame.MoverTopTexture:SetTexture(0, 0, 0, 1)
	GVAR.ConfigFrame.MoverTopTxt = GVAR.ConfigFrame.MoverTopFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.MoverTopTxt:SetPoint("CENTER", GVAR.ConfigFrame.MoverTopFrame, "CENTER", 0, 0)
	GVAR.ConfigFrame.MoverTopTxt:SetJustifyH("CENTER")
	GVAR.ConfigFrame.MoverTopTxt:SetTextColor(0.3, 0.3, 0.3, 1)
	GVAR.ConfigFrame.MoverTopTxt:SetText(L["click & move"])

	GVAR.ConfigFrame.Close = CreateFrame("Button", nil, GVAR.ConfigFrame.MoverTopFrame)
	TEMPLATE.IconButton(GVAR.ConfigFrame.Close, 1)
	GVAR.ConfigFrame.Close:SetWidth(20)
	GVAR.ConfigFrame.Close:SetHeight(20)
	GVAR.ConfigFrame.Close:SetPoint("RIGHT", GVAR.ConfigFrame.MoverTopFrame, "RIGHT", 0, 0)
	GVAR.ConfigFrame.Close:SetScript("OnClick", function() AHsearch:ConfigDisable() end)

	GVAR.ConfigFrame.MoverBottomFrame = CreateFrame("Frame", nil, GVAR.ConfigFrame)
	TEMPLATE.BorderTRBL(GVAR.ConfigFrame.MoverBottomFrame, 1)
	GVAR.ConfigFrame.MoverBottomFrame:SetWidth(mainWidth)
	GVAR.ConfigFrame.MoverBottomFrame:SetHeight(20)
	GVAR.ConfigFrame.MoverBottomFrame:SetPoint("TOP", GVAR.ConfigFrame, "BOTTOM", 0, sizeGeneral.Border)
	GVAR.ConfigFrame.MoverBottomFrame:EnableMouse(true)
	GVAR.ConfigFrame.MoverBottomFrame:EnableMouseWheel(true)
	GVAR.ConfigFrame.MoverBottomFrame:SetScript("OnMouseWheel", NOOP)
	GVAR.ConfigFrame.MoverBottomTexture = GVAR.ConfigFrame.MoverBottomFrame:CreateTexture(nil, "BACKGROUND")
	GVAR.ConfigFrame.MoverBottomTexture:SetAllPoints(GVAR.ConfigFrame.MoverBottomFrame)
	GVAR.ConfigFrame.MoverBottomTexture:SetTexture(0, 0, 0, 1)
	GVAR.ConfigFrame.MoverBottomTxt = GVAR.ConfigFrame.MoverBottomFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.ConfigFrame.MoverBottomTxt:SetPoint("CENTER", GVAR.ConfigFrame.MoverBottomFrame, "CENTER", 0, 0)
	GVAR.ConfigFrame.MoverBottomTxt:SetJustifyH("CENTER")
	GVAR.ConfigFrame.MoverBottomTxt:SetTextColor(0.3, 0.3, 0.3, 1)
	GVAR.ConfigFrame.MoverBottomTxt:SetText(L["click & move"])

	-- - move scripts START
	GVAR.ConfigFrame.MoverTopFrame:SetScript("OnEnter", function() GVAR.ConfigFrame.MoverTopTxt:SetTextColor(1, 1, 1, 1) end)
	GVAR.ConfigFrame.MoverTopFrame:SetScript("OnLeave", function() GVAR.ConfigFrame.MoverTopTxt:SetTextColor(0.3, 0.3, 0.3, 1) end)
	GVAR.ConfigFrame.MoverTopFrame:SetScript("OnMouseDown", function() GVAR.ConfigFrame:StartMoving() end)
	GVAR.ConfigFrame.MoverTopFrame:SetScript("OnMouseUp", function() GVAR.ConfigFrame:StopMovingOrSizing() AHsearch:SaveConfigFramePos() end)

	GVAR.ConfigFrame.MoverBottomFrame:SetScript("OnEnter", function() GVAR.ConfigFrame.MoverBottomTxt:SetTextColor(1, 1, 1, 1) end)
	GVAR.ConfigFrame.MoverBottomFrame:SetScript("OnLeave", function() GVAR.ConfigFrame.MoverBottomTxt:SetTextColor(0.3, 0.3, 0.3, 1) end)
	GVAR.ConfigFrame.MoverBottomFrame:SetScript("OnMouseDown", function() GVAR.ConfigFrame:StartMoving() end)
	GVAR.ConfigFrame.MoverBottomFrame:SetScript("OnMouseUp", function() GVAR.ConfigFrame:StopMovingOrSizing() AHsearch:SaveConfigFramePos() end)
	-- - move scripts END
	-- Mover Frame END ----------------------------------------
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_ShowContent()
	if GUI_Tab == 1 then
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabCategories, true)
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabSearchQueries, false)
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabOptions, false)
		GVAR.ConfigFrame.Categories.Content:Show()
		GVAR.ConfigFrame.SearchQueries.Content:Hide()
		GVAR.ConfigFrame.Selection.SearchQueryInfo:Hide()
		GVAR.ConfigFrame.Options.Content:Hide()
	elseif GUI_Tab == 2 then
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabCategories, false)
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabSearchQueries, true)
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabOptions, false)
		GVAR.ConfigFrame.Categories.Content:Hide()
		GVAR.ConfigFrame.SearchQueries.Content:Show()
		GVAR.ConfigFrame.Selection.SearchQueryInfo:Show()
		GVAR.ConfigFrame.Options.Content:Hide()
	elseif GUI_Tab == 3 then
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabCategories, false)
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabSearchQueries, false)
		TEMPLATE.SetTabButton(GVAR.ConfigFrame.TabOptions, true)
		GVAR.ConfigFrame.Categories.Content:Hide()
		GVAR.ConfigFrame.SearchQueries.Content:Hide()
		GVAR.ConfigFrame.Selection.SearchQueryInfo:Hide()
		GVAR.ConfigFrame.Options.Content:Show()
	end	
	if GUI_CatFrame_Count == 0 then
		TEMPLATE.DisableTabButton(GVAR.ConfigFrame.TabSearchQueries)
	else
		TEMPLATE.EnableTabButton(GVAR.ConfigFrame.TabSearchQueries)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- Categories Actions START  ----------------------------------------
function AHsearch:ConfigurationFrame_Categories_CreateNewCat() -- CREATENEW
	GVAR.ConfigFrame.TabCategories.editType = "new"
	-- values
	local icon = GVAR.ConfigFrame.Categories.IconPullDown.value
	if not icon then
		icon = 1
		GVAR.ConfigFrame.Categories.IconPullDown.value = icon
	end
	local color = GVAR.ConfigFrame.Categories.ColorPullDown.value
	if not color then
		color = "|cffffffff"
		GVAR.ConfigFrame.Categories.ColorPullDown.value = color
	end
	local order = GVAR.ConfigFrame.Categories.Order:GetNumber()
	GVAR.ConfigFrame.Categories.Order.value = order
	local cat = GVAR.ConfigFrame.Categories.NewCat:GetText()
	GVAR.ConfigFrame.Categories.CurrentCatTxt.value = cat

	GVAR.ConfigFrame.TabCategories.originalCat = nil

	-- setup display
	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexture(categoryIcons[icon].texture)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetText(color..cat.."|r")

	GVAR.ConfigFrame.Categories.NameCatTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.Categories.NameCat:SetText(cat)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.Categories.NameCat)

	GVAR.ConfigFrame.Categories.OrderTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.Order:SetNumber(order)
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.Categories.Order)

	GVAR.ConfigFrame.Categories.ColorTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.ColorPullDown.PullDownMenuText:SetText(color.."### "..categoryIntColors[color].."|r")
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.Categories.ColorPullDown)

	GVAR.ConfigFrame.Categories.IconTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.IconPullDown.PullDownMenuTexture:SetTexture(categoryIcons[icon].texture)
	GVAR.ConfigFrame.Categories.IconPullDown.PullDownMenuTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.Categories.IconPullDown)

	for i = 1, maxCat do
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	-- action buttons
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.Categories.ClearAllCat, 2)
	GVAR.ConfigFrame.Categories.UpdateCat:SetText(L["Create New"])
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.Categories.UpdateCat, 1)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.DeleteCat)
end

function AHsearch:ConfigurationFrame_Categories_SelectCat(cat)
	GVAR.ConfigFrame.TabCategories.editType = "selected"
	-- values
	local icon = AHsearch_Data[cat].icon or 1
	local color = AHsearch_Data[cat].color or "|cffffffff"
	local order = AHsearch_Data[cat].order or 1
	GVAR.ConfigFrame.Categories.Order.value = order
	GVAR.ConfigFrame.Categories.ColorPullDown.value = color
	GVAR.ConfigFrame.Categories.IconPullDown.value = icon
	GVAR.ConfigFrame.Categories.CurrentCatTxt.value = cat

	GVAR.ConfigFrame.TabCategories.originalCat = cat
	GVAR.ConfigFrame.TabCategories.originalOrder = order
	GVAR.ConfigFrame.TabCategories.originalColor = color
	GVAR.ConfigFrame.TabCategories.originalIcon = icon

	-- setup display
	GVAR.ConfigFrame.Categories.NewCat:ClearFocus()
	GVAR.ConfigFrame.Categories.NewCat:SetText("")

	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexture(categoryIcons[icon].texture)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetText(color..cat.."|r")

	GVAR.ConfigFrame.Categories.NameCatTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.NameCat:SetText(cat)
	GVAR.ConfigFrame.Categories.NameCat:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.Categories.NameCat)

	GVAR.ConfigFrame.Categories.OrderTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.Order:SetNumber(order)
	GVAR.ConfigFrame.Categories.Order:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.Categories.Order)

	GVAR.ConfigFrame.Categories.ColorTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.ColorPullDown.PullDownMenuText:SetText(color.."### "..categoryIntColors[color].."|r")
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.Categories.ColorPullDown)

	GVAR.ConfigFrame.Categories.IconTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.Categories.IconPullDown.PullDownMenuTexture:SetTexture(categoryIcons[icon].texture)
	GVAR.ConfigFrame.Categories.IconPullDown.PullDownMenuTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.Categories.IconPullDown)

	-- action buttons
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.Categories.ClearAllCat, 2)
	GVAR.ConfigFrame.Categories.UpdateCat:SetText(L["Update"])
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.UpdateCat)
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.Categories.DeleteCat, 1)
end

function AHsearch:ConfigurationFrame_Categories_ClearAllCat()
	GVAR.ConfigFrame.TabCategories.editType = nil
	-- values
	GVAR.ConfigFrame.Categories.Order.value = nil
	GVAR.ConfigFrame.Categories.ColorPullDown.value = nil
	GVAR.ConfigFrame.Categories.IconPullDown.value = nil
	GVAR.ConfigFrame.Categories.CurrentCatTxt.value = nil

	-- setup display
	GVAR.ConfigFrame.Categories.NewCat:SetText("")
	GVAR.ConfigFrame.Categories.NewCat:ClearFocus()

	GVAR.ConfigFrame.Categories.CurrentCatTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.Categories.CurrentCatTexture:SetTexture(nil)
	GVAR.ConfigFrame.Categories.CurrentCatTxt:SetText("")

	GVAR.ConfigFrame.Categories.NameCatTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.Categories.NameCat:SetText("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.Categories.NameCat)

	GVAR.ConfigFrame.Categories.OrderTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.Categories.Order:SetNumber(1)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.Categories.Order)

	GVAR.ConfigFrame.Categories.ColorTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.Categories.ColorPullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.Categories.ColorPullDown)

	GVAR.ConfigFrame.Categories.IconTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.Categories.IconPullDown.PullDownMenuTexture:SetTexture(nil)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.Categories.IconPullDown)

	for i = 1, maxCat do
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	-- action buttons
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.ClearAllCat)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.UpdateCat)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.Categories.DeleteCat)
end

function AHsearch:ConfigurationFrame_Categories_UpdateCat(confirm) --SAVEDATA
	local originalCat = GVAR.ConfigFrame.TabCategories.originalCat
	local currentCat = GVAR.ConfigFrame.Categories.CurrentCatTxt.value

	if originalCat then
		-- update existing category START
		if AHsearch_Data[currentCat] and originalCat ~= currentCat then
			-- - category exists START
			if confirm then
				-- - category exists confirm -> action START
				local oldqueries = AHsearch_Data[originalCat].queries
				
				AHsearch_Data[originalCat] = nil
				AHsearch_Data[currentCat] = nil
				AHsearch_Data[currentCat] = {}
				AHsearch_Data[currentCat].order = GVAR.ConfigFrame.Categories.Order.value
				AHsearch_Data[currentCat].icon = GVAR.ConfigFrame.Categories.IconPullDown.value
				AHsearch_Data[currentCat].color = GVAR.ConfigFrame.Categories.ColorPullDown.value

				if oldqueries then
					AHsearch_Data[currentCat].queries = {}
					AHsearch_Data[currentCat].queries = oldqueries
				end

				GUI_curCat = currentCat --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(GUI_curCat)
				AHsearch:FillList(GVAR.ConfigFrame.Categories.SelectCat)
				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectCat)
				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.MovePullDown)
				AHsearch:ConfigurationFrame_Categories_SelectCat(currentCat)
				AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
				GVAR.ConfigFrame.ConfirmFrame:Hide()
				confirmIsShown = false
				-- - category exists confirm -> action END
			else
				-- - category exists confirm popup START
				GVAR.ConfigFrame.Categories.NewCat:ClearFocus()
				GVAR.ConfigFrame.Categories.NameCat:ClearFocus()
				GVAR.ConfigFrame.Categories.Order:ClearFocus()
				GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Category '%s' exists!"], "|cffffffff"..currentCat.."|r") )
				GVAR.ConfigFrame.ConfirmText2:SetText( L["Overwrite?"] )
				GVAR.ConfigFrame.ConfirmText3:SetText( L["All search queries in the old category will be deleted!"] )
				GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_UpdateCat(true) end)
				GVAR.ConfigFrame.ConfirmFrame:Show()
				confirmIsShown = true
				-- - category exists confirm popup END
			end
			-- - category exists END
		else
			-- - category does not exists -> make new with old query data (if exists) -> simple rename START
			local oldqueries = AHsearch_Data[originalCat].queries

			AHsearch_Data[originalCat] = nil
			AHsearch_Data[currentCat] = nil
			AHsearch_Data[currentCat] = {}
			AHsearch_Data[currentCat].order = GVAR.ConfigFrame.Categories.Order.value
			AHsearch_Data[currentCat].icon = GVAR.ConfigFrame.Categories.IconPullDown.value
			AHsearch_Data[currentCat].color = GVAR.ConfigFrame.Categories.ColorPullDown.value

			if oldqueries then
				AHsearch_Data[currentCat].queries = {}
				AHsearch_Data[currentCat].queries = oldqueries
			end

			GUI_curCat = currentCat --LLL
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:QueryFrame_Show(GUI_curCat)
			AHsearch:FillList(GVAR.ConfigFrame.Categories.SelectCat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectCat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.MovePullDown)
			AHsearch:ConfigurationFrame_Categories_SelectCat(currentCat)
			AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
			-- - category does not exists -> make new with old query data (if exists) -> simple rename END
		end
		-- update existing category END
	else
		-- create new category START
		if AHsearch_Data[currentCat] then
			-- - category exists START
			if confirm then
				-- - category exists confirm -> action START
				AHsearch_Data[currentCat] = nil
				AHsearch_Data[currentCat] = {}
				AHsearch_Data[currentCat].order = GVAR.ConfigFrame.Categories.Order.value
				AHsearch_Data[currentCat].icon = GVAR.ConfigFrame.Categories.IconPullDown.value
				AHsearch_Data[currentCat].color = GVAR.ConfigFrame.Categories.ColorPullDown.value

				GUI_curCat = currentCat --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(GUI_curCat)
				AHsearch:FillList(GVAR.ConfigFrame.Categories.SelectCat)
				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectCat)
				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.MovePullDown)
				AHsearch:ConfigurationFrame_Categories_SelectCat(GUI_curCat)
				AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
				GVAR.ConfigFrame.ConfirmFrame:Hide()
				confirmIsShown = false
				-- - category exists confirm -> action END
			else
				-- - category exists confirm popup START
				GVAR.ConfigFrame.Categories.NewCat:ClearFocus()
				GVAR.ConfigFrame.Categories.NameCat:ClearFocus()
				GVAR.ConfigFrame.Categories.Order:ClearFocus()
				GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Category '%s' exists!"], "|cffffffff"..currentCat.."|r") )
				GVAR.ConfigFrame.ConfirmText2:SetText( L["Overwrite?"] )
				GVAR.ConfigFrame.ConfirmText3:SetText( L["All search queries in the old category will be deleted!"] )
				GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_UpdateCat(true) end)
				GVAR.ConfigFrame.ConfirmFrame:Show()
				confirmIsShown = true
				-- - category exists confirm popup END
			end
			-- - category exists END
		else
			-- - category does not exists -> make new START
			AHsearch_Data[currentCat] = {}
			AHsearch_Data[currentCat].order = GVAR.ConfigFrame.Categories.Order.value
			AHsearch_Data[currentCat].icon = GVAR.ConfigFrame.Categories.IconPullDown.value
			AHsearch_Data[currentCat].color = GVAR.ConfigFrame.Categories.ColorPullDown.value

			GUI_curCat = currentCat --LLL
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:QueryFrame_Show(GUI_curCat)
			AHsearch:FillList(GVAR.ConfigFrame.Categories.SelectCat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectCat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.MovePullDown)
			AHsearch:ConfigurationFrame_Categories_SelectCat(GUI_curCat)
			AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
			-- - category does not exists -> make new END
		end
		-- create new category END
	end
	AHsearch:ConfigurationFrame_ShowContent() -- update tabs
end

function AHsearch:ConfigurationFrame_Categories_DeleteCat(confirm)
	local originalCat = GVAR.ConfigFrame.TabCategories.originalCat
	if confirm then
		if originalCat and originalCat ~= "" then
			AHsearch_Data[originalCat] = nil
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:FillList(GVAR.ConfigFrame.Categories.SelectCat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectCat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.MovePullDown)
			AHsearch:QueryFrame_Show(nil)
			AHsearch:ConfigurationFrame_ClearAll()
			GVAR.ConfigFrame.ConfirmFrame:Hide()
			confirmIsShown = false
			local x = 0
			for cat in pairs(AHsearch_Data) do
				x = 1
				break
			end
			if x == 0 then
				GUI_Tab = 1
			end
			AHsearch:ConfigurationFrame_ShowContent() -- update tabs
		end
	else
		GVAR.ConfigFrame.Categories.NewCat:ClearFocus()
		GVAR.ConfigFrame.Categories.NameCat:ClearFocus()
		GVAR.ConfigFrame.Categories.Order:ClearFocus()
		GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Delete category '%s'?"], "|cffffffff"..originalCat.."|r") )
		GVAR.ConfigFrame.ConfirmText2:SetText( L["All search queries in that category will be deleted!"] )
		GVAR.ConfigFrame.ConfirmText3:SetText( "" )
		GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteCat(true) end)
		GVAR.ConfigFrame.ConfirmFrame:Show()
		confirmIsShown = true
	end
end
-- Categories Actions END  ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_SearchQueries_TestSearch()
	if not GVAR.ConfigFrame.SearchQueries.SortByPullDown.value then return end -- brainbug paranoia

	BrowseName:SetText(GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value)
	UIDropDownMenu_SetSelectedValue(BrowseDropDown, GVAR.ConfigFrame.SearchQueries.RarityPullDown.value)

	-- filter1
	AuctionFrameBrowse.selectedClassIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
	AuctionFrameBrowse.selectedClass = CLASS_FILTERS[GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value]
	-- filter2
	AuctionFrameBrowse.selectedSubclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
	local subx
	if curFilter1 and curFilter2 then
		for i = 1, #Filter2Table do
			if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
				subx = "|cffffffff"..Filter2Table[i].name.."|r"
				break
			end
		end
	end
	AuctionFrameBrowse.selectedSubclass = subx
	-- filter3
	AuctionFrameBrowse.selectedInvtypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value
	-- min/max
	BrowseMinLevel:SetNumber( GVAR.ConfigFrame.SearchQueries.LevelMin.value or "" )
	BrowseMaxLevel:SetNumber( GVAR.ConfigFrame.SearchQueries.LevelMax.value or "" )
	-- sort
	local sortOrder
	if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
		sortOrder = 1
	end
	AuctionFrame_SetSort("list", GVAR.ConfigFrame.SearchQueries.SortByPullDown.value, sortOrder)
	-- misc
	IsUsableCheckButton:SetChecked(GVAR.ConfigFrame.SearchQueries.Usable.value)
	ShowOnPlayerCheckButton:SetChecked(GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value)

	-- update AuctionFrame
	UIDropDownMenu_Initialize(BrowseDropDown, BrowseDropDown_Initialize)
	AuctionFrameFilters_Update()
	UIDropDownMenu_Refresh(BrowseDropDown, false)
	-- search
	AuctionFrameBrowse_Search()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:UpdateFilter()
	table.wipe(Filter1Table)
	table.wipe(Filter2Table)
	table.wipe(Filter3Table)

	local x = 1
	Filter1Table[x] = {}
	Filter1Table[x].index = -1
	Filter1Table[x].name = "-"
	x = x + 1

	for i = 1, #FilterTable do
		Filter1Table[x] = {}
		Filter1Table[x].index = FilterTable[i].itemClassIndex
		Filter1Table[x].name = FilterTable[i].itemClass
		x = x + 1
	end

	if curFilter1 then
		local x = 1
		for i = 1, #FilterTable do
			if curFilter1 == FilterTable[i].itemClassIndex then
				if FilterTable[i].subClasses then

					Filter2Table[x] = {}
					Filter2Table[x].index = -1
					Filter2Table[x].name = "-"
					x = x + 1

					for j = 1, #FilterTable[i].subClasses do
						Filter2Table[x] = {}
						Filter2Table[x].index = FilterTable[i].subClasses[j].subClassIndex
						Filter2Table[x].name = FilterTable[i].subClasses[j].subClass
						x = x + 1
					end
				end
			end
		end
	end

	if curFilter1 and curFilter2 then

		local x = 1
		for i = 1, #FilterTable do
			--print(i, "#", FilterTable[i].itemClassIndex, FilterTable[i].itemClass) -- TEST
			if FilterTable[i].subClasses then
				for j = 1, #FilterTable[i].subClasses do
					if curFilter1 == FilterTable[i].itemClassIndex then
						--print(i, j, "#", "-", FilterTable[i].subClasses[j].subClassIndex, FilterTable[i].subClasses[j].subClass) -- TEST
						if FilterTable[i].subClasses[j].subClassInvTypes then
							if curFilter2 == FilterTable[i].subClasses[j].subClassIndex then

								Filter3Table[x] = {}
								Filter3Table[x].index = -1
								Filter3Table[x].name = "-"
								x = x + 1

								for k = 1, #FilterTable[i].subClasses[j].subClassInvTypes do
									Filter3Table[x] = {}
									Filter3Table[x].index = FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex
									Filter3Table[x].name = FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp
									x = x + 1
									--print(i, j, k, "#", "--", FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp) -- TEST
								end
							end
						end
					end
				end
			end
		end

	end

	local locFilter1index, locFilter2index, locFilter3index
	local locFilter1name, locFilter2name, locFilter3name

	-- FILTER 1
	if curFilter1 then
		for i = 1, #Filter1Table do
			if Filter1Table[i].index == curFilter1 then
				locFilter1index = Filter1Table[i].index
				locFilter1name = Filter1Table[i].name
				break
			end
		end
		GVAR.ConfigFrame.SearchQueries.Filter1PullDown.PullDownMenuText:SetText(locFilter1name)
	else
		GVAR.ConfigFrame.SearchQueries.Filter1PullDown.PullDownMenuText:SetText("-")
	end
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)

	if locFilter1index and locFilter1index > 0 then
		GVAR.ConfigFrame.SearchQueries.FilterInfo:Enable() Desaturation(GVAR.ConfigFrame.SearchQueries.FilterInfoTexture, false) -- FINUPD
	else
		GVAR.ConfigFrame.SearchQueries.FilterInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.FilterInfoTexture, true) -- FINUPD
	end

	-- FILTER 2
	if curFilter1 and curFilter1 >= 0 and curFilter2 then
		for i = 1, #Filter2Table do
			if Filter2Table[i].index == curFilter2 then
				locFilter2index = Filter2Table[i].index
				locFilter2name = Filter2Table[i].name
				break
			end
		end
		GVAR.ConfigFrame.SearchQueries.Filter2PullDown.PullDownMenuText:SetText(locFilter2name)
	else
		GVAR.ConfigFrame.SearchQueries.Filter2PullDown.PullDownMenuText:SetText("-")
	end
	local isIn
	for i = 1, #Filter2Table do
		isIn = true
		break
	end
	if isIn then
		TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	else
		TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	end

	-- FILTER 3
	if curFilter1 and curFilter1 >= 0 and curFilter2 and curFilter2 >= 0 and curFilter3 then
		for i = 1, #Filter3Table do
			if Filter3Table[i].index == curFilter3 then
				locFilter3index = Filter3Table[i].index
				locFilter3name = Filter3Table[i].name
				break
			end
		end
		GVAR.ConfigFrame.SearchQueries.Filter3PullDown.PullDownMenuText:SetText(locFilter3name)
	else
		GVAR.ConfigFrame.SearchQueries.Filter3PullDown.PullDownMenuText:SetText("-")
	end
	local isIn
	for i = 1, #Filter3Table do
		isIn = true
		break
	end
	if isIn then
		TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)
	else
		TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- Search Queries Actions START  ----------------------------------------
function AHsearch:ConfigurationFrame_SearchQueries_CreateNewSQ() -- CREATENEW
	-- values
	--namequery
	local namequery
	if GVAR.ConfigFrame.TabSearchQueries.editType == "new" then
		namequery = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
	elseif GVAR.ConfigFrame.TabSearchQueries.editType == "link" then
		namequery = GVAR.ConfigFrame.SearchQueries.NewSQ:GetText()
	else
		namequery = BrowseName:GetText()
		GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value = namequery
	end
	--namequery

	--order
	local order
	if GVAR.ConfigFrame.TabSearchQueries.editType == "new" then
		order = GVAR.ConfigFrame.SearchQueries.Order:GetNumber()
	else
		order = 1
	end
	GVAR.ConfigFrame.SearchQueries.Order.value = order
	--order

	--rarity
	local qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
	if not qualityIndex then
		qualityIndex = UIDropDownMenu_GetSelectedValue(BrowseDropDown)
		GVAR.ConfigFrame.SearchQueries.RarityPullDown.value = qualityIndex
	end
	--rarity

	--filter1
	local classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
	if not classIndex then
		classIndex = AuctionFrameBrowse.selectedClassIndex
		GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = classIndex
	end
	curFilter1 = classIndex
	--filter1

	--filter2
	local subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
	if not subclassIndex then
		subclassIndex = AuctionFrameBrowse.selectedSubclassIndex
		GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = subclassIndex
	end
	curFilter2 = subclassIndex
	--local subclass = AuctionFrameBrowse.selectedSubclass -- setup in save action
	--filter2

	--filter3
	local invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value
	if not invTypeIndex then
		invTypeIndex = AuctionFrameBrowse.selectedInvtypeIndex
		GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = invTypeIndex
	end
	curFilter3 = invTypeIndex
	--filter3

	--level
	local minLevel
	local maxLevel
	if GVAR.ConfigFrame.TabSearchQueries.editType == "new" then
		minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
		maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value
	elseif GVAR.ConfigFrame.TabSearchQueries.editType == "link" then
		minLevel = nil
		maxLevel = nil
	else
		minLevel = BrowseMinLevel:GetNumber()
		if minLevel == 0 then
			minLevel = nil
		end
		GVAR.ConfigFrame.SearchQueries.LevelMin.value = minLevel
		maxLevel = BrowseMaxLevel:GetNumber()
		if maxLevel == 0 then
			maxLevel = nil
		end
		GVAR.ConfigFrame.SearchQueries.LevelMax.value = maxLevel
	end
	--level

	--sort by
	local primaryColumnPRE, reversedPRE = GetAuctionSort("list", 1)
	local primaryColumn = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
	if not primaryColumn then
		primaryColumn = primaryColumnPRE
		GVAR.ConfigFrame.SearchQueries.SortByPullDown.value = primaryColumn
	end
	--sort by

	--sort order
	local reversed
	local sortOrder1 = GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value
	local sortOrder2 = GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value
	if not sortOrder1 and not sortOrder2 then
		reversed = reversedPRE
		if reversed then
			GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = true
			GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = false
		else
			GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = false
			GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = true
		end
	else
		if sortOrder1 then
			reversed = 1
		end
	end
	--sort order

	--misc
	local isUsable = GVAR.ConfigFrame.SearchQueries.Usable.value
	if isUsable == nil then
		isUsable = IsUsableCheckButton:GetChecked()
	end
	GVAR.ConfigFrame.SearchQueries.Usable.value = isUsable
	local showOnPlayer = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
	if showOnPlayer == nil then
		showOnPlayer = ShowOnPlayerCheckButton:GetChecked()
	end
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value = showOnPlayer
	--misc

	local title = GVAR.ConfigFrame.SearchQueries.NewSQ:GetText()
	GVAR.ConfigFrame.TabSearchQueries.newSQtitle      = title
	GVAR.ConfigFrame.TabSearchQueries.originalSQtitle = nil
	GVAR.ConfigFrame.TabSearchQueries.currentSQtitle  = nil

	GVAR.ConfigFrame.TabSearchQueries.editType = "new"
	GVAR.ConfigFrame.SearchQueries.MovePullDown.value = nil

	-- setup display
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:Hide()
	GVAR.ConfigFrame.SearchQueries.MovePullDown.PullDownMenuText:SetText("")
	GVAR.ConfigFrame.SearchQueries.MovePullDown.PullDownMenuTexture:SetTexture(nil)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.MovePullDown)
	
	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetText(title)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.TitleSQ)

	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetText(namequery)
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.NameQuerySQ)

	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.Order:SetNumber(order)
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.Order)

	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.PullDownMenuText:SetText(itemRarity[qualityIndex+2].color..itemRarity[qualityIndex+2].name.."|r")
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.RarityPullDown)
	if qualityIndex > 0 then
		GVAR.ConfigFrame.SearchQueries.RarityInfo:Enable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, false) -- RARUPD
	else
		GVAR.ConfigFrame.SearchQueries.RarityInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, true) -- RARUPD
	end

	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetTextColor(1, 0.82, 0)
	AHsearch:UpdateFilter()
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)

	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetNumber(minLevel or "")
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMin)
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetNumber(maxLevel or "")
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMax)

	GVAR.ConfigFrame.SearchQueries.SortTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.PullDownMenuText:SetText(L[ primaryColumn ])
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.SortByPullDown)
	if reversed then
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(true)
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(false)
	else
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(false)
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(true)
	end
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderAsc)
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderDesc)

	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetTextColor(1, 0.82, 0)
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.Usable)
	GVAR.ConfigFrame.SearchQueries.Usable:SetChecked(isUsable)
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.DisplayOnChar)
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetChecked(showOnPlayer)

	for i = 1, maxQuery do
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	-- action buttons
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ, 2)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Create New"])
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ)
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.TestSearch, 3)
end

function AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)
	AHsearch:ConfigurationFrame_SearchQueries_ClearAfterSQDelete()
	GVAR.ConfigFrame.SearchQueries.SelectCat.cat = cat
	GVAR.ConfigFrame.SearchQueries.SelectCat.sq = nil

	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetTextColor(1, 0.82, 0)
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.NewSQ)

	-- action buttons
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ, 2)
	GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Create New"])
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.TestSearch)
end

function AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, sq)
	GVAR.ConfigFrame.TabSearchQueries.editType = "selected"

	GVAR.ConfigFrame.SearchQueries.SelectCat.cat = cat
	GVAR.ConfigFrame.SearchQueries.SelectCat.sq = sq

	GVAR.ConfigFrame.TabSearchQueries.newSQtitle = nil
	GVAR.ConfigFrame.TabSearchQueries.originalSQtitle = sq
	GVAR.ConfigFrame.TabSearchQueries.currentSQtitle = sq

	-- values
	local tab = AHsearch_Data[cat].queries[sq]

	--namequery
	local namequery = tab.name
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value = namequery
	GVAR.ConfigFrame.TabSearchQueries.originalNameQuerySQ = namequery
	--namequery

	--order
	local order = tab.order
	GVAR.ConfigFrame.SearchQueries.Order.value = order
	GVAR.ConfigFrame.TabSearchQueries.originalOrder = order
	--order

	--rarity
	local qualityIndex = tab.qualityIndex
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.value = qualityIndex
	GVAR.ConfigFrame.TabSearchQueries.originalRarity = qualityIndex
	--rarity

	--filter1
	local classIndex = tab.classIndex
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = classIndex
	GVAR.ConfigFrame.TabSearchQueries.originalFilter1 = classIndex
	curFilter1 = classIndex
	--filter1

	--filter2
	local subclassIndex = tab.subclassIndex
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = subclassIndex
	GVAR.ConfigFrame.TabSearchQueries.originalFilter2 = subclassIndex
	curFilter2 = subclassIndex
	--filter2

	--filter3
	local invTypeIndex = tab.invTypeIndex
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = invTypeIndex
	GVAR.ConfigFrame.TabSearchQueries.originalFilter3 = invTypeIndex
	curFilter3 = invTypeIndex
	--filter3

	--level
	local minLevel = tab.minLevel
	local maxLevel = tab.maxLevel
	GVAR.ConfigFrame.SearchQueries.LevelMin.value = minLevel
	GVAR.ConfigFrame.SearchQueries.LevelMax.value = maxLevel
	GVAR.ConfigFrame.TabSearchQueries.originalLevelMin = minLevel
	GVAR.ConfigFrame.TabSearchQueries.originalLevelMax = maxLevel
	--level

	--sort by
	local primaryColumn = tab.SortBy
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.value = primaryColumn
	GVAR.ConfigFrame.TabSearchQueries.originalSortBy = primaryColumn
	--sort by

	--sort order
	local reversed = tab.SortOrder
	if reversed then
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = true
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = false
		GVAR.ConfigFrame.TabSearchQueries.originalSortOrderAsc = true
		GVAR.ConfigFrame.TabSearchQueries.originalSortOrderDesc = false
	else
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = false
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = true
		GVAR.ConfigFrame.TabSearchQueries.originalSortOrderAsc = false
		GVAR.ConfigFrame.TabSearchQueries.originalSortOrderDesc = true
	end
	--sort order

	--misc
	local isUsable = tab.isUsable
	if not isUsable then isUsable = 0 end
	GVAR.ConfigFrame.SearchQueries.Usable.value = isUsable
	GVAR.ConfigFrame.TabSearchQueries.originalUsable = isUsable
	local showOnPlayer = tab.showOnPlayer
	if not showOnPlayer then showOnPlayer = 0 end
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value = showOnPlayer
	GVAR.ConfigFrame.TabSearchQueries.originalDisplayOnChar = showOnPlayer
	--misc

	-- setup display
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetTextColor(1, 0.82, 0)
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.MovePullDown)

	GVAR.ConfigFrame.SearchQueries.NewSQ:SetText("")
	GVAR.ConfigFrame.SearchQueries.NewSQ:ClearFocus()

	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetText(sq)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.TitleSQ)

	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetText(namequery)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.NameQuerySQ)

	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.Order:SetNumber(order)
	GVAR.ConfigFrame.SearchQueries.Order:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.Order)

	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.PullDownMenuText:SetText(itemRarity[qualityIndex+2].color..itemRarity[qualityIndex+2].name.."|r")
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.RarityPullDown)
	if qualityIndex > 0 then
		GVAR.ConfigFrame.SearchQueries.RarityInfo:Enable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, false) -- RARUPD
	else
		GVAR.ConfigFrame.SearchQueries.RarityInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, true) -- RARUPD
	end

	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetTextColor(1, 0.82, 0)
	AHsearch:UpdateFilter()
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)

	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetNumber(minLevel or "")
	GVAR.ConfigFrame.SearchQueries.LevelMin:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMin)
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetNumber(maxLevel or "")
	GVAR.ConfigFrame.SearchQueries.LevelMax:ClearFocus()
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMax)

	GVAR.ConfigFrame.SearchQueries.SortTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.PullDownMenuText:SetText(L[ primaryColumn ])
	TEMPLATE.EnablePullDownMenu(GVAR.ConfigFrame.SearchQueries.SortByPullDown)
	if reversed then
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(true)
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(false)
	else
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(false)
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(true)
	end
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderAsc)
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderDesc)

	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetTextColor(1, 0.82, 0)
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.Usable)
	GVAR.ConfigFrame.SearchQueries.Usable:SetChecked(isUsable)
	TEMPLATE.EnableCheckButton(GVAR.ConfigFrame.SearchQueries.DisplayOnChar)
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetChecked(showOnPlayer)

	-- action buttons
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ, 2)
	if GVAR.ConfigFrame.SearchQueries.MovePullDown.value then
		if GVAR.ConfigFrame.SearchQueries.MovePullDown.value == GVAR.ConfigFrame.SearchQueries.SelectCat.cat then
			GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"])
			TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
		else
			GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Move"])
			TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ, 1)
		end
	else
		GVAR.ConfigFrame.SearchQueries.SaveSQ:SetText(L["Update"])
		TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
	end
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ, 1)
	TEMPLATE.EnableTextButton(GVAR.ConfigFrame.SearchQueries.TestSearch, 3)
end

function AHsearch:ConfigurationFrame_SearchQueries_ClearAllSQ()
	GVAR.ConfigFrame.TabSearchQueries.editType = nil

	-- values
	GVAR.ConfigFrame.SearchQueries.SelectCat.cat = nil
	GVAR.ConfigFrame.SearchQueries.SelectCat.sq = nil

	GVAR.ConfigFrame.TabSearchQueries.newSQtitle = nil
	GVAR.ConfigFrame.TabSearchQueries.originalSQtitle = nil
	GVAR.ConfigFrame.TabSearchQueries.currentSQtitle = nil

	GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value = nil
	GVAR.ConfigFrame.SearchQueries.Order.value = nil
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.LevelMin.value = nil
	GVAR.ConfigFrame.SearchQueries.LevelMax.value = nil
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = nil
	GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = nil
	GVAR.ConfigFrame.SearchQueries.Usable.value = nil
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value = nil
	GVAR.ConfigFrame.SearchQueries.MovePullDown.value = nil

	-- setup display
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:Hide()
	GVAR.ConfigFrame.SearchQueries.MovePullDown.PullDownMenuText:SetText("")
	GVAR.ConfigFrame.SearchQueries.MovePullDown.PullDownMenuTexture:SetTexture(nil)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.MovePullDown)

	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.NewSQ:SetText("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.NewSQ)

	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetText("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.TitleSQ)

	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetText("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.NameQuerySQ)

	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.Order:SetNumber(1)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.Order)

	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.RarityPullDown)

	GVAR.ConfigFrame.SearchQueries.RarityInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, true) -- RARUPD

	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)

	GVAR.ConfigFrame.SearchQueries.FilterInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.FilterInfoTexture, true) -- FINUPD

	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetNumber("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMin)
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetNumber("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMax)

	GVAR.ConfigFrame.SearchQueries.SortTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.SortByPullDown)
	GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderAsc)
	GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderDesc)

	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.Usable:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.Usable)
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.DisplayOnChar)

	for i = 1, maxCat do
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	for i = 1, maxQuery do
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	GUI_curSQ = nil
	GUI_curCat = nil

	AHsearch:QueryFrame_Show(nil)

	-- action buttons
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.TestSearch)
end

function AHsearch:ConfigurationFrame_SearchQueries_ClearAfterSQDelete()
	GVAR.ConfigFrame.TabSearchQueries.editType = nil

	-- values
	--GVAR.ConfigFrame.SearchQueries.SelectCat.cat = nil
	GVAR.ConfigFrame.SearchQueries.SelectCat.sq = nil

	GVAR.ConfigFrame.TabSearchQueries.newSQtitle = nil
	GVAR.ConfigFrame.TabSearchQueries.originalSQtitle = nil
	GVAR.ConfigFrame.TabSearchQueries.currentSQtitle = nil

	GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value = nil
	GVAR.ConfigFrame.SearchQueries.Order.value = nil
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.LevelMin.value = nil
	GVAR.ConfigFrame.SearchQueries.LevelMax.value = nil
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.value = nil
	GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = nil
	GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = nil
	GVAR.ConfigFrame.SearchQueries.Usable.value = nil
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value = nil
	GVAR.ConfigFrame.SearchQueries.MovePullDown.value = nil

	-- setup display
	GVAR.ConfigFrame.SearchQueries.MoveTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.MoveAlertIcon:Hide()
	GVAR.ConfigFrame.SearchQueries.MovePullDown.PullDownMenuText:SetText("")
	GVAR.ConfigFrame.SearchQueries.MovePullDown.PullDownMenuTexture:SetTexture(nil)
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.MovePullDown)

	GVAR.ConfigFrame.SearchQueries.NewSQTitle:SetTextColor(1, 0.82, 0)
	GVAR.ConfigFrame.SearchQueries.NewSQ:SetText("")
	TEMPLATE.EnableEditBox(GVAR.ConfigFrame.SearchQueries.NewSQ)

	GVAR.ConfigFrame.SearchQueries.TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.TitleSQ:SetText("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.TitleSQ)

	GVAR.ConfigFrame.SearchQueries.NameQuerySQTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetText("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.NameQuerySQ)

	GVAR.ConfigFrame.SearchQueries.OrderTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.Order:SetNumber(1)
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.Order)

	GVAR.ConfigFrame.SearchQueries.RarityTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.RarityPullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.RarityPullDown)
	
	GVAR.ConfigFrame.SearchQueries.RarityInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.RarityInfoTexture, true) -- RARUPD

	GVAR.ConfigFrame.SearchQueries.FilterTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.Filter1PullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter1PullDown)
	GVAR.ConfigFrame.SearchQueries.Filter2PullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter2PullDown)
	GVAR.ConfigFrame.SearchQueries.Filter3PullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.Filter3PullDown)

	GVAR.ConfigFrame.SearchQueries.FilterInfo:Disable() Desaturation(GVAR.ConfigFrame.SearchQueries.FilterInfoTexture, true) -- FINUPD

	GVAR.ConfigFrame.SearchQueries.LevelTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMinTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMin:SetNumber("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMin)
	GVAR.ConfigFrame.SearchQueries.LevelMaxTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.LevelMax:SetNumber("")
	TEMPLATE.DisableEditBox(GVAR.ConfigFrame.SearchQueries.LevelMax)

	GVAR.ConfigFrame.SearchQueries.SortTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.SortByPullDown.PullDownMenuText:SetText("")
	TEMPLATE.DisablePullDownMenu(GVAR.ConfigFrame.SearchQueries.SortByPullDown)
	GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderAsc)
	GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(false)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.SortOrderDesc)

	GVAR.ConfigFrame.SearchQueries.MiscTitle:SetTextColor(0.5, 0.5, 0.5)
	GVAR.ConfigFrame.SearchQueries.Usable:SetChecked(0)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.Usable)
	GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetChecked(0)
	TEMPLATE.DisableCheckButton(GVAR.ConfigFrame.SearchQueries.DisplayOnChar)

	for i = 1, maxQuery do
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Hide()
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	GUI_curSQ = nil

	-- action buttons
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.ClearAllSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.SaveSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.DeleteSQ)
	TEMPLATE.DisableTextButton(GVAR.ConfigFrame.SearchQueries.TestSearch)
end

function AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(confirm) --SAVEDATA
	local cat = GVAR.ConfigFrame.SearchQueries.SelectCat.cat
	local sq  = GVAR.ConfigFrame.SearchQueries.SelectCat.sq
	local newSQ      = GVAR.ConfigFrame.TabSearchQueries.newSQtitle
	local originalSQ = GVAR.ConfigFrame.TabSearchQueries.originalSQtitle
	local currentSQ  = GVAR.ConfigFrame.TabSearchQueries.currentSQtitle
	local newCat     = GVAR.ConfigFrame.SearchQueries.MovePullDown.value

	if originalSQ then
		-- move START ----------
		if newCat and newCat ~= cat then
			if AHsearch_Data[newCat].queries and AHsearch_Data[newCat].queries[currentSQ] then
				if confirm then
					-- - searchquery exists in new cat confirm -> action START
					AHsearch_Data[cat].queries[originalSQ] = nil

					local x = 0
					for k in pairs(AHsearch_Data[cat].queries) do
						x = x + 1
					end
					if x == 0 then
						AHsearch_Data[cat].queries = nil
					end

					AHsearch_Data[newCat].queries[currentSQ] = {}
					AHsearch_Data[newCat].queries[currentSQ].name = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
					AHsearch_Data[newCat].queries[currentSQ].order = GVAR.ConfigFrame.SearchQueries.Order.value
					AHsearch_Data[newCat].queries[currentSQ].qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
					AHsearch_Data[newCat].queries[currentSQ].classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
					AHsearch_Data[newCat].queries[currentSQ].subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
					local subx
					if curFilter1 and curFilter2 then
						for i = 1, #Filter2Table do
							if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
								subx = "|cffffffff"..Filter2Table[i].name.."|r"
								break
							end
						end
					end
					AHsearch_Data[newCat].queries[currentSQ].subclass = subx
					AHsearch_Data[newCat].queries[currentSQ].invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value

					AHsearch_Data[newCat].queries[currentSQ].minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
					AHsearch_Data[newCat].queries[currentSQ].maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value

					AHsearch_Data[newCat].queries[currentSQ].SortBy = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
					local sortOrder
					if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
						sortOrder = 1
					end
					AHsearch_Data[newCat].queries[currentSQ].SortOrder = sortOrder

					local Usable = GVAR.ConfigFrame.SearchQueries.Usable.value
					if Usable == 0 or Usable == false then
						Usable = nil
					elseif Usable == true then
						Uasble = 1
					end
					AHsearch_Data[newCat].queries[currentSQ].isUsable = Usable

					local DisplayOnChar = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
					if DisplayOnChar == 0 or DisplayOnChar == false then
						DisplayOnChar = nil
					elseif DisplayOnChar == true then
						DisplayOnChar = 1
					end
					AHsearch_Data[newCat].queries[currentSQ].showOnPlayer = DisplayOnChar

					AHsearch:BuildStandardTable()
					AHsearch:CatFrame_Show(true)
					AHsearch:QueryFrame_Show(cat)

					AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
					AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

					GVAR.ConfigFrame.ConfirmFrame:Hide()
					confirmIsShown = false
					-- - searchquery exists in new cat confirm -> action END
				else
					-- - searchquery exists in new cat confirm popup START
					GVAR.ConfigFrame.SearchQueries.NewSQ:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.TitleSQ:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.NameQuerySQ:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.Order:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.LevelMin:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.LevelMax:ClearFocus()
					GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Search Query '%s' in category '%s' exists!"], "|cffffffff"..currentSQ.."|r", "|cffffffff"..newCat.."|r") )
					GVAR.ConfigFrame.ConfirmText2:SetText( L["Overwrite?"] )
					GVAR.ConfigFrame.ConfirmText3:SetText( "" )
					GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(true) end)
					GVAR.ConfigFrame.ConfirmFrame:Show()
					confirmIsShown = true
					-- - searchquery exists in new cat confirm popup END
				end
				-- - search query exists END
			else
				-- - search query does not exists -> make new -> simple move START
				if not AHsearch_Data[newCat].queries then AHsearch_Data[newCat].queries = {} end

				AHsearch_Data[cat].queries[originalSQ] = nil

				local x = 0
				for k in pairs(AHsearch_Data[cat].queries) do
					x = x + 1
				end
				if x == 0 then
					AHsearch_Data[cat].queries = nil
				end

				AHsearch_Data[newCat].queries[currentSQ] = {}
				AHsearch_Data[newCat].queries[currentSQ].name = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
				AHsearch_Data[newCat].queries[currentSQ].order = GVAR.ConfigFrame.SearchQueries.Order.value
				AHsearch_Data[newCat].queries[currentSQ].qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
				AHsearch_Data[newCat].queries[currentSQ].classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
				AHsearch_Data[newCat].queries[currentSQ].subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
				local subx
				if curFilter1 and curFilter2 then
					for i = 1, #Filter2Table do
						if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
							subx = "|cffffffff"..Filter2Table[i].name.."|r"
							break
						end
					end
				end
				AHsearch_Data[newCat].queries[currentSQ].subclass = subx
				AHsearch_Data[newCat].queries[currentSQ].invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value

				AHsearch_Data[newCat].queries[currentSQ].minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
				AHsearch_Data[newCat].queries[currentSQ].maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value

				AHsearch_Data[newCat].queries[currentSQ].SortBy = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
				local sortOrder
				if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
					sortOrder = 1
				end
				AHsearch_Data[newCat].queries[currentSQ].SortOrder = sortOrder

				local Usable = GVAR.ConfigFrame.SearchQueries.Usable.value
				if Usable == 0 or Usable == false then
					Usable = nil
				elseif Usable == true then
					Uasble = 1
				end
				AHsearch_Data[newCat].queries[currentSQ].isUsable = Usable

				local DisplayOnChar = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
				if DisplayOnChar == 0 or DisplayOnChar == false then
					DisplayOnChar = nil
				elseif DisplayOnChar == true then
					DisplayOnChar = 1
				end
				AHsearch_Data[newCat].queries[currentSQ].showOnPlayer = DisplayOnChar

				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(cat)

				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
				AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

				GVAR.ConfigFrame.ConfirmFrame:Hide()
				confirmIsShown = false
				-- - search query does not exists -> make new -> simple move END
			end
		-- move END ----------
		else
			-- update existing search query START
			if AHsearch_Data[cat].queries[currentSQ] and originalSQ ~= currentSQ then
				-- - search query exists START
				if confirm then
					-- - searchquery exists confirm -> action START
					AHsearch_Data[cat].queries[originalSQ] = nil

					AHsearch_Data[cat].queries[currentSQ] = {}
					AHsearch_Data[cat].queries[currentSQ].name = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
					AHsearch_Data[cat].queries[currentSQ].order = GVAR.ConfigFrame.SearchQueries.Order.value
					AHsearch_Data[cat].queries[currentSQ].qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
					AHsearch_Data[cat].queries[currentSQ].classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
					AHsearch_Data[cat].queries[currentSQ].subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
					local subx
					if curFilter1 and curFilter2 then
						for i = 1, #Filter2Table do
							if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
								subx = "|cffffffff"..Filter2Table[i].name.."|r"
								break
							end
						end
					end
					AHsearch_Data[cat].queries[currentSQ].subclass = subx
					AHsearch_Data[cat].queries[currentSQ].invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value

					AHsearch_Data[cat].queries[currentSQ].minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
					AHsearch_Data[cat].queries[currentSQ].maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value

					AHsearch_Data[cat].queries[currentSQ].SortBy = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
					local sortOrder
					if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
						sortOrder = 1
					end
					AHsearch_Data[cat].queries[currentSQ].SortOrder = sortOrder

					local Usable = GVAR.ConfigFrame.SearchQueries.Usable.value
					if Usable == 0 or Usable == false then
						Usable = nil
					elseif Usable == true then
						Uasble = 1
					end
					AHsearch_Data[cat].queries[currentSQ].isUsable = Usable

					local DisplayOnChar = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
					if DisplayOnChar == 0 or DisplayOnChar == false then
						DisplayOnChar = nil
					elseif DisplayOnChar == true then
						DisplayOnChar = 1
					end
					AHsearch_Data[cat].queries[currentSQ].showOnPlayer = DisplayOnChar

					GUI_curSQ = currentSQ --LLL
					AHsearch:BuildStandardTable()
					AHsearch:CatFrame_Show(true)
					AHsearch:QueryFrame_Show(cat)
					AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
					AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, currentSQ)

					GVAR.ConfigFrame.ConfirmFrame:Hide()
					confirmIsShown = false
					-- - searchquery exists confirm -> action END
				else
					-- - searchquery exists confirm popup START
					GVAR.ConfigFrame.SearchQueries.NewSQ:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.TitleSQ:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.NameQuerySQ:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.Order:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.LevelMin:ClearFocus()
					GVAR.ConfigFrame.SearchQueries.LevelMax:ClearFocus()
					GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Search Query '%s' in category '%s' exists!"], "|cffffffff"..currentSQ.."|r", "|cffffffff"..cat.."|r") )
					GVAR.ConfigFrame.ConfirmText2:SetText( L["Overwrite?"] )
					GVAR.ConfigFrame.ConfirmText3:SetText( "" )
					GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(true) end)
					GVAR.ConfigFrame.ConfirmFrame:Show()
					confirmIsShown = true
					-- - searchquery exists confirm popup END
				end
				-- - search query exists END
			else
				-- - search query does not exists -> make new -> simple rename START
				AHsearch_Data[cat].queries[originalSQ] = nil

				AHsearch_Data[cat].queries[currentSQ] = {}
				AHsearch_Data[cat].queries[currentSQ].name = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
				AHsearch_Data[cat].queries[currentSQ].order = GVAR.ConfigFrame.SearchQueries.Order.value
				AHsearch_Data[cat].queries[currentSQ].qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
				AHsearch_Data[cat].queries[currentSQ].classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
				AHsearch_Data[cat].queries[currentSQ].subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
				local subx
				if curFilter1 and curFilter2 then
					for i = 1, #Filter2Table do
						if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
							subx = "|cffffffff"..Filter2Table[i].name.."|r"
							break
						end
					end
				end
				AHsearch_Data[cat].queries[currentSQ].subclass = subx
				AHsearch_Data[cat].queries[currentSQ].invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value

				AHsearch_Data[cat].queries[currentSQ].minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
				AHsearch_Data[cat].queries[currentSQ].maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value

				AHsearch_Data[cat].queries[currentSQ].SortBy = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
				local sortOrder
				if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
					sortOrder = 1
				end
				AHsearch_Data[cat].queries[currentSQ].SortOrder = sortOrder

				local Usable = GVAR.ConfigFrame.SearchQueries.Usable.value
				if Usable == 0 or Usable == false then
					Usable = nil
				elseif Usable == true then
					Uasble = 1
				end
				AHsearch_Data[cat].queries[currentSQ].isUsable = Usable

				local DisplayOnChar = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
				if DisplayOnChar == 0 or DisplayOnChar == false then
					DisplayOnChar = nil
				elseif DisplayOnChar == true then
					DisplayOnChar = 1
				end
				AHsearch_Data[cat].queries[currentSQ].showOnPlayer = DisplayOnChar

				GUI_curSQ = currentSQ --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(cat)
				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
				AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, currentSQ)
				-- - search query does not exists -> make new -> simple rename END
			end
			-- update existing search query END
		end
	else
		-- create new searchquery START
		if AHsearch_Data[cat].queries and AHsearch_Data[cat].queries[newSQ] then
			-- - searchquery exists START
			if confirm then
				-- - searchquery exists confirm -> action START
				AHsearch_Data[cat].queries[newSQ] = nil

				AHsearch_Data[cat].queries[newSQ] = {}
				AHsearch_Data[cat].queries[newSQ].name = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
				AHsearch_Data[cat].queries[newSQ].order = GVAR.ConfigFrame.SearchQueries.Order.value
				AHsearch_Data[cat].queries[newSQ].qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
				AHsearch_Data[cat].queries[newSQ].classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
				AHsearch_Data[cat].queries[newSQ].subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
				local subx
				if curFilter1 and curFilter2 then
					for i = 1, #Filter2Table do
						if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
							subx = "|cffffffff"..Filter2Table[i].name.."|r"
							break
						end
					end
				end
				AHsearch_Data[cat].queries[newSQ].subclass = subx
				AHsearch_Data[cat].queries[newSQ].invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value

				AHsearch_Data[cat].queries[newSQ].minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
				AHsearch_Data[cat].queries[newSQ].maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value

				AHsearch_Data[cat].queries[newSQ].SortBy = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
				local sortOrder
				if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
					sortOrder = 1
				end
				AHsearch_Data[cat].queries[newSQ].SortOrder = sortOrder

				local Usable = GVAR.ConfigFrame.SearchQueries.Usable.value
				if Usable == 0 or Usable == false then
					Usable = nil
				elseif Usable == true then
					Uasble = 1
				end
				AHsearch_Data[cat].queries[newSQ].isUsable = Usable

				local DisplayOnChar = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
				if DisplayOnChar == 0 or DisplayOnChar == false then
					DisplayOnChar = nil
				elseif DisplayOnChar == true then
					DisplayOnChar = 1
				end
				AHsearch_Data[cat].queries[newSQ].showOnPlayer = DisplayOnChar

				GUI_curSQ = newSQ --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(cat)
				AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
				AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, newSQ)

				GVAR.ConfigFrame.ConfirmFrame:Hide()
				confirmIsShown = false
				-- - searchquery exists confirm -> action END
			else
				-- - searchquery exists confirm popup START
				GVAR.ConfigFrame.SearchQueries.NewSQ:ClearFocus()
				GVAR.ConfigFrame.SearchQueries.TitleSQ:ClearFocus()
				GVAR.ConfigFrame.SearchQueries.NameQuerySQ:ClearFocus()
				GVAR.ConfigFrame.SearchQueries.Order:ClearFocus()
				GVAR.ConfigFrame.SearchQueries.LevelMin:ClearFocus()
				GVAR.ConfigFrame.SearchQueries.LevelMax:ClearFocus()
				GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Search Query '%s' in category '%s' exists!"], "|cffffffff"..newSQ.."|r", "|cffffffff"..cat.."|r") )
				GVAR.ConfigFrame.ConfirmText2:SetText( L["Overwrite?"] )
				GVAR.ConfigFrame.ConfirmText3:SetText( "" )
				GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(true) end)
				GVAR.ConfigFrame.ConfirmFrame:Show()
				confirmIsShown = true
				-- - searchquery exists confirm popup END
			end
			-- - searchquery exists END
		else
			-- - searchquery does not exists -> make new START
			if not AHsearch_Data[cat].queries then AHsearch_Data[cat].queries = {} end

			AHsearch_Data[cat].queries[newSQ] = {}
			AHsearch_Data[cat].queries[newSQ].name = GVAR.ConfigFrame.SearchQueries.NameQuerySQ.value
			AHsearch_Data[cat].queries[newSQ].order = GVAR.ConfigFrame.SearchQueries.Order.value
			AHsearch_Data[cat].queries[newSQ].qualityIndex = GVAR.ConfigFrame.SearchQueries.RarityPullDown.value
			AHsearch_Data[cat].queries[newSQ].classIndex = GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value
			AHsearch_Data[cat].queries[newSQ].subclassIndex = GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value
			local subx
			if curFilter1 and curFilter2 then
				for i = 1, #Filter2Table do
					if Filter2Table[i].index == GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value then
						subx = "|cffffffff"..Filter2Table[i].name.."|r"
						break
					end
				end
			end
			AHsearch_Data[cat].queries[newSQ].subclass = subx
			AHsearch_Data[cat].queries[newSQ].invTypeIndex = GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value

			AHsearch_Data[cat].queries[newSQ].minLevel = GVAR.ConfigFrame.SearchQueries.LevelMin.value
			AHsearch_Data[cat].queries[newSQ].maxLevel = GVAR.ConfigFrame.SearchQueries.LevelMax.value

			AHsearch_Data[cat].queries[newSQ].SortBy = GVAR.ConfigFrame.SearchQueries.SortByPullDown.value
			local sortOrder
			if GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value then
				sortOrder = 1
			end
			AHsearch_Data[cat].queries[newSQ].SortOrder = sortOrder

			local Usable = GVAR.ConfigFrame.SearchQueries.Usable.value
			if Usable == 0 or Usable == false then
				Usable = nil
			elseif Usable == true then
				Uasble = 1
			end
			AHsearch_Data[cat].queries[newSQ].isUsable = Usable

			local DisplayOnChar = GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value
			if DisplayOnChar == 0 or DisplayOnChar == false then
				DisplayOnChar = nil
			elseif DisplayOnChar == true then
				DisplayOnChar = 1
			end
			AHsearch_Data[cat].queries[newSQ].showOnPlayer = DisplayOnChar

			GUI_curSQ = newSQ --LLL
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:QueryFrame_Show(cat)
			AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
			AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, newSQ)
			-- - searchquery does not exists -> make new END
		end
		-- create new searchquery END
	end
end

function AHsearch:ConfigurationFrame_Categories_DeleteSQ(confirm)
	local cat = GVAR.ConfigFrame.SearchQueries.SelectCat.cat
	local sq  = GVAR.ConfigFrame.SearchQueries.SelectCat.sq

	if confirm then

		AHsearch_Data[cat].queries[sq] = nil

		local x = 0
		for k in pairs(AHsearch_Data[cat].queries) do
			x = x + 1
		end
		if x == 0 then
			AHsearch_Data[cat].queries = nil
		end

		AHsearch:BuildStandardTable()
		AHsearch:CatFrame_Show(true)
		AHsearch:QueryFrame_Show(cat)
		AHsearch:FillList(GVAR.ConfigFrame.SearchQueries.SelectSQ)
		AHsearch:ConfigurationFrame_SearchQueries_ClearAfterSQDelete()
		GVAR.ConfigFrame.ConfirmFrame:Hide()
		confirmIsShown = false
	else
		GVAR.ConfigFrame.SearchQueries.NewSQ:ClearFocus()
		GVAR.ConfigFrame.SearchQueries.TitleSQ:ClearFocus()
		GVAR.ConfigFrame.SearchQueries.NameQuerySQ:ClearFocus()
		GVAR.ConfigFrame.SearchQueries.Order:ClearFocus()
		GVAR.ConfigFrame.SearchQueries.LevelMin:ClearFocus()
		GVAR.ConfigFrame.SearchQueries.LevelMax:ClearFocus()
		GVAR.ConfigFrame.ConfirmText1:SetText( string.format(L["Delete search query '%s' in category '%s'?"], "|cffffffff"..sq.."|r", "|cffffffff"..cat.."|r") )
		GVAR.ConfigFrame.ConfirmText2:SetText( "" )
		GVAR.ConfigFrame.ConfirmText3:SetText( "" )
		GVAR.ConfigFrame.ConfirmOkay:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteSQ(true) end)
		GVAR.ConfigFrame.ConfirmFrame:Show()
		confirmIsShown = true
	end
end
-- Search Queries Actions END  ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_ClearAll()
	AHsearch:ConfigurationFrame_Categories_ClearAllCat()
	AHsearch:ConfigurationFrame_SearchQueries_ClearAllSQ()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:SaveConfigFramePos()
	local point, relativeTo, relativePoint, x, y = GVAR.ConfigFrame:GetPoint()
	AHsearch_Options.ConfigFrame_posX = x
	AHsearch_Options.ConfigFrame_posY = y
	AHsearch_Options.ConfigFrame_relP = relativePoint
end

function AHsearch:ConfigFramePos()
	if AHsearch_Options.ConfigFrame_relP then
		GVAR.ConfigFrame:ClearAllPoints()
		GVAR.ConfigFrame:SetPoint(AHsearch_Options.ConfigFrame_relP, AHsearch_Options.ConfigFrame_posX, AHsearch_Options.ConfigFrame_posY)
	else
		GVAR.ConfigFrame:ClearAllPoints()
		GVAR.ConfigFrame:SetPoint("TOPLEFT", GVAR.MainFrame, "BOTTOMLEFT", AHsearch_Options.curCatWidth+AHsearch_Options.curQueryWidth-20-(2*sizeGeneral.Border), 47)-- -12) --14+(30-border)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Options Actions START  ----------------------------------------
function AHsearch:SetupCatWidth(value)
	AHsearch_Options.curCatWidth = value
	CatSizeUpdate(AHsearch_Options.curCatWidth)
	GVAR.CatFrame:SetWidth(AHsearch_Options.curCatWidth)
	AHsearch:FillList(GVAR.CatFrame)
end

function AHsearch:SetupCatHeight(value)
	AHsearch_Options.curCat = value
	local height = AHsearch_Options.curCat
	if AHsearch_Options.curCat > GUI_CatFrame_Count then
		height = GUI_CatFrame_Count
	end
	if GUI_CatFrame_Count == 0 then
		GVAR.CatFrame:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	else
		GVAR.CatFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	end
	AHsearch:FillList(GVAR.CatFrame)
	AHsearch:QueryFrame_Show(GUI_curCat) -- VODOO
end

function AHsearch:SetupSQWidth(value)
	AHsearch_Options.curQueryWidth = value
	QuerySizeUpdate(AHsearch_Options.curQueryWidth)
	GVAR.QueryFrame:SetWidth(AHsearch_Options.curQueryWidth)
	if not GUI_curCat then return end
	if not GUI_SQFrame_Count[GUI_curCat] then return end
	if GUI_CatFrame_Count > 0 then
		AHsearch:FillList(GVAR.QueryFrame)
	end
end

function AHsearch:SetupSQHeight(value)
	AHsearch_Options.curQuery = value
	if not GUI_curCat then return end
	if not GUI_SQFrame_Count[GUI_curCat] then return end
	if GUI_CatFrame_Count > 0 then
		local height = AHsearch_Options.curQuery
		if AHsearch_Options.curQuery > GUI_SQFrame_Count[GUI_curCat] then
			height = GUI_SQFrame_Count[GUI_curCat]
		end
		GVAR.QueryFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)
		AHsearch:FillList(GVAR.QueryFrame)
	end
end

function AHsearch:SetupMainScale(value)
	AHsearch_Options.scaleMain = value
	GVAR.CatFrame:SetScale(value/100)
	GVAR.CatFrame:ClearAllPoints()
	GVAR.CatFrame:SetPoint("TOPLEFT", GVAR.MainFrame, "BOTTOMLEFT", -(20/(value/100)), -(12/(value/100)))
end
-- Options Actions END  ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:Search(cat, sq)
	-- save data for cycle search
	cycleCat = cat
	cycleSQ = sq
	-- load fields in AuctionFrame
	local tab = AHsearch_Data[cat].queries[sq]

	BrowseName:SetText(tab.name)

	BrowseMinLevel:SetNumber(tab.minLevel or "")
	BrowseMaxLevel:SetNumber(tab.maxLevel or "")

	AuctionFrameBrowse.selectedClassIndex = tab.classIndex
	AuctionFrameBrowse.selectedClass = CLASS_FILTERS[tab.classIndex]
	AuctionFrameBrowse.selectedSubclassIndex = tab.subclassIndex
	AuctionFrameBrowse.selectedSubclass = tab.subclass
	AuctionFrameBrowse.selectedInvtypeIndex = tab.invTypeIndex

	AuctionFrame_SetSort("list", tab.SortBy, tab.SortOrder)

	IsUsableCheckButton:SetChecked(tab.isUsable)
	ShowOnPlayerCheckButton:SetChecked(tab.showOnPlayer)
	UIDropDownMenu_SetSelectedValue(BrowseDropDown, tab.qualityIndex)
	-- update AuctionFrame
	UIDropDownMenu_Initialize(BrowseDropDown, BrowseDropDown_Initialize)
	AuctionFrameFilters_Update()
	UIDropDownMenu_Refresh(BrowseDropDown, false)
	-- search
	AuctionFrameBrowse_Search()
	GVAR.CatFrame:Hide()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CycleSearch(dir)
	if not cycleChk then return end --> no AH GUI = no Action (Key Bindings)
	if not SearchQueryGlobalCheck() then return end --> no SQ = no Action

	if not cycleCat then
		local firstCat, firstSQ
		local lastCat, lastSQ
		for i = 1, GUI_CatFrame_Count do
			if GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] > 0 then
				for j = 1, GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] do
					if not firstCat then
						firstCat = GUI_CatFrame_data[i].Cat
						firstSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
					end
					lastCat = GUI_CatFrame_data[i].Cat
					lastSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
				end
			end
		end
		if dir == 1 then -- first cat
			cycleCat = firstCat
			cycleSQ  = firstSQ
		else             -- last cat
			cycleCat = lastCat
			cycleSQ  = lastSQ
		end
	else
		if not cycleCat and not cycleSQ then return end

		local nextCat, nextSQ
		local prevCat, prevSQ
		local num = 1
		local nextBingo, prevBingo
		for i = 1, GUI_CatFrame_Count do
			if GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] > 0 then
				for j = 1, GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] do
					if cycleCat == GUI_CatFrame_data[i].Cat and cycleSQ == GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query then
						nextBingo = num
						prevBingo = num
					end
					num = num + 1
				end
			end
		end

		if prevBingo == 1 then
			prevBingo = num
		end
		if nextBingo == num-1 then
			nextBingo = 0
		end

		num = 1
		for i = 1, GUI_CatFrame_Count do
			if GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] > 0 then
				for j = 1, GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] do
					if num == (prevBingo-1) then
						prevCat = GUI_CatFrame_data[i].Cat
						prevSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
					end
					if num == (nextBingo+1) then
						nextCat = GUI_CatFrame_data[i].Cat
						nextSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
					end
					num = num + 1
				end
			end
		end

		if dir == 1 then -- next cat
			cycleCat = nextCat
			cycleSQ  = nextSQ
		else             -- prev cat
			cycleCat = prevCat
			cycleSQ  = prevSQ
		end
	end
	if not cycleCat and not cycleSQ then return end
	AHsearch:Search(cycleCat, cycleSQ)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:RandomSearch()
	if not cycleChk then return end --> no AH GUI = no Action (Key Bindings)
	if GUI_CatFrame_Count == 0 then return end

	local cycleCat_temp
	cycleCat_temp = math.random(GUI_CatFrame_Count)
	cycleCat_temp = GUI_CatFrame_data[cycleCat_temp].Cat

	if AHsearch_Data[cycleCat_temp].queries then
		cycleCat = cycleCat_temp
		cycleSQ  = math.random(GUI_SQFrame_Count[cycleCat])
		cycleSQ  = GUI_SQFrame_data[cycleCat][cycleSQ].Query
		AHsearch:Search(cycleCat, cycleSQ)
	else
		if SearchQueryGlobalCheck() then
			AHsearch:RandomSearch()
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigEnable()
	configMode = true

	GVAR.MainFrame.AHsearchPrevButton:Disable()
	GVAR.MainFrame.AHsearchNextButton:Disable()
	GVAR.MainFrame.AHsearchRandomButton:Disable()

	GVAR.ConfigFrame.Selection.FrameBorder:SetTexture(unpack(configBorderColor))
	GVAR.ConfigFrame.Categories.Content.FrameBorder:SetTexture(unpack(configBorderColor))
	GVAR.ConfigFrame.SearchQueries.Content.FrameBorder:SetTexture(unpack(configBorderColor))
	GVAR.ConfigFrame.Options.Content.FrameBorder:SetTexture(unpack(configBorderColor))

	GVAR.QueryFrame.FrameBorder:SetTexture(unpack(configBorderColor))
	GVAR.CatFrame.FrameBorder:SetTexture(unpack(configBorderColor))

	AHsearch:ConfigurationFrame_ShowContent()
	GVAR.ConfigFrame:Show()

	if GUI_CatFrame_Count == 0 then
		GVAR.ConfigFrame.Categories.NewCat:SetFocus()
		GVAR.ConfigFrame.Categories.SelectCatTitle:SetTextColor(0.5, 0.5, 0.5)
		GVAR.ConfigFrame.Categories.SelectCat.FrameBorder:SetTexture(0.4, 0.4, 0.4, 1)
	else
		GVAR.ConfigFrame.Categories.SelectCatTitle:SetTextColor(1, 0.82, 0)
		GVAR.ConfigFrame.Categories.SelectCat.FrameBorder:SetTexture(0.8, 0.2, 0.2, 1)
	end

	for i = 1, maxCat do
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:SetTexture(1, 0, 0, 0.3)
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:SetTexture(1, 0, 0, 0.3)
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:SetTexture(1, 0, 0, 0.3)
		if GUI_curCat then
			if GVAR.CatFrame.ScrollFrame.Button[i].cat == GUI_curCat then
				GVAR.CatFrame.ScrollFrame.Button[i].Selected:Show()
				GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Show()
			end
			if GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].cat == GUI_curCat then
				GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:Show()
				GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Show()
			end
			if GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].cat == GUI_curCat then
				GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:Show()
				GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Show()
			end
		end
	end
	for i = 1, maxQuery do
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:SetTexture(1, 0, 0, 0.3)
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:SetTexture(1, 0, 0, 0.3)
		if GUI_curSQ then
			if GVAR.QueryFrame.ScrollFrame.Button[i].sq == GUI_curSQ then
				GVAR.QueryFrame.ScrollFrame.Button[i].Selected:Show()
				GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Show()
			end
			if GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].sq == GUI_curSQ then
				GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:Show()
				GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Show()
			end
		end
	end

	if GUI_curCat then
		AHsearch:CatFrame_ButtonEventUpdate(GUI_curCat)
		if GUI_curSQ then
			AHsearch:QueryFrame_ButtonEventUpdate(GUI_curCat, GUI_curSQ)
		end
	end
	AHsearch:CatFrame_Show(true)
end

function AHsearch:ConfigDisable(AHevent)
	configMode = false

	if not AHsearch_Data[cycleCat] or
	  (AHsearch_Data[cycleCat] and not AHsearch_Data[cycleCat].queries) or
	  (AHsearch_Data[cycleCat] and AHsearch_Data[cycleCat].queries and not AHsearch_Data[cycleCat].queries[cycleSQ])
	then
		cycleCat = false
	end

	GVAR.MainFrame.AHsearchPrevButton:Enable()
	GVAR.MainFrame.AHsearchNextButton:Enable()
	GVAR.MainFrame.AHsearchRandomButton:Enable()

	GVAR.ConfigFrame.Selection.FrameBorder:SetTexture(unpack(normalBorderColor))
	GVAR.ConfigFrame.Categories.Content.FrameBorder:SetTexture(unpack(normalBorderColor))
	GVAR.ConfigFrame.SearchQueries.Content.FrameBorder:SetTexture(unpack(normalBorderColor))
	GVAR.ConfigFrame.Options.Content.FrameBorder:SetTexture(unpack(normalBorderColor))

	GVAR.QueryFrame.FrameBorder:SetTexture(unpack(normalBorderColor))
	GVAR.CatFrame.FrameBorder:SetTexture(unpack(normalBorderColor))

	GVAR.ConfigFrame:Hide()
	GVAR.ConfigFrame.ConfirmFrame:Hide()
	confirmIsShown = false

	for i = 1, maxCat do
		GVAR.CatFrame.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0.1)
		GVAR.CatFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0.1)
		GVAR.ConfigFrame.Categories.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0.1)
		GVAR.ConfigFrame.SearchQueries.SelectCat.ScrollFrame.Button[i].SelectedArrow:Hide()
	end
	for i = 1, maxQuery do
		GVAR.QueryFrame.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0)
		GVAR.QueryFrame.ScrollFrame.Button[i].SelectedArrow:Hide()
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].Selected:SetTexture(1, 1, 1, 0.1)
		GVAR.ConfigFrame.SearchQueries.SelectSQ.ScrollFrame.Button[i].SelectedArrow:Hide()
	end

	if not AHevent then
		VisibilityCheck(2)
	end
end

function AHsearch:CloseAllFrames()
	configMode = false
	GVAR.ConfigFrame:Hide()
	GVAR.QueryFrame:Hide()
	GVAR.CatFrame:Hide()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:MakeAll()
	AHsearch:BuildStandardTable()
	AHsearch:BuildFilterTable()
	AHsearch:MainFrame_Create()
	AHsearch:CatFrame_CreateFrame()
	AHsearch:QueryFrame_CreateFrame()
	AHsearch:ConfigurationFrame_CreateFrame()
	AHsearch:InitFrames()
	hooksecurefunc("HandleModifiedItemClick", function(itemLink)
		--print("#", itemLink, type(itemLink))
		if not configMode then return end
		if not itemLink then return end
		if type(itemLink) ~= "string" then return end
		if not IsAltKeyDown() then return end
		if not GVAR.ConfigFrame.SearchQueries.Content:IsShown() then return end
		if not GVAR.ConfigFrame.SearchQueries.SelectCat.cat then return end -- no cat selected

		local newItemName, newItemRarity, newItemType, newItemSubType, newInvType
		local itemID = string.match(itemLink, "item:(%-?%d+):")
		itemID = tonumber(itemID)
		--print("*", itemID)
		if itemID then
			local itemName, _, itemRarity, _, _, itemType, itemSubType, _, invType = GetItemInfo(itemID)
			newItemName    = itemName
			newItemRarity  = itemRarity
			newItemType    = itemType
			newItemSubType = itemSubType
			newInvType     = invType
			--print(itemID, "#", newItemName, newItemRarity, newItemType, newItemSubType, newInvType)
		else
			local petID, _, petRarity = string.match(itemLink, "battlepet:(%-?%d+):(%-?%d+):(%-?%d+):")
			petID = tonumber(petID)
			petRarity = tonumber(petRarity)
			--print("*", petID, petRarity)
			if petID and petRarity then
				local name, _, petType = C_PetJournal.GetPetInfoBySpeciesID(petID)
				if name and petType then
					newItemName    = name
					newItemRarity  = petRarity
					newItemType    = FilterTable[11].itemClass -- TODO this needs a better solution.
					newItemSubType = _G["BATTLE_PET_NAME_"..petType]
					--print(petID, "#", newItemName, newItemRarity, newItemType, newItemSubType, newInvType)
				end
			end
		end
		
		if not newItemName then return end
		if not newItemRarity then return end

		GVAR.ConfigFrame.TabSearchQueries.editType = "link"
		GVAR.ConfigFrame.SearchQueries.NewSQ:SetText("") -- ok, that's not good. this force a OnTextChanged in case the same item is clicked again. this needs a better solution.
		GVAR.ConfigFrame.SearchQueries.NewSQ:SetText(newItemName)
		GVAR.ConfigFrame.SearchQueries.NameQuerySQ:SetText(newItemName)
		GVAR.ConfigFrame.SearchQueries.RarityPullDown.PullDownMenuText:SetText(itemRarity[newItemRarity + 2].color..itemRarity[newItemRarity + 2].name.."|r")
		GVAR.ConfigFrame.SearchQueries.RarityPullDown.value = newItemRarity

		GVAR.ConfigFrame.SearchQueries.LevelMin.value = nil
		GVAR.ConfigFrame.SearchQueries.LevelMin:SetNumber("")
		GVAR.ConfigFrame.SearchQueries.LevelMax.value = nil
		GVAR.ConfigFrame.SearchQueries.LevelMax:SetNumber("")

		GVAR.ConfigFrame.SearchQueries.SortByPullDown.PullDownMenuText:SetText(L["bid"])
		GVAR.ConfigFrame.SearchQueries.SortByPullDown.value = "bid"
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc:SetChecked(1)
		GVAR.ConfigFrame.SearchQueries.SortOrderAsc.value = true
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc:SetChecked(0)
		GVAR.ConfigFrame.SearchQueries.SortOrderDesc.value = false

		GVAR.ConfigFrame.SearchQueries.Usable:SetChecked(0)
		GVAR.ConfigFrame.SearchQueries.Usable.value = 0
		GVAR.ConfigFrame.SearchQueries.DisplayOnChar:SetChecked(0)
		GVAR.ConfigFrame.SearchQueries.DisplayOnChar.value = 0

		if newItemType and newItemType ~= "" then
			if newItemType == newItemSubType then
				-- 1. pulldown
				for i = 1, #FilterTable do
					if newItemType == FilterTable[i].itemClass then
						--print(i, FilterTable[i].itemClass)
						GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = i
						ConfigurationFrame_SearchQueries_SetFilter1(i)
						break
					end
				end
			elseif newItemType ~= newItemSubType and newInvType == "" then
				-- 1. + 2. pulldown
				for i = 1, #FilterTable do
					if newItemType == FilterTable[i].itemClass then
						--print(i, FilterTable[i].itemClass)
						GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = i
						ConfigurationFrame_SearchQueries_SetFilter1(i)
						break
					end
				end
				for i = 1, #FilterTable do
					if newItemType == FilterTable[i].itemClass then
						if FilterTable[i].subClasses then
							for j = 1, #FilterTable[i].subClasses do
								if newItemSubType == FilterTable[i].subClasses[j].subClass then
									--print(j, FilterTable[i].subClasses[j].subClass)
									GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = j
									ConfigurationFrame_SearchQueries_SetFilter2(j)
									break
								end
							end
						end
						break
					end
				end
			elseif newItemType ~= newItemSubType and newInvType ~= "" then
				-- 1. + 2. + 3. pulldown
				for i = 1, #FilterTable do
					if newItemType == FilterTable[i].itemClass then
						--print(i, FilterTable[i].itemClass)
						GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = i
						ConfigurationFrame_SearchQueries_SetFilter1(i)
						break
					end
				end
				for i = 1, #FilterTable do
					if newItemType == FilterTable[i].itemClass then
						if FilterTable[i].subClasses then
							for j = 1, #FilterTable[i].subClasses do
								if newItemSubType == FilterTable[i].subClasses[j].subClass then
									--print(j, FilterTable[i].subClasses[j].subClass)
									GVAR.ConfigFrame.SearchQueries.Filter2PullDown.value = j
									ConfigurationFrame_SearchQueries_SetFilter2(j)
									break
								end
							end
						end
						break
					end
				end
				for i = 1, #FilterTable do
					if newItemType == FilterTable[i].itemClass then
						if FilterTable[i].subClasses then
							for j = 1, #FilterTable[i].subClasses do
								if newItemSubType == FilterTable[i].subClasses[j].subClass then

									if FilterTable[i].subClasses[j].subClassInvTypes then -- filter items that has invType but the AH filtertable not

										for k = 1, #FilterTable[i].subClasses[j].subClassInvTypes do
											if newInvType == FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvRealTyp then
												--print(i, j, k, "#", "--", FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvRealTyp)
												GVAR.ConfigFrame.SearchQueries.Filter3PullDown.value = FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex
												ConfigurationFrame_SearchQueries_SetFilter3(FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex)
												break
											end
										end
										break

									end

								end
							end
						end
						break
					end
				end
			else
				GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = -1
				ConfigurationFrame_SearchQueries_SetFilter1(-1)
			end
		else
			GVAR.ConfigFrame.SearchQueries.Filter1PullDown.value = -1
			ConfigurationFrame_SearchQueries_SetFilter1(-1)
		end
		GVAR.ConfigFrame:Hide() -- TODO
		GVAR.ConfigFrame:Show() -- TODO
	end)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event)
	if event == "AUCTION_HOUSE_SHOW" then
		if not GVAR.MainFrame then
			AHsearch:MakeAll()
		end
		AHsearch:CycleButtons_Toggle()
		cycleChk = true
	elseif event == "AUCTION_HOUSE_CLOSED" then
		-- Operation CloseForce: Fix for evil AH addons that trigger AUCTION_HOUSE_CLOSED _before_ AUCTION_HOUSE_SHOW ... STOP that shit
		if not GVAR.MainFrame and AuctionFrameBrowse then
			AHsearch:MakeAll()
		end
		AHsearch:ConfigDisable(1)
		AHsearch:CloseAllFrames()
		cycleChk = false
		cycleCat = false
	elseif event == "PLAYER_LOGIN" then
		AHsearch:InitOptions()
		AHsearch:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

AHsearch:RegisterEvent("PLAYER_LOGIN")
AHsearch:RegisterEvent("AUCTION_HOUSE_SHOW")
AHsearch:RegisterEvent("AUCTION_HOUSE_CLOSED")
AHsearch:SetScript("OnEvent", OnEvent)